#pragma implementation	"queue.h"
#pragma implementation	"priorityq.h"

#include <string.h>	/* For SunOS */
#include <strings.h>	/* For SunOS */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#ifdef SunOS
#include <sys/filio.h>
#include <sys/select.h>
#endif

#include <netdb.h>

#include "cif.h"
#include "interrupt.h"
#include "device.h"
#include "clock.h"
#include "sab82526sim.h"
#include "sab82525sim.h"
#include "m2681sim.h"
#include "octetrx.h"
#include "sim68k/simulator/def.h"

#ifdef sun4_solaris2
   #include <sys/filio.h>	/* For Solaris */
#endif

#define SIMIOBASE	MEMSIZE

using namespace Oscl;

class CifOctetReceiver : public OctetReceiver {
	private:
		int		_fileDesc;
		bool	_openForBusiness;
	public:
		CifOctetReceiver():_openForBusiness(false){};
		void open(int fileDesc){
			_fileDesc			= fileDesc;
			_openForBusiness	= true;
			}
		void close(){
			_openForBusiness	= false;
			}
		void receiveBreak(){
			}
		void receive(Oscl::Octet data){
			if(_openForBusiness){
				write(_fileDesc, &data, 1);
				}
			}
	};

#define SUCCESS			0x0000	/* general failure */
#define FAILURE			0x1111	/* general failure */

#ifndef TRUE
#define TRUE	1
#define FALSE	0
#endif

#define HOSTNAME_LENGTH	128

DeviceQ	devices;

////////////////////////////////////////////////////////////////////////
// About Clock Cycles:
//	Cycles are a (rough) indication of the number of CPU clock
//	cycles required to perform an operation. The clock frequency
//	affects the meaning of a clock cycle. In this implementation
//	we assume a 16.78 MHz clock frequency.
//
//	The RTC needs to know the number of clock cycles between
//	interrupts. We want to generate interrupts every 10ms.
//	We have a clock frequency of 16MHz, or a clock cycle period
//	of 69.59ns.
//	Thus the :
//		1ms		= cycle_factor * 69.59ms
//		cycle_factor	= 1ms/69.39ns
//		cycle_factor	= 16,780 cycles/millisecond
//
//		10ms	= cycle_factor * 10
//		10ms	= 167,800 cycles
//

#define CYCLES_PER_MILLISECOND			16780

CifOctetReceiver	asyncPortARx;
CifOctetReceiver	asyncPortBRx;
Clock				rtClock(10*CYCLES_PER_MILLISECOND,64,1,SIMIOBASE+0x0000100);
Sab82526Sim			hscx(15*27,65,1,SIMIOBASE+0x0000200);
Sab82525Sim			hscxB(15*27,67,1,SIMIOBASE+0x0000400);
M2681Sim			serial(30*27,66,1,SIMIOBASE+0x0000300,asyncPortARx,asyncPortBRx);

static unsigned long	lastCycle;

static fd_set	readfds;
static int		debugPortListenSocket;
static int		serialPortAListenSocket;
static int		serialPortBListenSocket;
static int		debugPortSocket;
static int		serialPortASocket;
static int		serialPortBSocket;
static int		theInputFileDesc;
static bool		debugPortSocketOpen = false;
static bool		serialPortASocketOpen = false;
static bool		serialPortBSocketOpen = false;

static int	/* break */ processStreamInput();
static int /* boolean:break */ readUserBreak();
static void prepareServer(int serialPortATcpPort,int serialPortBTcpPort,int debugPortTcpPort);

void resetDevices(){
	Device *dev;
	lastCycle	= 0;
	for(dev=devices.first();dev;dev=devices.next(dev)){
		dev->reset();
		}
	}

int /* break */ processStdin(){
	static int	instCount;

	instCount++;

	instCount	%= 1000;

	if(instCount) return FALSE;
	return processStreamInput();
	}

static int	/* break */ processStreamInput(){
	int				result;
	int				userbreak	= FALSE;	/* boolean is true when user break encountered */
	struct timeval	timeout;
	fd_set			testfds	= readfds;

	timeout.tv_sec	= 0;
	timeout.tv_usec	= 1;

	result	= select(FD_SETSIZE,&testfds, (fd_set*)0,(fd_set*)0,&timeout);

	if(result<0){
		perror("select failed");
		exit(1);
		}

	if(!result) return FALSE;

	if(FD_ISSET(theInputFileDesc,&testfds)){
		userbreak	= readUserBreak();
		}
	if(FD_ISSET(debugPortListenSocket,&testfds)){
		unsigned				client_len;
		debugPortSocket	= accept(debugPortListenSocket,(struct sockaddr *)0,&client_len);
		FD_SET(debugPortSocket,&readfds);
		debugPortSocketOpen	= true;
//		asyncPortXRx.open(debugPortSocket);
		}
	if(FD_ISSET(serialPortAListenSocket,&testfds)){
		unsigned				client_len;
		serialPortASocket	= accept(serialPortAListenSocket,(struct sockaddr *)0,&client_len);
		FD_SET(serialPortASocket,&readfds);
		serialPortASocketOpen	= true;
		asyncPortARx.open(serialPortASocket);
		}
	if(FD_ISSET(serialPortBListenSocket,&testfds)){
		unsigned				client_len;
		serialPortBSocket	= accept(serialPortBListenSocket,(struct sockaddr *)0,&client_len);
		FD_SET(serialPortBSocket,&readfds);
		serialPortBSocketOpen	= true;
		asyncPortBRx.open(serialPortBSocket);
		}
	if(FD_ISSET(debugPortSocket,&testfds)){
		int	nread;
		ioctl(debugPortSocket,FIONREAD,&nread);
		if(nread == 0){
//			asyncPortXRx.close();
			close(debugPortSocket);
			debugPortSocketOpen	= false;
			FD_CLR(debugPortSocket,&readfds);
			}
		else{
			Oscl::Octet	data;
			read(debugPortSocket,&data,1);
//			serial.sendToReceiverA(data);	>>>>> append to command line editor
			}
		}
	if(FD_ISSET(serialPortASocket,&testfds)){
		int	nread;
		ioctl(serialPortASocket,FIONREAD,&nread);
		if(nread == 0){
			asyncPortARx.close();
			close(serialPortASocket);
			serialPortASocketOpen	= false;
			FD_CLR(serialPortASocket,&readfds);
			}
		else{
			Oscl::Octet	data;
			read(serialPortASocket,&data,1);
			serial.sendToReceiverA(data);
			}
		}
	if(FD_ISSET(serialPortBSocket,&testfds)){
		int	nread;
		ioctl(serialPortBSocket,FIONREAD,&nread);
		if(nread == 0){
			asyncPortBRx.close();
			close(serialPortBSocket);
			serialPortBSocketOpen	= false;
			FD_CLR(serialPortBSocket,&readfds);
			}
		else{
			Oscl::Octet	data;
			read(serialPortBSocket,&data,1);
			serial.sendToReceiverB(data);
			}
		}
	return userbreak;
	}

static int /* boolean:break */ readUserBreak(){
	char		ch;
	int			result;
	result	= read(theInputFileDesc, &ch, 1);

	if(result == 1){
		switch (ch){
			case 'b':
			case '\003':
				return TRUE;
			}
		return FALSE;
		}
	else if(result == 0){
		return FALSE;
		}
	else if((result == -1)&&(errno==EAGAIN)){
		return FALSE;
		}
	else return TRUE;

	return FALSE;
	}

void pollDevices(unsigned long cycles){
	Device*					dev;
	unsigned long		cyclesSinceLastPoll;

	if(cycles < lastCycle){
		// cycle count has rolled
		cyclesSinceLastPoll	= (unsigned long)(-1L);
		cyclesSinceLastPoll	-= lastCycle;
		cyclesSinceLastPoll	+= cycles;
		}
	else{
		cyclesSinceLastPoll	= cycles - lastCycle;
		}
	lastCycle	= cycles;

	for(dev=devices.first();dev;dev=devices.next(dev)){
		dev->poll(cyclesSinceLastPoll);
		}
	}

int /* IPL */ interruptPending(int* vector){
	Interrupt*	irq	= pendingInterrupts.first();
	if(irq){
		*vector	= irq->getVector();
		return irq->getInterruptLevel();
		}
	return(0);
	}

void initializeDevices(FILE* inputFileDesc,int baseIpPort){
	theInputFileDesc	= fileno(inputFileDesc);
	FD_ZERO(&readfds);
	FD_SET(theInputFileDesc,&readfds);
	prepareServer(baseIpPort+1,baseIpPort+2,baseIpPort+3);
	devices.put(&rtClock);
	devices.put(&hscx);
	devices.put(&hscxB);
	devices.put(&serial);
	}

void	closeDevices(){
	Device*	dev;
	for(dev=devices.first();dev;dev=devices.next(dev)){
		dev->close();
		}
	if(debugPortSocketOpen){
//		asyncPortXRx.close();
		close(debugPortSocket);
		}
	if(serialPortASocketOpen){
		asyncPortARx.close();
		close(serialPortASocket);
		}
	if(serialPortBSocketOpen){
		asyncPortBRx.close();
		close(serialPortBSocket);
		}
	close(debugPortListenSocket);
	close(serialPortAListenSocket);
	close(serialPortBListenSocket);
	}

int isIoAddress(unsigned long address){
	Device*	dev;
	for(dev=devices.first();dev;dev=devices.next(dev)){
		if(dev->hasAddress(address)){
			return TRUE;
			}
		}
	return FALSE;
	}

static Device*	findReadAddressedDevice(unsigned long address){
	Device*	dev;
	for(dev=devices.first();dev;dev=devices.next(dev)){
		if(dev->hasReadAddress(address)){
			return dev;
			}
		}
	return dev;
	}

static Device*	findWriteAddressedDevice(unsigned long address){
	Device*	dev;
	for(dev=devices.first();dev;dev=devices.next(dev)){
		if(dev->hasWriteAddress(address)){
			return dev;
			}
		}
	return dev;
	}

int readByte(unsigned long address,unsigned char* data){
	Device*	dev	= findReadAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasByteAddress(address)) return FAILURE;
	*data = dev->readByte(address);
	return SUCCESS;
	}

int readWord(unsigned long address,unsigned short* data){
	Device*	dev	= findReadAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasWordAddress(address)) return FAILURE;
	*data = dev->readWord(address);
	return SUCCESS;
	}

int readLong(unsigned long address,unsigned long* data){
	Device*	dev	= findReadAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasLongAddress(address)) return FAILURE;
	*data = dev->readLong(address);
	return SUCCESS;
	}

int writeByte(unsigned long address,unsigned char data){
	Device*	dev	= findWriteAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasByteAddress(address)) return FAILURE;
	dev->writeByte(address,data);
	return SUCCESS;
	}

int writeWord(unsigned long address,unsigned short data){
	Device*	dev	= findWriteAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasWordAddress(address)) return FAILURE;
	dev->writeWord(address,data);
	return SUCCESS;
	}

int writeLong(unsigned long address,unsigned long data){
	Device*	dev	= findWriteAddressedDevice(address);
	if(!dev) return FAILURE;
	if(!dev->hasLongAddress(address)) return FAILURE;
	dev->writeLong(address,data);
	return SUCCESS;
	}

static void prepareServer(int serialPortATcpPort,int serialPortBTcpPort,int debugPortTcpPort){
	int					debugPortListenerLength;
	int					serialPortAListenerLength;
	int					serialPortBListenerLength;
	struct sockaddr_in	debugPortListenerAddress;
	struct sockaddr_in	serialPortAListenerAddress;
	struct sockaddr_in	serialPortBListenerAddress;

	/*	Create a socket for the server */

	debugPortListenSocket	= socket(AF_INET,SOCK_STREAM,0);
	serialPortAListenSocket	= socket(AF_INET,SOCK_STREAM,0);
	serialPortBListenSocket	= socket(AF_INET,SOCK_STREAM,0);

	/* Name the socket */

	debugPortListenerAddress.sin_family	= AF_INET;
	serialPortAListenerAddress.sin_family	= AF_INET;
	serialPortBListenerAddress.sin_family	= AF_INET;

	// NEED htonl() and htons() for endianess!
	debugPortListenerAddress.sin_addr.s_addr		= htonl(INADDR_ANY);
	serialPortAListenerAddress.sin_addr.s_addr		= htonl(INADDR_ANY);
	serialPortBListenerAddress.sin_addr.s_addr		= htonl(INADDR_ANY);
	debugPortListenerAddress.sin_port		= htons(debugPortTcpPort);
	serialPortAListenerAddress.sin_port		= htons(serialPortATcpPort);
	serialPortBListenerAddress.sin_port		= htons(serialPortBTcpPort);
	debugPortListenerLength				= sizeof(debugPortListenerAddress);
	serialPortAListenerLength				= sizeof(serialPortAListenerAddress);
	serialPortBListenerLength				= sizeof(serialPortBListenerAddress);
	bind(debugPortListenSocket,(struct sockaddr *)&debugPortListenerAddress, debugPortListenerLength);
	bind(serialPortAListenSocket,(struct sockaddr *)&serialPortAListenerAddress, serialPortAListenerLength);
	bind(serialPortBListenSocket,(struct sockaddr *)&serialPortBListenerAddress, serialPortBListenerLength);

	listen(debugPortListenSocket,1);
	listen(serialPortAListenSocket,1);
	listen(serialPortBListenSocket,1);
	FD_SET(debugPortListenSocket,&readfds);
	FD_SET(serialPortAListenSocket,&readfds);
	FD_SET(serialPortBListenSocket,&readfds);
	}
