#include "clock.h"
#include "interrupt.h"

using namespace Oscl;

Clock::Clock(long cpc,int vector,int level,unsigned long base)
			:Device(vector,level,base),
			cyclesPerClock(cpc),
			interruptEnabled(false)
			{
	lastCycle	= 0;
	}

void Clock::reset(){
	Device::reset();
	lastCycle		= 0;
	timeRemaining	= cyclesPerClock;
	}

void Clock::poll(unsigned long cyclesSinceLastPoll){
	timeRemaining	-= cyclesSinceLastPoll;

	if(timeRemaining <= 0){
		timeRemaining	+= cyclesPerClock;
		if(interruptEnabled){
			setInterrupt();
			}
		}
	}

bool	Clock::hasAddress(unsigned long address){
	return (address == baseAddress+controlRegOffset)?true:false;
	}

bool	Clock::hasReadAddress(unsigned long address){
	return (address == baseAddress+controlRegOffset)?true:false;
	}

bool	Clock::hasWriteAddress(unsigned long address){
	return (address == baseAddress+controlRegOffset)?true:false;
	}

bool	Clock::hasByteAddress(unsigned long address){
	return (address == baseAddress+controlRegOffset)?true:false;
	}

bool	Clock::hasWordAddress(unsigned long address){
	return false;
	}

bool	Clock::hasLongAddress(unsigned long address){
	return false;
	}

unsigned char Clock::readByte(unsigned long address){
	return 0;
	}

unsigned short Clock::readWord(unsigned long address){
	return 0;
	}

unsigned long Clock::readLong(unsigned long address){
	return 0;
	}

void Clock::writeByte(unsigned long address,unsigned char data){
	if(data & 0x80){
		interruptEnabled	= false;
		}
	else if(data & 0x20){
		interruptEnabled	= true;
		}
	clearInterrupt();
	}

void Clock::writeWord(unsigned long address,unsigned short data){}

void Clock::writeLong(unsigned long address,unsigned long data){}

void Clock::open(){
	}

void Clock::close(){
	}

