#ifndef _sim68k_devices_deviceh_
#define _sim68k_devices_deviceh_
#include "oscl/queue/queue.h"
#include "interrupt.h"

/** */
class Device : public Oscl::QueueItem {
	private:
		/** */
		Interrupt			irq;
		/** */
		int					interrupting;
	protected:
		/** */
		const unsigned long baseAddress;
	public:
		/** */
		Device(int vector,int level,unsigned long base):
				irq(vector,level),
				interrupting(0),
				baseAddress(base)
				{
			}
		/** */
		inline virtual void reset(){
			if(interrupting) pendingInterrupts.remove(&irq);
			interrupting	= 0;
			}
		/** */
		virtual void poll(unsigned long cyclesSinceLastPoll) = 0;
		/** */
		virtual bool	hasAddress(unsigned long address) = 0;
		/** */
		virtual bool	hasReadAddress(unsigned long address) = 0;
		/** */
		virtual bool	hasWriteAddress(unsigned long address) = 0;
		/** */
		virtual bool	hasByteAddress(unsigned long address) = 0;
		/** */
		virtual bool	hasWordAddress(unsigned long address) = 0;
		/** */
		virtual bool	hasLongAddress(unsigned long address) = 0;
		/** */
		virtual unsigned char readByte(unsigned long address) = 0;
		/** */
		virtual unsigned short readWord(unsigned long address) = 0;
		/** */
		virtual unsigned long readLong(unsigned long address) = 0;
		/** */
		virtual void writeByte(unsigned long address,unsigned char data) = 0;
		/** */
		virtual void writeWord(unsigned long address,unsigned short data) = 0;
		/** */
		virtual void writeLong(unsigned long address,unsigned long data) = 0;
		/** */
		virtual void open()=0;
		/** */
		virtual void close()=0;
	protected:
		/** */
		inline void setInterrupt(){
			if(!interrupting){
				pendingInterrupts.put(&irq);
				interrupting++;
				}
			}
		/** */
		inline void addInterrupt(){
			if(!interrupting){
				pendingInterrupts.put(&irq);
				}
			interrupting++;
			}
		/** */
		inline void clearInterrupt(){
			if(interrupting){
				interrupting--;
				if(!interrupting) pendingInterrupts.remove(&irq);
				}
			}
	};

/** */
typedef Oscl::Queue<Device>	DeviceQ;

#endif
