#include <stdio.h>
#include "m2681rx.h"
#include "m2681sim.h"

#undef RX_PKT_TRACE

using namespace Oscl;

M2681RxFifo::M2681RxFifo(M2681RxFifoOwner& owner):
		_owner(owner),
		_nextOctet(0),
		_length(0)
		{
	}

void	M2681RxFifo::reset(){
	_nextOctet	= 0;
	_length		= 0;
	}

unsigned char	M2681RxFifo::read(){
	unsigned char	data;
	data	= _fifo[_nextOctet];
	_nextOctet++;
	_nextOctet	%= _fifoSize;
	return data;
	}

unsigned short	M2681RxFifo::getLength(){
	return _length;
	}

// input from line

void	M2681RxFifo::put(unsigned char data){
	_fifo[_nextOctet]	= data;
	_nextOctet++;
	_length++;
	if(_nextOctet < _fifoSize) return;
	_nextOctet	= 0;
	_owner.fifoFull(this);
	}

// M2681Rx

M2681Rx::M2681Rx(M2681RxOwner& owner):
						_owner(owner),
						_currentFifo(0),
						_activeFifo(0),
						_rxFifo1(*this),
						_rxFifo2(*this)
						{
	_rxFifo1.reset();
	_rxFifo2.reset();
	_emptyList.put(&_rxFifo1);
	_emptyList.put(&_rxFifo2);
	}

void M2681Rx::reset(){
	M2681RxFifo*	next;
	if(_activeFifo) _activeFifo->reset();
	if(_currentFifo){
		_currentFifo->reset();
		_emptyList.put(_currentFifo);
		_currentFifo	= 0;
		}
	while((next=_pendingList.get())){
		next->reset();
		_emptyList.put(next);
		}
	}

void M2681Rx::clock(unsigned long bitClocksSinceLastClock){
	}

void M2681Rx::enable(){
	}

void M2681Rx::disable(){
	}

void M2681Rx::enableInterrupt(){
	}

void M2681Rx::disableInterrupt(){
	}

unsigned char M2681Rx::readByte(){
	unsigned char	data;
	if(_currentFifo){
		data	= _currentFifo->read();
		releaseFifo();
		return data;
		}
	return 0;
	}

void M2681Rx::releaseFifo(){
	if(_currentFifo){
		_currentFifo->reset();
		_emptyList.put(_currentFifo);
		}
	_currentFifo	= _pendingList.get();
	if(_currentFifo){
		_owner.receiverNotEmpty();
		}
	else{
		_owner.receiverEmpty();
		}
	}

// OctetReceiver Interface
void M2681Rx::receiveBreak(){
#ifdef RX_PKT_TRACE
	fprintf(stderr," Break\n");
#endif
	}

void M2681Rx::receive(Oscl::Octet data){
	if(fifoActive()){
		putInActiveFifo(data);
		}
	else{
		receiveOverflow();
		}
	}

// M2681RxFifoOwner Interface

void M2681Rx::fifoFull(M2681RxFifo* fifo){
	if(!_currentFifo){
		_currentFifo	= fifo;
		_owner.receiverNotEmpty();
		}
	else{
		_pendingList.put(fifo);
		}
	_activeFifo	= _emptyList.get();
	}

void				M2681Rx::frameComplete(){	// check CRC,put RSTA copy in FIFO indicate EOF to FIFO
//	// CRC is always good for now
//	_rsta	|= (1<<RSTA_VFR) | (1<<RSTA_CRC);
//	if(_activeFifo){
//		putInActiveFifo(_rsta);
//		_activeFifo->endOfFrame();
//		}
//
//	flushDelayQueue();
	}

void				M2681Rx::putInActiveFifo(Oscl::Octet data){
	if(!_activeFifo)
		_activeFifo	= _emptyList.get();
	if(_activeFifo)
		_activeFifo->put(data);
	}

void				M2681Rx::receiveOverflow(){
	_owner.receiveOverflow();
	}

bool				M2681Rx::fifoActive(){
	if(!_activeFifo) _activeFifo	= _emptyList.get();
	return _activeFifo;
	}

