#ifndef _sim68k_devices_m2681rxh_
#define _sim68k_devices_m2681rxh_
#include "octetrx.h"
#include "oscl/queue/queue.h"

class M2681Sim;
class M2681Rx;
class M2681RxFifo;

class M2681RxOwner{
	public:
		virtual void	receiverNotEmpty()=0;
		virtual void	receiverEmpty()=0;
		virtual void	receiveOverflow()=0;
	};

class M2681RxFifoOwner{
	public:
		virtual void fifoFull(M2681RxFifo* fifo)=0;
	};

class M2681RxFifo : public Oscl::QueueItem {
	private:
		M2681RxFifoOwner&			_owner;
		static const unsigned int	_fifoSize = 1;
		Oscl::Octet					_fifo[_fifoSize];
		unsigned int				_nextOctet;
		unsigned int				_length;
	public:
		M2681RxFifo(M2681RxFifoOwner& owner);
		void			reset();
		unsigned char	read();
		unsigned short	getLength();
		// input from line
		void			put(unsigned char data);
		// Receiver interface
		bool			isEndOfFrame();
	};

class M2681Rx: public OctetReceiver , M2681RxFifoOwner{
	protected:
		M2681RxOwner&			_owner;
	private:
		Oscl::Queue<M2681RxFifo>	_pendingList;
		Oscl::Queue<M2681RxFifo>	_emptyList;
		M2681RxFifo*				_currentFifo;
		M2681RxFifo*				_activeFifo;
		M2681RxFifo					_rxFifo1;
		M2681RxFifo					_rxFifo2;
	public:
		M2681Rx(M2681RxOwner& owner);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void enableInterrupt();
		void disableInterrupt();
		unsigned char readByte();
		void releaseFifo();
		unsigned short	getLength();
	// OctetReceiver Interface
		void receiveBreak();
		void receive(Oscl::Octet data);
	// M2681RxFifoOwner Interface
		void fifoFull(M2681RxFifo* fifo);
	public: // RxState interface
		void				frameComplete();			// check CRC,put RSTA copy in FIFO indicate EOF to FIFO
		void				putInActiveFifo(Oscl::Octet data);
		void				receiveOverflow();
		bool				fifoActive();
	};

#endif
