#include "m2681sim.h"
#include "interrupt.h"

using namespace Oscl;

M2681Sim::M2681Sim(	unsigned long cyclesPerClock,
					int vector,
					int level,
					unsigned long base,
					OctetReceiver& receiverA,
					OctetReceiver& receiverB
					):
				Device(vector,level,base),
				_cyclesPerClock(cyclesPerClock),
				_receiverA(receiverA),
				_receiverB(receiverB),
				_txAOwner(*this),
				_txBOwner(*this),
				_rxAOwner(*this),
				_rxBOwner(*this),
				_rxa(_rxAOwner),
				_rxb(_rxBOwner),
				_txa(_txAOwner,&receiverA),
				_txb(_txBOwner,&receiverB),
				_transmitterAEnabled(false),
				_transmitterBEnabled(false),
				_transmitterReadyA(true),
				_transmitterReadyB(true),
				_receiverReadyA(false),
				_receiverReadyB(false),
				_location00(*this),
				_location01(*this),
				_location02(*this),
				_location03(*this),
				_location04(*this),
				_location05(*this),
				_location10(*this),
				_location11(*this),
				_location12(*this),
				_location13(*this),
				_location14(*this),
				_location15(*this),
				_location16(*this),
				_location17(*this),
				_location18(*this),
				_location19(*this),
				_location1A(*this),
				_location1B(*this),
				_location1C(*this),
				_location1D(*this),
				_location1E(*this),
				_location1F(*this),
				_location20(*this),
				_location21(*this)
				{
	_lastCycle						= 0;
	_cyclesRemainingFromLastPoll	= 0;
	}

void M2681Sim::reset(){
	unsigned char*	p;
	Device::reset();

	_rwShadow.mcr.h		= 0;
	_rwShadow.mcr.l		= 0;
	_rwShadow.ilr		= 0;
	_rwShadow.ivr		= 0x0F;	// unitialized interrupt indication
	_rwShadow.mr1a		= 0x00;
	_rwShadow.mr1b		= 0x00;
	p					= (unsigned char*)&_readShadow.sra;
	*p					= 0x00;
	p					= (unsigned char*)&_readShadow.srb;
	*p					= 0x00;
	_writeShadow.csra	= 0x00;
	_writeShadow.csrb	= 0x00;
	_writeShadow.cra	= 0x00;
	_writeShadow.crb	= 0x00;
	p					= (unsigned char*)&_readShadow.rba;
	*p					= 0x00;
	p					= (unsigned char*)&_readShadow.rbb;
	*p					= 0x00;
	_writeShadow.tba	= 0x00;
	_writeShadow.tbb	= 0x00;
	p					= (unsigned char*)&_readShadow.ipcr;
	*p					= 0x00;	// b000000UU
	_writeShadow.acr	= 0x00;
	p					= (unsigned char*)&_readShadow.isr;
	*p					= 0x00;
	_writeShadow.ier	= 0x00;
	p					= (unsigned char*)&_readShadow.ip;
	*p					= 0x00;	// b000000UU
	_writeShadow.opcr	= 0xFF;
	_writeShadow.set	= 0x00;
	_writeShadow.clear	= 0x00;
	_rwShadow.mr2a		= 0x00;
	_rwShadow.mr2b		= 0x00;

	_txa.reset();
	_txb.reset();
	_rxa.reset();
	_rxb.reset();
	_transmitterAEnabled	= false;
	_transmitterBEnabled	= false;
	_transmitterReadyA		= true;
	_transmitterReadyB		= true;
	_receiverReadyA			= false;
	_receiverReadyB			= false;
	updateAll();
	}

void M2681Sim::poll(unsigned long cyclesSinceLastPoll){
	unsigned long	clocksThisPoll;
	_cyclesRemainingFromLastPoll	+= cyclesSinceLastPoll;
	clocksThisPoll					= _cyclesRemainingFromLastPoll/_cyclesPerClock;
	_cyclesRemainingFromLastPoll	%=_cyclesPerClock;
	if(clocksThisPoll){
		_txa.clock(clocksThisPoll);
		_txb.clock(clocksThisPoll);
		_rxa.clock(clocksThisPoll);
		_rxb.clock(clocksThisPoll);
		}
	updateAll();
	}

bool	M2681Sim::hasAddress(unsigned long address){
	return inRange(address);
	}

bool	M2681Sim::hasReadAddress(unsigned long address){
	return inRange(address);
	}

bool	M2681Sim::hasWriteAddress(unsigned long address){
	return inRange(address);
	}

bool	M2681Sim::hasByteAddress(unsigned long address){
	return inRange(address);
	}

bool	M2681Sim::hasWordAddress(unsigned long address){
	return false;
	}

bool	M2681Sim::hasLongAddress(unsigned long address){
	return false;
	}

unsigned char M2681Sim::readByte(unsigned long address){
	unsigned long	relativeAddress	= address-baseAddress;
	M2681Loc& loc	= location(relativeAddress);
	return loc.readByte();
	}

unsigned short M2681Sim::readWord(unsigned long address){
	return 0;
	}

unsigned long M2681Sim::readLong(unsigned long address){
	return 0;
	}

void M2681Sim::writeByte(unsigned long address,unsigned char data){
	unsigned long	relativeAddress	= address-baseAddress;
	M2681Loc& loc	= location(relativeAddress);
	loc.writeByte(data);
	}

void M2681Sim::writeWord(unsigned long address,unsigned short data){}

void M2681Sim::writeLong(unsigned long address,unsigned long data){}

bool	M2681Sim::inRange(unsigned long address){
	return ( 		( (address >= (baseAddress+0x00) )	&& (address < (baseAddress+0x06) ) )
				||	( (address >= (baseAddress+0x10) )	&& (address < (baseAddress+0x22) ) )
				);
	}

M2681Loc& M2681Sim::location(unsigned long offset){
	switch(offset){
		case 0x00:
			return _location00;
		case 0x01:
			return _location01;
		case 0x02:
			return _location02;
		case 0x03:
			return _location03;
		case 0x04:
			return _location04;
		case 0x05:
			return _location05;
		case 0x10:
			return _location10;
		case 0x11:
			return _location11;
		case 0x12:
			return _location12;
		case 0x13:
			return _location13;
		case 0x14:
			return _location14;
		case 0x15:
			return _location15;
		case 0x16:
			return _location16;
		case 0x17:
			return _location17;
		case 0x18:
			return _location18;
		case 0x19:
			return _location19;
		case 0x1A:
			return _location1A;
		case 0x1B:
			return _location1B;
		case 0x1C:
			return _location1C;
		case 0x1D:
			return _location1D;
		case 0x1E:
			return _location1E;
		case 0x1F:
			return _location1F;
		case 0x20:
			return _location20;
		case 0x21:
			return _location21;
		default:
			return _location21;	// well....
		};
	}

// CR A/B Commands

void M2681Sim::resetReceiverA(){
	}

void M2681Sim::resetReceiverB(){
	}

void M2681Sim::resetTransmitterA(){
	}

void M2681Sim::resetTransmitterB(){
	}

void M2681Sim::resetErrorStatusA(){
	}

void M2681Sim::resetErrorStatusB(){
	}

void M2681Sim::resetBreakChangeInterruptA(){
	}

void M2681Sim::resetBreakChangeInterruptB(){
	}

void M2681Sim::startBreakA(){
	}

void M2681Sim::startBreakB(){
	}

void M2681Sim::stopBreakA(){
	}

void M2681Sim::stopBreakB(){
	}

void M2681Sim::assertRtsA(){
	}

void M2681Sim::assertRtsB(){
	}

void M2681Sim::negateRtsA(){
	}

void M2681Sim::negateRtsB(){
	}

void M2681Sim::transmitterEnableA(){
	_transmitterAEnabled	= true;
	updateISR();
	}

void M2681Sim::transmitterEnableB(){
	_transmitterBEnabled	= true;
	updateISR();
	}

void M2681Sim::transmitterDisableA(){
	_transmitterAEnabled	= false;
	updateISR();
	}

void M2681Sim::transmitterDisableB(){
	_transmitterBEnabled	= false;
	updateISR();
	}

void M2681Sim::receiverEnableA(){
	_receiverAEnabled	= true;
	updateISR();
	}

void M2681Sim::receiverEnableB(){
	_receiverBEnabled	= true;
	updateISR();
	}

void M2681Sim::receiverDisableA(){
	_receiverAEnabled	= false;
	updateISR();
	}

void M2681Sim::receiverDisableB(){
	_receiverBEnabled	= false;
	updateISR();
	}

// Receive and Transmit FIFO operations

void M2681Sim::writeTxOctetA(Oscl::Octet data){
	_txa.writeByte(data);
	}

void M2681Sim::writeTxOctetB(Oscl::Octet data){
	_txb.writeByte(data);
	}

unsigned char M2681Sim::readRxOctetA(){
	return _rxa.readByte();
	}

unsigned char M2681Sim::readRxOctetB(){
	return _rxb.readByte();
	}

// Transmitter interface

void M2681Sim::transmitFifoEmptyA(){
	_transmitterReadyA	= true;
	updateISR();
	}

void M2681Sim::transmitFifoNotEmptyA(){
	_transmitterReadyA	= false;
	updateISR();
	}

void M2681Sim::transmitFifoEmptyB(){
	_transmitterReadyB	= true;
	updateISR();
	}

void M2681Sim::transmitFifoNotEmptyB(){
	_transmitterReadyB	= false;
	updateISR();
	}

// Receiver interface

void M2681Sim::receiverNotEmptyA(){
	_receiverReadyA	= true;
	updateISR();
	}

void M2681Sim::receiverEmptyA(){
	_receiverReadyA	= false;
	updateISR();
	}

void M2681Sim::receiverNotEmptyB(){
	_receiverReadyB	= true;
	updateISR();
	}

void M2681Sim::receiverEmptyB(){
	_receiverReadyB	= false;
	updateISR();
	}

// Master status update operation

void M2681Sim::updateAll(){
	updateISR();
	}

// Interrupt Output Update

void M2681Sim::updateISR(){
	Fields*	p	= (Fields*)&_readShadow.isr;
	Fields*	sra	= (Fields*)&_readShadow.sra;
	Fields*	srb	= (Fields*)&_readShadow.srb;
	*p		&=	~(	m2681IsrTxRdyAMask
				|	m2681IsrTxRdyBMask
				|	m2681IsrRxRdyAMask
				|	m2681IsrRxRdyBMask
				);
	*sra	&=	~(	m2681SrTxEmpMask
				|	m2681SrTxRdyMask
				|	m2681SrFFullMask
				|	m2681SrRxRdyMask
				);
	*srb	&=	~(	m2681SrTxEmpMask
				|	m2681SrTxRdyMask
				|	m2681SrFFullMask
				|	m2681SrRxRdyMask
				);
	if(_transmitterReadyA && _transmitterAEnabled){
		*p		|=	m2681IsrTxRdyATransmitterReady;
		*sra	|=	m2681SrTxRdyMask;
		}
	else{
		*p		|=	m2681IsrTxRdyATransmitterNotReady;
		*sra	|=	m2681SrTxRdyTxNotReady;
		}
	if(_transmitterReadyB && _transmitterBEnabled){
		*p		|=	m2681IsrTxRdyBTransmitterReady;
		*srb	|=	m2681SrTxRdyMask;
		}
	else{
		*p		|=	m2681IsrTxRdyBTransmitterNotReady;
		*srb	|=	m2681SrTxRdyTxNotReady;
		}
	if(_receiverAEnabled && _receiverReadyA){
		*p		|=	m2681IsrRxRdyAReceiverReady;
		*sra	|=	m2681SrRxRdyRxReady;
		}
	else{
		*p		|=	m2681IsrRxRdyAReceiverNotReady;
		*sra	|=	m2681SrRxRdyRxNotReady;
		}
	if(_receiverBEnabled && _receiverReadyB){
		*p		|=	m2681IsrRxRdyBReceiverReady;
		*srb	|=	m2681SrRxRdyRxReady;
		}
	else{
		*p		|=	m2681IsrRxRdyBReceiverNotReady;
		*srb	|=	m2681SrRxRdyRxNotReady;
		}
	updateChipInterrupt();
	}

void M2681Sim::updateChipInterrupt(){
	if(_readShadow.isr & _writeShadow.ier){
		setInterrupt();
		}
	else{
		clearInterrupt();
		}
	}

void	M2681Sim::receiveOverflowA(){
	unsigned char	*sr = (unsigned char *)&_readShadow.sra;
	*sr	&= ~m2681SrOeMask;
	*sr	|= m2681SrOeRxOverrun;
	}

void	M2681Sim::receiveOverflowB(){
	unsigned char	*sr = (unsigned char *)&_readShadow.srb;
	*sr	&= ~m2681SrOeMask;
	*sr	|= m2681SrOeRxOverrun;
	}

void M2681Sim::sendToReceiverA(Oscl::Octet data){
	_rxa.receive(data);
	}

void M2681Sim::sendToReceiverB(Oscl::Octet data){
	_rxb.receive(data);
	}

void M2681Sim::open(){
	}

void M2681Sim::close(){
	}

///////////

M2681TxAOwnerImp::M2681TxAOwnerImp(M2681Sim& ser):_ser(ser){
	}

void M2681TxAOwnerImp::transmitFifoEmpty(){
	_ser.transmitFifoEmptyA();
	}

void M2681TxAOwnerImp::transmitFifoNotEmpty(){
	_ser.transmitFifoNotEmptyA();
	}

M2681RxAOwnerImp::M2681RxAOwnerImp(M2681Sim& ser):_ser(ser){
	}

void	M2681RxAOwnerImp::receiverNotEmpty(){
	_ser.receiverNotEmptyA();
	}

void	M2681RxAOwnerImp::receiverEmpty(){
	_ser.receiverEmptyA();
	}

void	M2681RxAOwnerImp::receiveOverflow(){
	_ser.receiveOverflowA();
	}

M2681TxBOwnerImp::M2681TxBOwnerImp(M2681Sim& ser):_ser(ser){
	}

void M2681TxBOwnerImp::transmitFifoEmpty(){
	_ser.transmitFifoEmptyB();
	}

void M2681TxBOwnerImp::transmitFifoNotEmpty(){
	_ser.transmitFifoNotEmptyB();
	}

M2681RxBOwnerImp::M2681RxBOwnerImp(M2681Sim& ser):_ser(ser){
	}

void	M2681RxBOwnerImp::receiverNotEmpty(){
	_ser.receiverNotEmptyB();
	}

void	M2681RxBOwnerImp::receiverEmpty(){
	_ser.receiverEmptyB();
	}

void	M2681RxBOwnerImp::receiveOverflow(){
	_ser.receiveOverflowB();
	}

