#ifndef _sim68k_devices_m2681simh_
#define _sim68k_devices_m2681simh_
#include "m2681loc.h"
#include "m2681tx.h"
#include "m2681rx.h"
#include "device.h"
#include "oscl/hw/motorola/m2681.h"

class M2681Sim;

class M2681TxAOwnerImp : public M2681TxOwner {
	private:
		M2681Sim&	_ser;
	public:
		M2681TxAOwnerImp(M2681Sim& ser);
		void transmitFifoEmpty();
		void transmitFifoNotEmpty();
	};

class M2681TxBOwnerImp : public M2681TxOwner {
	private:
		M2681Sim&	_ser;
	public:
		M2681TxBOwnerImp(M2681Sim& ser);
		void transmitFifoEmpty();
		void transmitFifoNotEmpty();
	};

class M2681RxAOwnerImp : public M2681RxOwner{
	private:
		M2681Sim&	_ser;
	public:
		M2681RxAOwnerImp(M2681Sim& ser);
		void	receiverEmpty();
		void	receiverNotEmpty();
		void	receiveOverflow();
	};

class M2681RxBOwnerImp : public M2681RxOwner{
	private:
		M2681Sim&	_ser;
	public:
		M2681RxBOwnerImp(M2681Sim& ser);
		void	receiverEmpty();
		void	receiverNotEmpty();
		void	receiveOverflow();
	};

class M2681Sim : public Device {
	public:
		const long	_cyclesPerClock;
	private:
		OctetReceiver& 		_receiverA;
		OctetReceiver& 		_receiverB;
		static const long	_commandPauseTicks	= 400;
		long				_lastCycle;
		unsigned long		_cyclesRemainingFromLastPoll;
		long				_commandTimer;
		M2681TxAOwnerImp	_txAOwner;
		M2681TxBOwnerImp	_txBOwner;
		M2681RxAOwnerImp	_rxAOwner;
		M2681RxBOwnerImp	_rxBOwner;
		M2681Rx				_rxa;
		M2681Rx				_rxb;
		M2681Tx				_txa;
		M2681Tx				_txb;
		bool				_transmitterAEnabled;
		bool				_transmitterBEnabled;
		bool				_receiverAEnabled;
		bool				_receiverBEnabled;
		bool				_transmitterReadyA;
		bool				_transmitterReadyB;
		bool				_receiverReadyA;
		bool				_receiverReadyB;
	public:
		volatile	M2681ReadOnly	_readShadow;
		volatile	M2681WriteOnly	_writeShadow;
					M2681ReadWrite	_rwShadow;
		M2681Loc00 _location00;
		M2681Loc01 _location01;
		M2681Loc02 _location02;
		M2681Loc03 _location03;
		M2681Loc04 _location04;
		M2681Loc05 _location05;
		M2681Loc10 _location10;
		M2681Loc11 _location11;
		M2681Loc12 _location12;
		M2681Loc13 _location13;
		M2681Loc14 _location14;
		M2681Loc15 _location15;
		M2681Loc16 _location16;
		M2681Loc17 _location17;
		M2681Loc18 _location18;
		M2681Loc19 _location19;
		M2681Loc1A _location1A;
		M2681Loc1B _location1B;
		M2681Loc1C _location1C;
		M2681Loc1D _location1D;
		M2681Loc1E _location1E;
		M2681Loc1F _location1F;
		M2681Loc20 _location20;
		M2681Loc21 _location21;
	public:
		M2681Sim(	unsigned long cpc,
					int vector,
					int level,
					unsigned long baseAddress,
					OctetReceiver& receiverA,
					OctetReceiver& receiverB
					);
		virtual void	reset();
		virtual void	poll(unsigned long cyclesSinceLastPoll);
		virtual bool	hasAddress(unsigned long address);
		virtual bool	hasReadAddress(unsigned long address);
		virtual bool	hasWriteAddress(unsigned long address);
		virtual bool	hasByteAddress(unsigned long address);
		virtual bool	hasWordAddress(unsigned long address);
		virtual bool	hasLongAddress(unsigned long address);
		virtual unsigned char readByte(unsigned long address);
		virtual unsigned short readWord(unsigned long address);
		virtual unsigned long readLong(unsigned long address);
		virtual void writeByte(unsigned long address,unsigned char data);
		virtual void writeWord(unsigned long address,unsigned short data);
		virtual void writeLong(unsigned long address,unsigned long data);
		virtual void open();
		virtual void close();
		// Memmory/Loc map interface
		// CR A/B Commands
		void resetReceiverA();
		void resetReceiverB();
		void resetTransmitterA();
		void resetTransmitterB();
		void resetErrorStatusA();
		void resetErrorStatusB();
		void resetBreakChangeInterruptA();
		void resetBreakChangeInterruptB();
		void startBreakA();
		void startBreakB();
		void stopBreakA();
		void stopBreakB();
		void assertRtsA();
		void assertRtsB();
		void negateRtsA();
		void negateRtsB();
		void transmitterEnableA();
		void transmitterEnableB();
		void transmitterDisableA();
		void transmitterDisableB();
		void receiverEnableA();
		void receiverEnableB();
		void receiverDisableA();
		void receiverDisableB();
		// Receive and Transmit FIFO operations
		void writeTxOctetA(Oscl::Octet data);
		void writeTxOctetB(Oscl::Octet data);
		unsigned char readRxOctetA();
		unsigned char readRxOctetB();
		// transmitter interface
		void transmitFifoEmptyA();
		void transmitFifoNotEmptyA();
		void transmitFifoEmptyB();
		void transmitFifoNotEmptyB();
		// receiver interface
		void receiverNotEmptyA();
		void receiverEmptyA();
		void receiverNotEmptyB();
		void receiverEmptyB();
		void receiveOverflowA();
		void receiveOverflowB();
		// External character source input to receivers
		void sendToReceiverA(Oscl::Octet data);
		void sendToReceiverB(Oscl::Octet data);
	private:
		// Master status update operation
		void updateISR();
		void updateAll();
		void updateChipInterrupt();
	public:
		M2681Loc& location(unsigned long offset);
	private:
		bool	inRange(unsigned long address);
	};

#endif

