#ifndef _sim68k_devices_m2681txh_
#define _sim68k_devices_m2681txh_
#include "oscl/queue/queue.h"
#include "octetrx.h"

class M2681Sim;
class M2681TxFifo;
class M2681Tx;

class M2681TxOwner{
	public:
		virtual void transmitFifoEmpty()=0;
		virtual void transmitFifoNotEmpty()=0;
	};

class M2681TxFifoOwner{
	public:
		virtual void txFifoEmpty(M2681TxFifo* theFifo)=0;
	};

class M2681TxFifoState{
	protected:
		M2681TxFifo&	_fifo;
	public:
		M2681TxFifoState(M2681TxFifo& fifo);
		virtual void clock(unsigned long bitClocksSinceLastClock)=0;
	};

class M2681TxFifoStateReset:public M2681TxFifoState{
	public:
		M2681TxFifoStateReset(M2681TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class M2681TxFifoStateEmpty:public M2681TxFifoState{
	public:
		M2681TxFifoStateEmpty(M2681TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class M2681TxFifoStateNewFrame:public M2681TxFifoState{
	public:
		M2681TxFifoStateNewFrame(M2681TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class M2681TxFifoStateInfo:public M2681TxFifoState{
	public:
		M2681TxFifoStateInfo(M2681TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class M2681TxFifo: public Oscl::QueueItem {
	private:
		M2681TxFifoStateReset		_resetState;
		M2681TxFifoState*			_state;
		M2681TxFifoOwner&			_owner;
		static const unsigned int	_fifoSize = 1;
		OctetReceiver*				_receiver;
		unsigned int				_octetsToTransmit;
		Oscl::Octet					_fifo[_fifoSize];
		unsigned int				_currentOctet;
		unsigned int				_remainingClocksBeforeNextTransmit;
	public:	// Transmitter interface
		M2681TxFifo(M2681TxFifoOwner& owner,OctetReceiver* receiver);
		M2681TxFifo(M2681TxFifoOwner& owner);
		OctetReceiver* changeReceiver(OctetReceiver* receiver);
		void reset();
		void byteClock();
		void put(Oscl::Octet data);
	};

class M2681TxState{
	protected:
		M2681Tx&	_tx;
	public:
		M2681TxState(M2681Tx& tx);
		virtual void reset() = 0;
		virtual void poll(unsigned long cyclesSinceLastPoll) = 0;
		virtual void enable() = 0;
		virtual void disable() = 0;
	};

class M2681TxStateReset : public M2681TxState{
	public:
		M2681TxStateReset(M2681Tx& tx);
		void reset();
		void poll(unsigned long cyclesSinceLastPoll);
		void enable();
		void disable();
	};

class M2681Tx:public M2681TxFifoOwner{
	protected:
		OctetReceiver*		_rx;
	private:
		Oscl::Queue<class M2681TxFifo>	_emptyList;
		Oscl::Queue<class M2681TxFifo>	_activeList;
		M2681TxOwner&				_owner;
		M2681TxStateReset			_resetState;
		M2681TxState*				_state;
		M2681TxFifo					_txFifo1;
		M2681TxFifo					_txFifo2;
		M2681TxFifo					_txFifo3;
		M2681TxFifo					_txFifo4;
		M2681TxFifo*				_currentFifo;
		M2681TxFifo*				_activeFifo;
		bool						_enabled;
		unsigned long				_remainingClocksBeforeNextByteClock;
	public:
		M2681Tx(M2681TxOwner& owner);
		M2681Tx(M2681TxOwner& owner,OctetReceiver* rx);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void writeByte(unsigned char data);
		OctetReceiver* changeReceiver(OctetReceiver* rx);	// returns old receiver

		// M2681TxFifoOwner Interface
		void txFifoEmpty(M2681TxFifo* theFifo);
	};

#endif
