#include "sab82525sim.h"
#include "interrupt.h"

using namespace Oscl;

Sab82525Sim::Sab82525Sim(unsigned long cyclesPerClock,int vector,int level,unsigned long base):
		Device(vector,level,base),
		_deviceA(*this),
		_deviceB(*this),
		_channelA(_deviceA),
		_channelB(_deviceB),
		_cyclesPerClock(cyclesPerClock)
		{
	_lastCycle						= 0;
	_cyclesRemainingFromLastPoll	= 0;
	}

void Sab82525Sim::reset(){
	Device::reset();
	_channelA.reset();
	_channelB.reset();
	updateAll();
	}

void Sab82525Sim::poll(unsigned long cyclesSinceLastPoll){
	unsigned long	clocksThisPoll;
	_cyclesRemainingFromLastPoll	+= cyclesSinceLastPoll;
	clocksThisPoll					= _cyclesRemainingFromLastPoll/_cyclesPerClock;
	_cyclesRemainingFromLastPoll	%=_cyclesPerClock;
	if(clocksThisPoll){
		_channelA.poll(clocksThisPoll);
		_channelB.poll(clocksThisPoll);
		}
//	updateAll();
	}

bool	Sab82525Sim::hasAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82525Sim::hasReadAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82525Sim::hasWriteAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82525Sim::hasByteAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82525Sim::hasWordAddress(unsigned long address){
	return false;
	}

bool	Sab82525Sim::hasLongAddress(unsigned long address){
	return false;
	}

unsigned char Sab82525Sim::readByte(unsigned long address){
	unsigned long	relativeAddress	= address-baseAddress;
	if(relativeAddress > 0x3F){
		relativeAddress -= 0x040;
		return _channelB.readByte(relativeAddress);
		}
	else{
		return _channelA.readByte(relativeAddress);
		}
	}

unsigned short Sab82525Sim::readWord(unsigned long address){
	return 0;
	}

unsigned long Sab82525Sim::readLong(unsigned long address){
	return 0;
	}

void Sab82525Sim::writeByte(unsigned long address,unsigned char data){
	unsigned long	relativeAddress	= address-baseAddress;
	if(relativeAddress > 0x3F){
		relativeAddress -= 0x040;
		_channelB.writeByte(relativeAddress,data);
		}
	else{
		_channelA.writeByte(relativeAddress,data);
		}
	}

void Sab82525Sim::writeWord(unsigned long address,unsigned short data){}

void Sab82525Sim::writeLong(unsigned long address,unsigned long data){}

bool	Sab82525Sim::inRange(unsigned long address){
	return ((address >= baseAddress) && (address < baseAddress+sizeof(SAB82525)));
	}

void	Sab82525Sim::updateAll(){
	updateChipInterrupt();
	}

void Sab82525Sim::updateChipInterrupt(){
	if(		_channelB.channelInterruptIsPending()
		|	_channelA.channelInterruptIsPending()
		|	_channelB.extendedInterruptIsPending()
		|	_channelA.extendedInterruptIsPending()
		)
		{
		setInterrupt();
		}
	else{
		clearInterrupt();
		}
	}

void Sab82525Sim::open(){
	}

void Sab82525Sim::close(){
	}

bool			Sab82525Sim::channelInterruptPendingA(){
	return _channelA.channelInterruptIsPending();
	}

bool			Sab82525Sim::channelExtendedInterruptPendingA(){
	return _channelA.extendedInterruptIsPending();
	}

bool			Sab82525Sim::channelRawExtendedInterruptPendingA(){
	return _channelA.rawExtendedInterruptIsPending();
	}

bool			Sab82525Sim::channelExtendedInterruptMaskedA(){
	return _channelA.extendedInterruptIsMasked();
	}

void			Sab82525Sim::channelInterruptChangedA(){
	_channelA.oppositeChannelInterruptStatusChanged();
	}

bool			Sab82525Sim::channelInterruptPendingB(){
	return _channelB.channelInterruptIsPending();
	}

bool			Sab82525Sim::channelExtendedInterruptPendingB(){
	return _channelB.extendedInterruptIsPending();
	}

bool			Sab82525Sim::channelRawExtendedInterruptPendingB(){
	return _channelB.rawExtendedInterruptIsPending();
	}

bool			Sab82525Sim::channelExtendedInterruptMaskedB(){
	return _channelB.extendedInterruptIsMasked();
	}

void			Sab82525Sim::channelInterruptChangedB(){
	_channelB.oppositeChannelInterruptStatusChanged();
	}

////////////////// Sab82525ChannelA //////////////////

Sab82525ChannelA::Sab82525ChannelA(Sab82525Sim& device):
					_device(device)
					{
	}

bool	Sab82525ChannelA::oppositeChannelInterruptPending(){
	return _device.channelInterruptPendingB();
	}

bool	Sab82525ChannelA::oppositeChannelExtendedInterruptPending(){
	return _device.channelExtendedInterruptPendingB();
	}

bool	Sab82525ChannelA::oppositeChannelRawExtendedInterruptPending(){
	return _device.channelRawExtendedInterruptPendingB();
	}

bool	Sab82525ChannelA::thisChannelRawExtendedInterruptPending(){
	return _device.channelRawExtendedInterruptPendingA();
	}

bool	Sab82525ChannelA::oppositeChannelExtendedInterrruptMasked(){
	return _device.channelExtendedInterruptMaskedB();
	}

void	Sab82525ChannelA::opositeChannelInterruptStatusChanged(){
	// _device.channelInterruptChangedB();
	}

void	Sab82525ChannelA::updateChipInterrupt(){
	_device.updateChipInterrupt();
	}

////////////////// Sab82525ChannelB //////////////////

Sab82525ChannelB::Sab82525ChannelB(Sab82525Sim& device):
					_device(device)
					{
	}

bool	Sab82525ChannelB::oppositeChannelInterruptPending(){
	return _device.channelInterruptPendingA();
	}

bool	Sab82525ChannelB::oppositeChannelExtendedInterruptPending(){
	return _device.channelExtendedInterruptPendingA();
	}

bool	Sab82525ChannelB::oppositeChannelRawExtendedInterruptPending(){
	return _device.channelRawExtendedInterruptPendingA();
	}

bool	Sab82525ChannelB::thisChannelRawExtendedInterruptPending(){
	return _device.channelRawExtendedInterruptPendingB();
	}

bool	Sab82525ChannelB::oppositeChannelExtendedInterrruptMasked(){
	return _device.channelExtendedInterruptMaskedA();
	}

void	Sab82525ChannelB::opositeChannelInterruptStatusChanged(){
	_device.channelInterruptChangedA();
	}

void	Sab82525ChannelB::updateChipInterrupt(){
	_device.updateChipInterrupt();
	}

