#ifndef _sim68k_devices_sab82525simh_
#define _sim68k_devices_sab82525simh_
#include "sab8252xsim.h"
#include "device.h"
#include "oscl/hw/siemens/hscx.h"

class Sab82525Sim;

class Sab82525ChannelA : public Sab8252xDevice {
	private:
		Sab82525Sim&	_device;
	public:
		Sab82525ChannelA(Sab82525Sim& device);
	public:
		bool	oppositeChannelInterruptPending();
		bool	oppositeChannelExtendedInterruptPending();
		bool	oppositeChannelRawExtendedInterruptPending();
		bool	thisChannelRawExtendedInterruptPending();
	public:
		bool	oppositeChannelExtendedInterrruptMasked();
		void	opositeChannelInterruptStatusChanged();
	public:
		void	updateChipInterrupt();
	};

class Sab82525ChannelB : public Sab8252xDevice {
	private:
		Sab82525Sim&	_device;
	public:
		Sab82525ChannelB(Sab82525Sim& device);
	public:
		bool	oppositeChannelInterruptPending();
		bool	oppositeChannelExtendedInterruptPending();
		bool	oppositeChannelRawExtendedInterruptPending();
		bool	thisChannelRawExtendedInterruptPending();
	public:
		bool	oppositeChannelExtendedInterrruptMasked();
		void	opositeChannelInterruptStatusChanged();
	public:
		void	updateChipInterrupt();
	};

class Sab82525Sim : public Device {
	private:	// Device data
		long					_lastCycle;
		unsigned long			_cyclesRemainingFromLastPoll;
		Sab82525ChannelA		_deviceA;
		Sab82525ChannelB		_deviceB;
		Sab8252xSim				_channelA;
		Sab8252xSim				_channelB;
		const long				_cyclesPerClock;
	public:	// Constructor
		Sab82525Sim(unsigned long cpc,int vector,int level,unsigned long baseAddress);
	public:	// Device interface
		void			reset();
		void			poll(unsigned long cyclesSinceLastPoll);
		bool			hasAddress(unsigned long address);
		bool			hasReadAddress(unsigned long address);
		bool			hasWriteAddress(unsigned long address);
		bool			hasByteAddress(unsigned long address);
		bool			hasWordAddress(unsigned long address);
		bool			hasLongAddress(unsigned long address);
		unsigned char	readByte(unsigned long address);
		unsigned short	readWord(unsigned long address);
		unsigned long	readLong(unsigned long address);
		void			writeByte(unsigned long address,unsigned char data);
		void			writeWord(unsigned long address,unsigned short data);
		void			writeLong(unsigned long address,unsigned long data);
		void			open();
		void			close();
	public: // Sab82525ChannelA/B interface
		bool			channelInterruptPendingA();
		bool			channelExtendedInterruptPendingA();
		bool			channelRawExtendedInterruptPendingA();
		bool			channelExtendedInterruptMaskedA();
		void			channelInterruptChangedA();
	public: // Sab82525ChannelA/B interface
		bool			channelInterruptPendingB();
		bool			channelExtendedInterruptPendingB();
		bool			channelRawExtendedInterruptPendingB();
		bool			channelExtendedInterruptMaskedB();
		void			channelInterruptChangedB();
	public:
		void			updateChipInterrupt();
	private:
		void			updateAll();
	private:
		bool			inRange(unsigned long address);
	};

#endif

