#ifndef _sim68k_devices_sab82526rxh_
#define _sim68k_devices_sab82526rxh_
#include "framerx.h"
#include "oscl/queue/queue.h"

class Sab82526Sim;
class Sab82526RxState;
class Sab82526Rx;
class Sab82526RxFifo;

class Sab82526RxFifoOwner{
	public:
		virtual void fifoFull(Sab82526RxFifo* fifo)=0;
		virtual void endOfFrame(Sab82526RxFifo* fifo)=0;
	};

class Sab82526RxFifo : public Oscl::QueueItem {
	private:
		Sab82526RxFifoOwner&		_owner;
		static const unsigned int	_fifoSize = 32;
		Oscl::Octet					_fifo[_fifoSize];
		unsigned int				_nextOctet;
		unsigned int				_length;
		bool						_endOfFrame;
	public:
		Sab82526RxFifo(Sab82526RxFifoOwner& owner);
		void			reset();
		unsigned char	read();
		unsigned short	getLength();
		// input from line
		void			put(unsigned char data);
		void			endOfFrame();
		// Receiver interface
		bool			isEndOfFrame();
	};

class Sab82526RxState{
	protected:
		Sab82526Rx&	_rx;
	public:
		Sab82526RxState(Sab82526Rx& rx);
		void reset();
		virtual void flag()=0;
		virtual void receive(Oscl::Octet data)=0;
	};

class Sab82526RxStateHunt: public Sab82526RxState{
	public:
		Sab82526RxStateHunt(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxStateMds01Adm0Stage1: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm0Stage1(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxStateMds01Adm0Stage2: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm0Stage2(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxStateMds01Adm0Stage3: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm0Stage3(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxStateMds01Adm0Stage4: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm0Stage4(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};


class Sab82526RxStateMds01Adm1Stage1: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm1Stage1(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxStateMds01Adm1Stage2: public Sab82526RxState{
	public:
		Sab82526RxStateMds01Adm1Stage2(Sab82526Rx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab82526RxDataNode : public Oscl::QueueItem {
	public:
		Oscl::Octet	_data;
	};

class Sab82526Rx: public FrameReceiver,Sab82526RxFifoOwner{
	protected:
		Sab82526Sim&			_sab;
	private:
		Sab82526RxStateHunt				_huntState;
		Sab82526RxStateMds01Adm0Stage1	_mds01Adm0Stage1State;
		Sab82526RxStateMds01Adm0Stage2	_mds01Adm0Stage2State;
		Sab82526RxStateMds01Adm0Stage3	_mds01Adm0Stage3State;
		Sab82526RxStateMds01Adm0Stage4	_mds01Adm0Stage4State;
		Sab82526RxStateMds01Adm1Stage1	_mds01Adm1Stage1State;
		Sab82526RxStateMds01Adm1Stage2	_mds01Adm1Stage2State;
		unsigned char					_rsta;
		Sab82526RxState*				_state;
		Oscl::Queue<Sab82526RxDataNode>		_delayQ;
		Oscl::Queue<Sab82526RxDataNode>		_freeList;
		Oscl::Queue<Sab82526RxFifo>			_pendingList;
		Oscl::Queue<Sab82526RxFifo>			_emptyList;
		Sab82526RxDataNode				_delayNode1;
		Sab82526RxDataNode				_delayNode2;
		Sab82526RxFifo*					_currentFifo;
		Sab82526RxFifo*					_activeFifo;
		Sab82526RxFifo					_rxFifo1;
		Sab82526RxFifo					_rxFifo2;
		unsigned short					_rxLength;
	public:
		Sab82526Rx(Sab82526Sim& sab);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void enableInterrupt();
		void disableInterrupt();
		unsigned char readByte(unsigned short index);
		void releaseFifo();
		unsigned short	getLength();
	// FrameReceiver Interface
		void receiveFlag();
		void receive(Oscl::Octet data);
	// Sab82526RxFifoOwner Interface
		void fifoFull(Sab82526RxFifo* fifo);
		void endOfFrame(Sab82526RxFifo* fifo);
	public: // RxState state change interface
		void setStartOfFrameState();
		void setRxStateHunt();
		void setRxStateMds01Adm0Stage1();
		void setRxStateMds01Adm0Stage2();
		void setRxStateMds01Adm0Stage3();
		void setRxStateMds01Adm0Stage4();
		void setRxStateMds01Adm1Stage1();
		void setRxStateMds01Adm1Stage2();
	public: // RxState interface
		bool				isMyAddressH(Oscl::Octet data);	// true if data is same as RAH1 or RAH2
		bool				isMyAddressL(Oscl::Octet data);	// true if data is same as RAL1 or RAL2
		void				resetCRC();					// reset the CRC accumulator
		void				accumulateCRC(Oscl::Octet data);	// accumulate data into current CRC
		void				runtFrameReceived();
		Sab82526RxDataNode*	getFreeNode();
		void				putNodeInDelayQueue(Sab82526RxDataNode* node);
		void				frameComplete();			// check CRC,put RSTA copy in FIFO indicate EOF to FIFO
		void				flushDelayQueue();
		Sab82526RxDataNode*	getDelayQueue();
		void				putInActiveFifo(Oscl::Octet data);
		void				receiveOverflow();
		bool				fifoActive();
	};

#endif
