#include "sab82526sim.h"
#include "interrupt.h"

using namespace Oscl;

Sab82526Sim::Sab82526Sim(unsigned long cyclesPerClock,int vector,int level,unsigned long base):
		Device(vector,level,base),
		_cyclesPerClock(cyclesPerClock),
		_rx(*this),
		_tx(*this,&_rx),	// Setup for echo
		_location00(*this),
		_location01(*this),
		_location02(*this),
		_location03(*this),
		_location04(*this),
		_location05(*this),
		_location06(*this),
		_location07(*this),
		_location08(*this),
		_location09(*this),
		_location0A(*this),
		_location0B(*this),
		_location0C(*this),
		_location0D(*this),
		_location0E(*this),
		_location0F(*this),
		_location10(*this),
		_location11(*this),
		_location12(*this),
		_location13(*this),
		_location14(*this),
		_location15(*this),
		_location16(*this),
		_location17(*this),
		_location18(*this),
		_location19(*this),
		_location1A(*this),
		_location1B(*this),
		_location1C(*this),
		_location1D(*this),
		_location1E(*this),
		_location1F(*this),
		_location20(*this),
		_location21(*this),
		_location22(*this),
		_location23(*this),
		_location24(*this),
		_location25(*this),
		_location26(*this),
		_location27(*this),
		_location28(*this),
		_location29(*this),
		_location2A(*this),
		_location2B(*this),
		_location2C(*this),
		_location2D(*this),
		_location2E(*this),
		_location2F(*this),
		_location30(*this),
		_location31(*this),
		_location32(*this),
		_location33(*this),
		_location34(*this),
		_location35(*this),
		_location36(*this),
		_location37(*this),
		_location38(*this),
		_location39(*this),
		_location3A(*this),
		_location3B(*this),
		_location3C(*this),
		_location3D(*this),
		_location3E(*this),
		_location3F(*this)
		{
	_lastCycle						= 0;
	_cyclesRemainingFromLastPoll	= 0;
	}

void Sab82526Sim::reset(){
	unsigned char*	p;
	Device::reset();
	p					= (unsigned char*)&_readShadow.ISTA;
	*p					= 0;
	_writeShadow.MASK	= 0;	// all enabled
	p					= (unsigned char*)&_readShadow.EXIR;
	*p					= 0;
	p					= (unsigned char*)&_readShadow.STAR;
	*p					= 0x48;
	_writeShadow.CMDR	= 0;	// according to the manual
	_rwShadow.MODE		= 0;
	_rwShadow.TIMR		= 0;	// not specified in the manual
	_writeShadow.XAD1	= 0;	// not specified in the manual
	_writeShadow.XAD2	= 0;	// not specified in the manual
	p					= (unsigned char*)&_readShadow.RBCL;
	*p					= 0;	// not specified in the manual
	_writeShadow.RAH1	= 0;	// not specified in the manual
	_writeShadow.RAH2	= 0;	// not specified in the manual
	p					= (unsigned char*)&_readShadow.RSTA;
	*p					= 0;	// not specified in the manual
	_rwShadow.RAL1		= 0;	// not specified in the manual
	_writeShadow.RAL2	= 0;	// not specified in the manual
	p					= (unsigned char*)&_readShadow.RHCR;
	*p					= 0;	// not specified in the manual
	_writeShadow.XBCL	= 0;	// not specified in the manual
	_writeShadow.BGR	= 0;	// not specified in the manual
	_rwShadow.CCR2		= 0;
	_writeShadow.XBCH	= 0;	// 000XXXXX
	p					= (unsigned char*)&_readShadow.RBCH;
	*p					= 0;	// 000XXXXX
	p					= (unsigned char*)&_readShadow.VSTR;
	*p					= 0x02;	// version number of chip
	_writeShadow.RLCR	= 0;	// not specified in the manual
	_writeShadow.TSAX	= 0;	// not specified in the manual
	_writeShadow.TSAR	= 0;	// not specified in the manual
	_writeShadow.XCCR	= 0;
	_writeShadow.RCCR	= 0;
	_ISTA				= 0;
	_tx.reset();
	_rx.reset();
	updateAll();
	}

void Sab82526Sim::poll(unsigned long cyclesSinceLastPoll){
	unsigned long	clocksThisPoll;
	_cyclesRemainingFromLastPoll	+= cyclesSinceLastPoll;
	clocksThisPoll					= _cyclesRemainingFromLastPoll/_cyclesPerClock;
	_cyclesRemainingFromLastPoll	%=_cyclesPerClock;
	if(clocksThisPoll){
		_tx.clock(clocksThisPoll);
		_rx.clock(clocksThisPoll);
		}
	updateAll();
	if(_commandTimer){
		_commandTimer -= cyclesSinceLastPoll;
		if (_commandTimer <= 0){
			unsigned char*		star	= (unsigned char *)&_readShadow.STAR;
			_commandTimer = 0;
			*star	&= ~STAR_CEC;
			}
		}
	}

bool	Sab82526Sim::hasAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82526Sim::hasReadAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82526Sim::hasWriteAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82526Sim::hasByteAddress(unsigned long address){
	return inRange(address);
	}

bool	Sab82526Sim::hasWordAddress(unsigned long address){
	return false;
	}

bool	Sab82526Sim::hasLongAddress(unsigned long address){
	return false;
	}

unsigned char Sab82526Sim::readByte(unsigned long address){
	unsigned long	relativeAddress	= address-baseAddress;
	Sab82526Loc& loc	= location(relativeAddress);
	return loc.readByte();
	}

unsigned short Sab82526Sim::readWord(unsigned long address){
	return 0;
	}

unsigned long Sab82526Sim::readLong(unsigned long address){
	return 0;
	}

void Sab82526Sim::writeByte(unsigned long address,unsigned char data){
	unsigned long	relativeAddress	= address-baseAddress;
	Sab82526Loc& loc	= location(relativeAddress);
	loc.writeByte(data);
	}

void Sab82526Sim::writeWord(unsigned long address,unsigned short data){}

void Sab82526Sim::writeLong(unsigned long address,unsigned long data){}

bool	Sab82526Sim::inRange(unsigned long address){
	return ((address >= baseAddress) && (address < baseAddress+sizeof(SAB82526)));
	}

Sab82526Loc& Sab82526Sim::location(unsigned long offset){
	switch(offset){
		case 0x00:
			return _location00;
		case 0x01:
			return _location01;
		case 0x02:
			return _location02;
		case 0x03:
			return _location03;
		case 0x04:
			return _location04;
		case 0x05:
			return _location05;
		case 0x06:
			return _location06;
		case 0x07:
			return _location07;
		case 0x08:
			return _location08;
		case 0x09:
			return _location09;
		case 0x0A:
			return _location0A;
		case 0x0B:
			return _location0B;
		case 0x0C:
			return _location0C;
		case 0x0D:
			return _location0D;
		case 0x0E:
			return _location0E;
		case 0x0F:
			return _location0F;
		case 0x10:
			return _location10;
		case 0x11:
			return _location11;
		case 0x12:
			return _location12;
		case 0x13:
			return _location13;
		case 0x14:
			return _location14;
		case 0x15:
			return _location15;
		case 0x16:
			return _location16;
		case 0x17:
			return _location17;
		case 0x18:
			return _location18;
		case 0x19:
			return _location19;
		case 0x1A:
			return _location1A;
		case 0x1B:
			return _location1B;
		case 0x1C:
			return _location1C;
		case 0x1D:
			return _location1D;
		case 0x1E:
			return _location1E;
		case 0x1F:
			return _location1F;
		case 0x20:
			return _location20;
		case 0x21:
			return _location21;
		case 0x22:
			return _location22;
		case 0x23:
			return _location23;
		case 0x24:
			return _location24;
		case 0x25:
			return _location25;
		case 0x26:
			return _location26;
		case 0x27:
			return _location27;
		case 0x28:
			return _location28;
		case 0x29:
			return _location29;
		case 0x2A:
			return _location2A;
		case 0x2B:
			return _location2B;
		case 0x2C:
			return _location2C;
		case 0x2D:
			return _location2D;
		case 0x2E:
			return _location2E;
		case 0x2F:
			return _location2F;
		case 0x30:
			return _location30;
		case 0x31:
			return _location31;
		case 0x32:
			return _location32;
		case 0x33:
			return _location33;
		case 0x34:
			return _location34;
		case 0x35:
			return _location35;
		case 0x36:
			return _location36;
		case 0x37:
			return _location37;
		case 0x38:
			return _location38;
		case 0x39:
			return _location39;
		case 0x3A:
			return _location3A;
		case 0x3B:
			return _location3B;
		case 0x3C:
			return _location3C;
		case 0x3D:
			return _location3D;
		case 0x3E:
			return _location3E;
		case 0x3F:
			return _location3F;
		default:
			return _location3F;	// well....
		};
	}

void Sab82526Sim::hscxReceiveMessageComplete(){
	_rx.releaseFifo();
	}

void Sab82526Sim::hscxResetHdlcReceiver(){
	_rx.reset();
	}

void Sab82526Sim::hscxReceiverNotReady(){
	}

void Sab82526Sim::hscxTransmissionRepeat(){
	}

void Sab82526Sim::hscxStartTimer(){
	}

void Sab82526Sim::hscxTransmitTransparentFrame(){
	// XTF
	_tx.transmitContentsOfFifo();
	}

void Sab82526Sim::hscxTransmitIFrame(){
	}

void Sab82526Sim::hscxTransmitMessageEnd(){
	// XME
	_tx.endOfFrame();
	}

void Sab82526Sim::hscxTransmitReset(){
	_tx.reset();
	}

unsigned char Sab82526Sim::hscxReadISTA(){
	unsigned char	save = _readShadow.ISTA;
	_ISTA	&= (~save);
	updateChipInterrupt();
	return save;
	}

void Sab82526Sim::writeTxOctet(Octet data){
	_tx.writeByte(data);
	}

unsigned char Sab82526Sim::readRxOctet(unsigned int index){
	return _rx.readByte(index);
	}

unsigned char	Sab82526Sim::getRBCH(){
	unsigned lenHigh	= _rx.getLength();
	return (unsigned char)(lenHigh>>8);
	}

unsigned char	Sab82526Sim::getRBCL(){
	unsigned lenLow	= _rx.getLength();
	return (unsigned char)lenLow;
	}

void Sab82526Sim::startCommand(){
	unsigned char*		star	= (unsigned char *)&_readShadow.STAR;
	_commandTimer		= _commandPauseTicks;
	*star	|= STAR_CEC;
	}

void Sab82526Sim::receiveMessageComplete(void){
	_ISTA	|= (1<<ISTA_RME);
	updateChipInterrupt();
	}

void Sab82526Sim::receiveFifoFull(void){
	_ISTA	|= (1<<ISTA_RPF);
	updateChipInterrupt();
	}

void Sab82526Sim::updateAll(){
	updateChipInterrupt();
	}

void Sab82526Sim::updateChipInterrupt(){
	updateISTA();
	if(_readShadow.ISTA){
		setInterrupt();
		}
	else{
		clearInterrupt();
		}
	}

void Sab82526Sim::updateISTA(){
	Fields*	ista = (Fields *)&_readShadow.ISTA;
	updateIstaRME();
	updateIstaRPF();
	updateIstaRSC();
	updateIstaXPR();
	updateIstaTIN();
	updateIstaICA();
	updateIstaEXA();
	updateIstaEXB();
	*ista	= _ISTA & (~_writeShadow.MASK);
	}

void Sab82526Sim::updateIstaRME(void){
	}

void Sab82526Sim::updateIstaRPF(void){
	}

void Sab82526Sim::updateIstaRSC(void){
	}

void Sab82526Sim::updateIstaXPR(void){
	}

void Sab82526Sim::updateIstaTIN(void){
	}

void Sab82526Sim::updateIstaICA(void){
	}

void Sab82526Sim::updateIstaEXA(void){
	}

void Sab82526Sim::updateIstaEXB(void){
	}

void Sab82526Sim::transmitFifoEmpty(){
	_ISTA	|= (1<<ISTA_XPR);
	// a transmit FIFO is available
	// generate an interrupt
	}

void Sab82526Sim::open(){
	}

void Sab82526Sim::close(){
	}

