#ifndef _sim68k_devices_sab82526simh_
#define _sim68k_devices_sab82526simh_
#include "sab82526loc.h"
#include "sab82526tx.h"
#include "sab82526rx.h"
#include "device.h"
#include "oscl/hw/siemens/hscx.h"

class Sab82526Sim : public Device {
	private:	// Device data
		const long						_cyclesPerClock;
		long							_lastCycle;
		unsigned long					_cyclesRemainingFromLastPoll;
	private:	// Channel data
		long							_commandTimer;
		static const long				_commandPauseTicks	= 400;
		Sab82526Rx						_rx;
		Sab82526Tx						_tx;
		Oscl::Flags						_ISTA;
	public:		// Channel data
		volatile HscxWriteChannel		_writeShadow;
		volatile HscxReadChannel		_readShadow;
		volatile HscxReadWriteChannel	_rwShadow;
	private:	// Channel data
		Sab82526Loc00 _location00;
		Sab82526Loc01 _location01;
		Sab82526Loc02 _location02;
		Sab82526Loc03 _location03;
		Sab82526Loc04 _location04;
		Sab82526Loc05 _location05;
		Sab82526Loc06 _location06;
		Sab82526Loc07 _location07;
		Sab82526Loc08 _location08;
		Sab82526Loc09 _location09;
		Sab82526Loc0A _location0A;
		Sab82526Loc0B _location0B;
		Sab82526Loc0C _location0C;
		Sab82526Loc0D _location0D;
		Sab82526Loc0E _location0E;
		Sab82526Loc0F _location0F;
		Sab82526Loc10 _location10;
		Sab82526Loc11 _location11;
		Sab82526Loc12 _location12;
		Sab82526Loc13 _location13;
		Sab82526Loc14 _location14;
		Sab82526Loc15 _location15;
		Sab82526Loc16 _location16;
		Sab82526Loc17 _location17;
		Sab82526Loc18 _location18;
		Sab82526Loc19 _location19;
		Sab82526Loc1A _location1A;
		Sab82526Loc1B _location1B;
		Sab82526Loc1C _location1C;
		Sab82526Loc1D _location1D;
		Sab82526Loc1E _location1E;
		Sab82526Loc1F _location1F;
		Sab82526Loc20 _location20;
		Sab82526Loc21 _location21;
		Sab82526Loc22 _location22;
		Sab82526Loc23 _location23;
		Sab82526Loc24 _location24;
		Sab82526Loc25 _location25;
		Sab82526Loc26 _location26;
		Sab82526Loc27 _location27;
		Sab82526Loc28 _location28;
		Sab82526Loc29 _location29;
		Sab82526Loc2A _location2A;
		Sab82526Loc2B _location2B;
		Sab82526Loc2C _location2C;
		Sab82526Loc2D _location2D;
		Sab82526Loc2E _location2E;
		Sab82526Loc2F _location2F;
		Sab82526Loc30 _location30;
		Sab82526Loc31 _location31;
		Sab82526Loc32 _location32;
		Sab82526Loc33 _location33;
		Sab82526Loc34 _location34;
		Sab82526Loc35 _location35;
		Sab82526Loc36 _location36;
		Sab82526Loc37 _location37;
		Sab82526Loc38 _location38;
		Sab82526Loc39 _location39;
		Sab82526Loc3A _location3A;
		Sab82526Loc3B _location3B;
		Sab82526Loc3C _location3C;
		Sab82526Loc3D _location3D;
		Sab82526Loc3E _location3E;
		Sab82526Loc3F _location3F;
	public:	// Constructor
		Sab82526Sim(unsigned long cpc,int vector,int level,unsigned long baseAddress);
	public:	// Device interface
		virtual void	reset();
		virtual void	poll(unsigned long cyclesSinceLastPoll);
		virtual bool	hasAddress(unsigned long address);
		virtual bool	hasReadAddress(unsigned long address);
		virtual bool	hasWriteAddress(unsigned long address);
		virtual bool	hasByteAddress(unsigned long address);
		virtual bool	hasWordAddress(unsigned long address);
		virtual bool	hasLongAddress(unsigned long address);
		virtual unsigned char readByte(unsigned long address);
		virtual unsigned short readWord(unsigned long address);
		virtual unsigned long readLong(unsigned long address);
		virtual void writeByte(unsigned long address,unsigned char data);
		virtual void writeWord(unsigned long address,unsigned short data);
		virtual void writeLong(unsigned long address,unsigned long data);
		virtual void open();
		virtual void close();
	public:	// Memmory/Loc map interface
		void hscxReceiveMessageComplete();
		void hscxResetHdlcReceiver();
		void hscxReceiverNotReady();
		void hscxTransmissionRepeat();
		void hscxStartTimer();
		void hscxTransmitTransparentFrame();
		void hscxTransmitIFrame();
		void hscxTransmitMessageEnd();
		void hscxTransmitReset();
		unsigned char hscxReadISTA();
		void startCommand();
		void writeTxOctet(Oscl::Octet data);
		unsigned char readRxOctet(unsigned int index);
		unsigned char	getRBCH();
		unsigned char	getRBCL();
	public:	// Sab82526tx transmitter interface
		void transmitFifoEmpty();
	public:	// Sab82526Rx receiver interface
		void receiveFifoFull();
		void receiveMessageComplete();
	private:
		void updateAll();
		void updateChipInterrupt();
		void updateISTA();
		void updateIstaRME();
		void updateIstaRPF();
		void updateIstaRSC();
		void updateIstaXPR();
		void updateIstaTIN();
		void updateIstaICA();
		void updateIstaEXA();
		void updateIstaEXB();
	public:
		Sab82526Loc& location(unsigned long offset);
	private:
		bool	inRange(unsigned long address);
	};

#endif

