#include "sab82526sim.h"
#include "sab82526tx.h"

using namespace Oscl;

Sab82526Tx::Sab82526Tx(Sab82526Sim& sab):
		_sab(sab),
		_resetState(*this),
		_txFifo1(*this),
		_txFifo2(*this)
		{
	_state			= &_resetState;
	_rx				= 0;
	_currentFifo	= 0;
	_activeFifo		= 0;
	_enabled		= false;
	_txFifo1.reset();
	_txFifo2.reset();
	_enabled		= true;
	reset();
	}

Sab82526Tx::Sab82526Tx(Sab82526Sim& sab,FrameReceiver* rx):
		_sab(sab),
		_rx(rx),
		_resetState(*this),
		_txFifo1(*this),
		_txFifo2(*this)
		{
	_state			= &_resetState;
	_currentFifo	= 0;
	_activeFifo		= 0;
	_enabled		= false;
	_txFifo1.reset();
	_txFifo2.reset();
	_enabled		= true;
	changeReceiver(_rx);
	reset();
	}

void Sab82526Tx::reset(){
	Sab82526TxFifo*	savedCurrentFifo	= _currentFifo;
	Sab82526TxFifo*	savedActiveFifo		= _activeFifo;
	Sab82526TxFifo*	next;
	_enabled		= false;
	_currentFifo	= 0;
	_activeFifo		= 0;
	if(savedCurrentFifo) _activeList.put(savedCurrentFifo);
	if(savedActiveFifo) _activeList.put(savedActiveFifo);
	while((next=_emptyList.get())){
		_activeList.put(next);
		}
	while((next=_activeList.get())){
		next->reset();
		}
	_twoOctetAddress	= false;
	_state				= &_resetState;
	_state->reset();
	_enabled			= true;
	_frameInProgress	= false;
	}

void Sab82526Tx::clock(unsigned long bitClocksSinceLastClock){
	unsigned long	byteClocks;
	bitClocksSinceLastClock				+= _remainingClocksBeforeNextByteClock;
	byteClocks							= bitClocksSinceLastClock/8;
	_remainingClocksBeforeNextByteClock	= bitClocksSinceLastClock%8;
	for(;byteClocks;byteClocks--){
		if(_activeFifo) _activeFifo->byteClock();
		}
	}

void Sab82526Tx::enable(){
	if(!_enabled){
		_enabled			= true;
		_currentFifo		= _emptyList.get();
		if(_currentFifo)	_sab.transmitFifoEmpty();
		}
	}

void Sab82526Tx::disable(){
	Sab82526TxFifo*	next;
	_enabled	= false;
	if(_currentFifo){
		_currentFifo->reset();
		_currentFifo	= 0;
		}
	if(_activeFifo){
		_activeFifo->reset();
		_activeFifo	= 0;
		}
	while((next=_activeList.get())){
		next->reset();
		}
	}

void Sab82526Tx::writeByte(unsigned char data){
	if(_currentFifo) _currentFifo->put((Octet)data);
	}

void Sab82526Tx::transmitContentsOfFifo(){
	if(_currentFifo){
		if(!_frameInProgress){
			_frameInProgress	= true;
			if(		( (_sab._rwShadow.MODE & MODE_MDS_MASK) == MODE_MDS_AUTO)
				||	( (_sab._rwShadow.MODE & MODE_MDS_MASK) == MODE_MDS_NON_AUTO)
					){
				if((_sab._rwShadow.MODE & (1<<MODE_ADM)) == (1<<MODE_ADM)){
					_currentFifo->newFrame((Octet)_sab._writeShadow.XAD1,(Octet)_sab._writeShadow.XAD2);
					}
				else{
					_currentFifo->newFrame((Octet)_sab._writeShadow.XAD1);
					}
				}
			else{
				_currentFifo->newFrame();
				}
			}
		if(_currentFifo->isLastFifoInFrame()){
			_frameInProgress	= false;
			}
		if(!_activeFifo){
			_activeFifo	= _currentFifo;
			}
		else{
			_activeList.put(_currentFifo);
			}
		_currentFifo		= _emptyList.get();
		if(_currentFifo)	_sab.transmitFifoEmpty();
		}
	}

void Sab82526Tx::endOfFrame(){
	if(_currentFifo) _currentFifo->endOfFrame();
	}

FrameReceiver* Sab82526Tx::changeReceiver(FrameReceiver* rx){
	_txFifo1.changeReceiver(rx);
	return _txFifo2.changeReceiver(rx);
	}

// Sab82526TxFifoOwner Interface

void Sab82526Tx::txFifoEmpty(Sab82526TxFifo* theFifo){
	if(!_currentFifo){
		_currentFifo	= theFifo;
		_sab.transmitFifoEmpty();
		}
	else{
		_emptyList.put(theFifo);
		}
	if(_enabled){
		_activeFifo		= _activeList.get();
		}
	}

/////////////////////////// Sab82526TxState ///////////////

Sab82526TxState::Sab82526TxState(Sab82526Tx& tx):_tx(tx){
	}

/////////////////////////// Sab82526TxStateReset ///////////////

Sab82526TxStateReset::Sab82526TxStateReset(Sab82526Tx& tx):Sab82526TxState(tx){
			}

void Sab82526TxStateReset::reset(){
	// no change in state
	}

void Sab82526TxStateReset::poll(unsigned long cyclesSinceLastPoll){
	// nothing to do
	}

void Sab82526TxStateReset::enable(){
	//_tx.state	= &_tx.enableState;
	//_tx.enableState->reset();
	}

void Sab82526TxStateReset::disable(){
	//_tx.state	= &_tx.disableState;
	}

void Sab82526TxStateReset::enableInterrupt(){
	//_tx.state	= &_tx.disableState;
	}

void Sab82526TxStateReset::disableInterrupt(){
	}

void Sab82526TxStateReset::writeByte(unsigned char data){
	}

void Sab82526TxStateReset::transmitContentsOfFifo(){
	}

void Sab82526TxStateReset::endOfFrame(){
	}

// Transmit FIFO

Sab82526TxFifo::Sab82526TxFifo(Sab82526TxFifoOwner& owner,FrameReceiver* receiver):
								_resetState(*this),_owner(owner),_receiver(receiver){
	_state	= &_resetState;
	};

Sab82526TxFifo::Sab82526TxFifo(Sab82526TxFifoOwner& owner):
								_resetState(*this),_owner(owner),_receiver(0){
	_state	= &_resetState;
	};

FrameReceiver* Sab82526TxFifo::changeReceiver(FrameReceiver* receiver){
	FrameReceiver*	rx;
	rx			= _receiver;
	_receiver	= receiver;
	return rx;
	}

void Sab82526TxFifo::reset(){
	_remainingClocksBeforeNextTransmit	=0;
	_octetsToTransmit		= 0;
	_currentOctet			= 0;
	_srcAddress[0]			= 0;	// default
	_srcAddress[1]			= 0;	// default
	_transmitEndOfFrame		= false;
	_transmitStartOfFrame	= false;
	_owner.txFifoEmpty(this);
	}

void Sab82526TxFifo::byteClock(){
	if(_transmitStartOfFrame){
		if(_receiver) _receiver->receiveFlag();
		_transmitStartOfFrame	= false;
		_transmitSourceAddress	= (_nAddressOctets)?true:false;
		_nextSrcAddress			= 0;
		}
	else if(_transmitSourceAddress){
		if(_receiver) _receiver->receive(_srcAddress[_nextSrcAddress++]);
		if(_nextSrcAddress >= _nAddressOctets){
			_transmitSourceAddress	= false;
			}
		}
	else if(_octetsToTransmit){
		if(_receiver) _receiver->receive(_fifo[_currentOctet]);
		_currentOctet++;
		_octetsToTransmit--;
		_crcOctetsToTransmit	= 2;
		}
	else if(_transmitEndOfFrame){
		if(_crcOctetsToTransmit){
			_crcOctetsToTransmit--;
			if(_receiver) _receiver->receive(0xFF);	// bogus CRC since loop receiver doesn't check
			}
		else{
			if(_receiver) _receiver->receiveFlag();
			_transmitEndOfFrame	= false;
			}
		}
	// If all done, then
	if((!_transmitStartOfFrame) && (!_transmitSourceAddress) && (!_octetsToTransmit) && (!_transmitEndOfFrame)){
		_currentOctet	= 0;
		_owner.txFifoEmpty(this);
		}
	}

void Sab82526TxFifo::put(Octet data){
	if(_octetsToTransmit<_fifoSize){
		_fifo[_octetsToTransmit]	= data;
		_octetsToTransmit++;
		}
	}

void Sab82526TxFifo::newFrame(){
	_nAddressOctets	= 0;
	_transmitStartOfFrame	= true;
	}

void Sab82526TxFifo::newFrame(Octet srcAddress){
	_nAddressOctets	= 1;
	_transmitStartOfFrame	= true;
	}

void Sab82526TxFifo::newFrame(Octet firstOctetOfAddress,Octet secondOctetOfAddress){
	_nAddressOctets	= 2;
	_nextSrcAddress	= 0;
	_srcAddress[0]	= firstOctetOfAddress;
	_srcAddress[1]	= secondOctetOfAddress;
	_transmitStartOfFrame	= true;
	}

void Sab82526TxFifo::endOfFrame(){
	_transmitEndOfFrame	= true;
	}

bool Sab82526TxFifo::isLastFifoInFrame(){
	return _transmitEndOfFrame;
	}

// Sab82526TxFifoState
Sab82526TxFifoState::Sab82526TxFifoState(Sab82526TxFifo& fifo):_fifo(fifo){
	}

// Sab82526TxFifoStateReset

Sab82526TxFifoStateReset::Sab82526TxFifoStateReset(Sab82526TxFifo& fifo):Sab82526TxFifoState(fifo){
	}

void Sab82526TxFifoStateReset::clock(unsigned long bitClocksSinceLastClock){
	}

