#ifndef _sim68k_devices_sab82526txh_
#define _sim68k_devices_sab82526txh_
#include "framerx.h"
#include "oscl/queue/queue.h"

class Sab82526Sim;
class Sab82526TxFifo;
class Sab82526Tx;

class Sab82526TxFifoOwner{
	public:
		virtual void txFifoEmpty(Sab82526TxFifo* theFifo)=0;
	};

class Sab82526TxFifoState{
	protected:
		Sab82526TxFifo&	_fifo;
	public:
		Sab82526TxFifoState(Sab82526TxFifo& fifo);
		virtual void clock(unsigned long bitClocksSinceLastClock)=0;
	};

class Sab82526TxFifoStateReset:public Sab82526TxFifoState{
	public:
		Sab82526TxFifoStateReset(Sab82526TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab82526TxFifoStateEmpty:public Sab82526TxFifoState{
	public:
		Sab82526TxFifoStateEmpty(Sab82526TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab82526TxFifoStateNewFrame:public Sab82526TxFifoState{
	public:
		Sab82526TxFifoStateNewFrame(Sab82526TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab82526TxFifoStateInfo:public Sab82526TxFifoState{
	public:
		Sab82526TxFifoStateInfo(Sab82526TxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab82526TxFifo: public Oscl::QueueItem {
	private:
		Sab82526TxFifoStateReset	_resetState;
		Sab82526TxFifoState*		_state;
		Sab82526TxFifoOwner&		_owner;
		static const unsigned int	_fifoSize = 32;
		FrameReceiver*				_receiver;
		unsigned int				_octetsToTransmit;
		Oscl::Octet					_srcAddress[2];
		unsigned int				_nextSrcAddress;
		unsigned int				_nAddressOctets;
		Oscl::Octet					_fifo[_fifoSize];
		unsigned int				_currentOctet;
		unsigned int				_remainingClocksBeforeNextTransmit;
		unsigned int				_crcOctetsToTransmit;
		bool						_transmitStartOfFrame;
		bool						_transmitSourceAddress;
		bool						_transmitEndOfFrame;
	public:	// Transmitter interface
		Sab82526TxFifo(Sab82526TxFifoOwner& owner,FrameReceiver* receiver);
		Sab82526TxFifo(Sab82526TxFifoOwner& owner);
		FrameReceiver* changeReceiver(FrameReceiver* receiver);
		void reset();
		void byteClock();
		void put(Oscl::Octet data);
		void newFrame();
		void newFrame(Oscl::Octet srcAddress);
		void newFrame(Oscl::Octet firstOctetOfAddress,Oscl::Octet secondOctetOfAddress);
		void endOfFrame();
		bool isLastFifoInFrame();
	};

class Sab82526TxState{
	protected:
		Sab82526Tx&	_tx;
	public:
		Sab82526TxState(Sab82526Tx& tx);
		virtual void reset() = 0;
		virtual void poll(unsigned long cyclesSinceLastPoll) = 0;
		virtual void enable() = 0;
		virtual void disable() = 0;
		virtual void enableInterrupt() = 0;
		virtual void disableInterrupt() = 0;
		virtual void writeByte(unsigned char data) = 0;
		virtual void transmitContentsOfFifo() = 0;
		virtual void endOfFrame() = 0;
	};

class Sab82526TxStateReset : public Sab82526TxState{
	public:
		Sab82526TxStateReset(Sab82526Tx& tx);
		void reset();
		void poll(unsigned long cyclesSinceLastPoll);
		void enable();
		void disable();
		void enableInterrupt();
		void disableInterrupt();
		void writeByte(unsigned char data);
		void transmitContentsOfFifo();
		void endOfFrame();
	};

class Sab82526Tx:public Sab82526TxFifoOwner{
	protected:
		Sab82526Sim&		_sab;
		FrameReceiver*		_rx;
	private:
		Oscl::Queue<class Sab82526TxFifo>	_emptyList;
		Oscl::Queue<class Sab82526TxFifo>	_activeList;
		Sab82526TxStateReset		_resetState;
		Sab82526TxState*			_state;
		Sab82526TxFifo				_txFifo1;
		Sab82526TxFifo				_txFifo2;
		Sab82526TxFifo*				_currentFifo;
		Sab82526TxFifo*				_activeFifo;
		bool						_enabled;
		bool						_frameInProgress;
		bool						_twoOctetAddress;
		unsigned long				_remainingClocksBeforeNextByteClock;
	public:
		Sab82526Tx(Sab82526Sim& sab);
		Sab82526Tx(Sab82526Sim& sab,FrameReceiver* rx);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void writeByte(unsigned char data);
		void transmitContentsOfFifo();
		void endOfFrame();
		FrameReceiver* changeReceiver(FrameReceiver* rx);	// returns old receiver

		// Sab82526TxFifoOwner Interface
		void txFifoEmpty(Sab82526TxFifo* theFifo);
	};

#endif
