#ifndef _sim68k_devices_sab8252xrxh_
#define _sim68k_devices_sab8252xrxh_
#include "framerx.h"
#include "oscl/queue/queue.h"

class Sab8252xSim;
class Sab8252xRxState;
class Sab8252xRx;
class Sab8252xRxFifo;

class Sab8252xRxFifoOwner{
	public:
		virtual void fifoFull(Sab8252xRxFifo* fifo)=0;
		virtual void endOfFrame(Sab8252xRxFifo* fifo)=0;
	};

class Sab8252xRxFifo : public Oscl::QueueItem {
	private:
		static const unsigned int	_fifoSize = 32;
		Oscl::Octet					_fifo[_fifoSize];
		Sab8252xRxFifoOwner&		_owner;
		unsigned int				_nextOctet;
		unsigned int				_length;
		bool						_endOfFrame;
	public:
		Sab8252xRxFifo(Sab8252xRxFifoOwner& owner);
		void			reset();
		unsigned char	read();
		unsigned short	getLength();
		// input from line
		void			put(unsigned char data);
		void			endOfFrame();
		// Receiver interface
		bool			isEndOfFrame();
	};

class Sab8252xRxState{
	protected:
		Sab8252xRx&	_rx;
	public:
		Sab8252xRxState(Sab8252xRx& rx);
		void reset();
		virtual void flag()=0;
		virtual void receive(Oscl::Octet data)=0;
	};

class Sab8252xRxStateHunt: public Sab8252xRxState{
	public:
		Sab8252xRxStateHunt(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxStateMds01Adm0Stage1: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm0Stage1(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxStateMds01Adm0Stage2: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm0Stage2(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxStateMds01Adm0Stage3: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm0Stage3(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxStateMds01Adm0Stage4: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm0Stage4(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};


class Sab8252xRxStateMds01Adm1Stage1: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm1Stage1(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxStateMds01Adm1Stage2: public Sab8252xRxState{
	public:
		Sab8252xRxStateMds01Adm1Stage2(Sab8252xRx& rx);
		void flag();
		void receive(Oscl::Octet data);
	};

class Sab8252xRxDataNode : public Oscl::QueueItem {
	public:
		Oscl::Octet	_data;
	};

class Sab8252xRx: public FrameReceiver,Sab8252xRxFifoOwner{
	protected:
		Sab8252xSim&			_sab;
	private:
		Sab8252xRxStateHunt				_huntState;
		Sab8252xRxStateMds01Adm0Stage1	_mds01Adm0Stage1State;
		Sab8252xRxStateMds01Adm0Stage2	_mds01Adm0Stage2State;
		Sab8252xRxStateMds01Adm0Stage3	_mds01Adm0Stage3State;
		Sab8252xRxStateMds01Adm0Stage4	_mds01Adm0Stage4State;
		Sab8252xRxStateMds01Adm1Stage1	_mds01Adm1Stage1State;
		Sab8252xRxStateMds01Adm1Stage2	_mds01Adm1Stage2State;
		Sab8252xRxState*				_state;
		Sab8252xRxFifo*					_currentFifo;
		Sab8252xRxFifo*					_activeFifo;
		Sab8252xRxFifo					_rxFifo1;
		Sab8252xRxFifo					_rxFifo2;
		unsigned short					_rxLength;
		unsigned char					_rsta;
		Oscl::Queue<Sab8252xRxDataNode>		_delayQ;
		Oscl::Queue<Sab8252xRxDataNode>		_freeList;
		Oscl::Queue<Sab8252xRxFifo>			_pendingList;
		Oscl::Queue<Sab8252xRxFifo>			_emptyList;
		Sab8252xRxDataNode				_delayNode1;
		Sab8252xRxDataNode				_delayNode2;
	public:
		Sab8252xRx(Sab8252xSim& sab);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void enableInterrupt();
		void disableInterrupt();
		unsigned char readByte(unsigned short index);
		void releaseFifo();
		unsigned short	getLength();
	// FrameReceiver Interface
		void receiveFlag();
		void receive(Oscl::Octet data);
	// Sab8252xRxFifoOwner Interface
		void fifoFull(Sab8252xRxFifo* fifo);
		void endOfFrame(Sab8252xRxFifo* fifo);
	public: // RxState state change interface
		void setStartOfFrameState();
		void setRxStateHunt();
		void setRxStateMds01Adm0Stage1();
		void setRxStateMds01Adm0Stage2();
		void setRxStateMds01Adm0Stage3();
		void setRxStateMds01Adm0Stage4();
		void setRxStateMds01Adm1Stage1();
		void setRxStateMds01Adm1Stage2();
	public: // RxState interface
		bool				isMyAddressH(Oscl::Octet data);	// true if data is same as RAH1 or RAH2
		bool				isMyAddressL(Oscl::Octet data);	// true if data is same as RAL1 or RAL2
		void				resetCRC();					// reset the CRC accumulator
		void				accumulateCRC(Oscl::Octet data);	// accumulate data into current CRC
		void				runtFrameReceived();
		Sab8252xRxDataNode*	getFreeNode();
		void				putNodeInDelayQueue(Sab8252xRxDataNode* node);
		void				frameComplete();			// check CRC,put RSTA copy in FIFO indicate EOF to FIFO
		void				flushDelayQueue();
		Sab8252xRxDataNode*	getDelayQueue();
		void				putInActiveFifo(Oscl::Octet data);
		void				receiveOverflow();
		bool				fifoActive();
	};

#endif
