#ifndef _sim68k_devices_sab8252xsimh_
#define _sim68k_devices_sab8252xsimh_
#include "sab8252xloc.h"
#include "sab8252xtx.h"
#include "sab8252xrx.h"
#include "oscl/hw/siemens/hscx.h"

/** This class defines the interface used by the Sab8252xSim
	to stimulate and receive status from the chip as a whole,
	and from the opposite channel in the case of channel A
	on the Sab82525.
 */

class Sab8252xDevice {
	public:
		virtual bool	oppositeChannelInterruptPending()=0;
		virtual bool	oppositeChannelExtendedInterruptPending()=0;
		virtual bool	oppositeChannelRawExtendedInterruptPending()=0;
		virtual bool	thisChannelRawExtendedInterruptPending()=0;
	public:
		virtual bool	oppositeChannelExtendedInterrruptMasked()=0;
		virtual void	opositeChannelInterruptStatusChanged()=0;
	public:
		virtual void	updateChipInterrupt()=0;
	};

class Sab8252xSim {
	private:	// Channel data
		Sab8252xDevice&					_device;
		long							_commandTimer;
		static const long				_commandPauseTicks	= 400;
		Sab8252xRx						_rx;
		Sab8252xTx						_tx;
		Oscl::Flags						_ISTA;
		Oscl::Flags						_EXIR;
		bool							_channelAInterruptPending;
		bool							_channelAExtendedInterruptPending;
	public:		// Channel data
		volatile HscxWriteChannel		_writeShadow;
		volatile HscxReadChannel		_readShadow;
		volatile HscxReadWriteChannel	_rwShadow;
	private:	// Channel data
		Sab8252xLoc00 _location00;
		Sab8252xLoc01 _location01;
		Sab8252xLoc02 _location02;
		Sab8252xLoc03 _location03;
		Sab8252xLoc04 _location04;
		Sab8252xLoc05 _location05;
		Sab8252xLoc06 _location06;
		Sab8252xLoc07 _location07;
		Sab8252xLoc08 _location08;
		Sab8252xLoc09 _location09;
		Sab8252xLoc0A _location0A;
		Sab8252xLoc0B _location0B;
		Sab8252xLoc0C _location0C;
		Sab8252xLoc0D _location0D;
		Sab8252xLoc0E _location0E;
		Sab8252xLoc0F _location0F;
		Sab8252xLoc10 _location10;
		Sab8252xLoc11 _location11;
		Sab8252xLoc12 _location12;
		Sab8252xLoc13 _location13;
		Sab8252xLoc14 _location14;
		Sab8252xLoc15 _location15;
		Sab8252xLoc16 _location16;
		Sab8252xLoc17 _location17;
		Sab8252xLoc18 _location18;
		Sab8252xLoc19 _location19;
		Sab8252xLoc1A _location1A;
		Sab8252xLoc1B _location1B;
		Sab8252xLoc1C _location1C;
		Sab8252xLoc1D _location1D;
		Sab8252xLoc1E _location1E;
		Sab8252xLoc1F _location1F;
		Sab8252xLoc20 _location20;
		Sab8252xLoc21 _location21;
		Sab8252xLoc22 _location22;
		Sab8252xLoc23 _location23;
		Sab8252xLoc24 _location24;
		Sab8252xLoc25 _location25;
		Sab8252xLoc26 _location26;
		Sab8252xLoc27 _location27;
		Sab8252xLoc28 _location28;
		Sab8252xLoc29 _location29;
		Sab8252xLoc2A _location2A;
		Sab8252xLoc2B _location2B;
		Sab8252xLoc2C _location2C;
		Sab8252xLoc2D _location2D;
		Sab8252xLoc2E _location2E;
		Sab8252xLoc2F _location2F;
		Sab8252xLoc30 _location30;
		Sab8252xLoc31 _location31;
		Sab8252xLoc32 _location32;
		Sab8252xLoc33 _location33;
		Sab8252xLoc34 _location34;
		Sab8252xLoc35 _location35;
		Sab8252xLoc36 _location36;
		Sab8252xLoc37 _location37;
		Sab8252xLoc38 _location38;
		Sab8252xLoc39 _location39;
		Sab8252xLoc3A _location3A;
		Sab8252xLoc3B _location3B;
		Sab8252xLoc3C _location3C;
		Sab8252xLoc3D _location3D;
		Sab8252xLoc3E _location3E;
		Sab8252xLoc3F _location3F;
	public:	// Constructor
		Sab8252xSim(Sab8252xDevice& device);
	public:	// Sub-Device interface
		void			reset();
		void			poll(unsigned long clocksThisPoll);
		unsigned char	readByte(unsigned long offset);
		void			writeByte(unsigned long offset,unsigned char data);
		void			channelAInterruptPending();
		void			channelAExtendedInterruptPending();
		void			open();
		void			close();
	public:	// Sub-Device control interface
		bool			channelInterruptIsPending();
		bool			extendedInterruptIsPending();
		bool			extendedInterruptIsMasked();
		bool			rawExtendedInterruptIsPending();
		void			oppositeChannelInterruptStatusChanged();
	public:	// Memmory/Loc map interface
		void			hscxReceiveMessageComplete();
		void			hscxResetHdlcReceiver();
		void			hscxReceiverNotReady();
		void			hscxTransmissionRepeat();
		void			hscxStartTimer();
		void			hscxTransmitTransparentFrame();
		void			hscxTransmitIFrame();
		void			hscxTransmitMessageEnd();
		void			hscxTransmitReset();
		unsigned char	hscxReadISTA();
		unsigned char	hscxReadEXIR();
		void			hscxWriteMask(unsigned char mask);
		void			startCommand();
		void			writeTxOctet(Oscl::Octet data);
		unsigned char	readRxOctet(unsigned int index);
		unsigned char	getRBCH();
		unsigned char	getRBCL();
	public:	// Sab8252xtx transmitter interface
		void			transmitFifoEmpty();
	public:	// Sab8252xRx receiver interface
		void			receiveFifoFull();
		void			receiveMessageComplete();
	private:
		void updateAll();
		void updateChipInterrupt();
		void updateISTA();
		void updateIstaRME();
		void updateIstaRPF();
		void updateIstaRSC();
		void updateIstaXPR();
		void updateIstaTIN();
		void updateIstaICA();
		void updateIstaEXA();
		void updateIstaEXB();
	public:
		Sab8252xLoc& location(unsigned long offset);
	private:
		bool	inRange(unsigned long address);
	};

#endif
