#include "sab8252xsim.h"
#include "sab8252xtx.h"

using namespace Oscl;

Sab8252xTx::Sab8252xTx(Sab8252xSim& sab):
		_sab(sab),
		_resetState(*this),
		_txFifo1(*this),
		_txFifo2(*this)
		{
	_state			= &_resetState;
	_rx				= 0;
	_currentFifo	= 0;
	_activeFifo		= 0;
	_enabled		= false;
	_txFifo1.reset();
	_txFifo2.reset();
	_enabled		= true;
	reset();
	}

Sab8252xTx::Sab8252xTx(Sab8252xSim& sab,FrameReceiver* rx):
		_sab(sab),
		_rx(rx),
		_resetState(*this),
		_txFifo1(*this),
		_txFifo2(*this)
		{
	_state			= &_resetState;
	_currentFifo	= 0;
	_activeFifo		= 0;
	_enabled		= false;
	_txFifo1.reset();
	_txFifo2.reset();
	_enabled		= true;
	changeReceiver(_rx);
	reset();
	}

void Sab8252xTx::reset(){
	Sab8252xTxFifo*	savedCurrentFifo	= _currentFifo;
	Sab8252xTxFifo*	savedActiveFifo		= _activeFifo;
	Sab8252xTxFifo*	next;
	_enabled		= false;
	_currentFifo	= 0;
	_activeFifo		= 0;
	if(savedCurrentFifo) _activeList.put(savedCurrentFifo);
	if(savedActiveFifo) _activeList.put(savedActiveFifo);
	while((next=_emptyList.get())){
		_activeList.put(next);
		}
	while((next=_activeList.get())){
		next->reset();
		}
	_twoOctetAddress	= false;
	_state				= &_resetState;
	_state->reset();
	_enabled			= true;
	_frameInProgress	= false;
	}

void Sab8252xTx::clock(unsigned long bitClocksSinceLastClock){
	unsigned long	byteClocks;
	bitClocksSinceLastClock				+= _remainingClocksBeforeNextByteClock;
	byteClocks							= bitClocksSinceLastClock/8;
	_remainingClocksBeforeNextByteClock	= bitClocksSinceLastClock%8;
	for(;byteClocks;byteClocks--){
		if(_activeFifo) _activeFifo->byteClock();
		}
	}

void Sab8252xTx::enable(){
	if(!_enabled){
		_enabled			= true;
		_currentFifo		= _emptyList.get();
		if(_currentFifo)	_sab.transmitFifoEmpty();
		}
	}

void Sab8252xTx::disable(){
	Sab8252xTxFifo*	next;
	_enabled	= false;
	if(_currentFifo){
		_currentFifo->reset();
		_currentFifo	= 0;
		}
	if(_activeFifo){
		_activeFifo->reset();
		_activeFifo	= 0;
		}
	while((next=_activeList.get())){
		next->reset();
		}
	}

void Sab8252xTx::writeByte(unsigned char data){
	if(_currentFifo) _currentFifo->put((Octet)data);
	}

void Sab8252xTx::transmitContentsOfFifo(){
	if(_currentFifo){
		if(!_frameInProgress){
			_frameInProgress	= true;
			if(		( (_sab._rwShadow.MODE & MODE_MDS_MASK) == MODE_MDS_AUTO)
				||	( (_sab._rwShadow.MODE & MODE_MDS_MASK) == MODE_MDS_NON_AUTO)
					){
				if((_sab._rwShadow.MODE & (1<<MODE_ADM)) == (1<<MODE_ADM)){
					_currentFifo->newFrame((Octet)_sab._writeShadow.XAD1,(Octet)_sab._writeShadow.XAD2);
					}
				else{
					_currentFifo->newFrame((Octet)_sab._writeShadow.XAD1);
					}
				}
			else{
				_currentFifo->newFrame();
				}
			}
		if(_currentFifo->isLastFifoInFrame()){
			_frameInProgress	= false;
			}
		if(!_activeFifo){
			_activeFifo	= _currentFifo;
			}
		else{
			_activeList.put(_currentFifo);
			}
		_currentFifo		= _emptyList.get();
		if(_currentFifo)	_sab.transmitFifoEmpty();
		}
	}

void Sab8252xTx::endOfFrame(){
	if(_currentFifo) _currentFifo->endOfFrame();
	}

FrameReceiver* Sab8252xTx::changeReceiver(FrameReceiver* rx){
	_txFifo1.changeReceiver(rx);
	return _txFifo2.changeReceiver(rx);
	}

// Sab8252xTxFifoOwner Interface

void Sab8252xTx::txFifoEmpty(Sab8252xTxFifo* theFifo){
	if(!_currentFifo){
		_currentFifo	= theFifo;
		_sab.transmitFifoEmpty();
		}
	else{
		_emptyList.put(theFifo);
		}
	if(_enabled){
		_activeFifo		= _activeList.get();
		}
	}

/////////////////////////// Sab8252xTxState ///////////////

Sab8252xTxState::Sab8252xTxState(Sab8252xTx& tx):_tx(tx){
	}

/////////////////////////// Sab8252xTxStateReset ///////////////

Sab8252xTxStateReset::Sab8252xTxStateReset(Sab8252xTx& tx):Sab8252xTxState(tx){
			}

void Sab8252xTxStateReset::reset(){
	// no change in state
	}

void Sab8252xTxStateReset::poll(unsigned long cyclesSinceLastPoll){
	// nothing to do
	}

void Sab8252xTxStateReset::enable(){
	//_tx.state	= &_tx.enableState;
	//_tx.enableState->reset();
	}

void Sab8252xTxStateReset::disable(){
	//_tx.state	= &_tx.disableState;
	}

void Sab8252xTxStateReset::enableInterrupt(){
	//_tx.state	= &_tx.disableState;
	}

void Sab8252xTxStateReset::disableInterrupt(){
	}

void Sab8252xTxStateReset::writeByte(unsigned char data){
	}

void Sab8252xTxStateReset::transmitContentsOfFifo(){
	}

void Sab8252xTxStateReset::endOfFrame(){
	}

// Transmit FIFO

Sab8252xTxFifo::Sab8252xTxFifo(Sab8252xTxFifoOwner& owner,FrameReceiver* receiver):
								_resetState(*this),_owner(owner),_receiver(receiver){
	_state	= &_resetState;
	};

Sab8252xTxFifo::Sab8252xTxFifo(Sab8252xTxFifoOwner& owner):
								_resetState(*this),_owner(owner),_receiver(0){
	_state	= &_resetState;
	};

FrameReceiver* Sab8252xTxFifo::changeReceiver(FrameReceiver* receiver){
	FrameReceiver*	rx;
	rx			= _receiver;
	_receiver	= receiver;
	return rx;
	}

void Sab8252xTxFifo::reset(){
	_remainingClocksBeforeNextTransmit	=0;
	_octetsToTransmit		= 0;
	_currentOctet			= 0;
	_srcAddress[0]			= 0;	// default
	_srcAddress[1]			= 0;	// default
	_transmitEndOfFrame		= false;
	_transmitStartOfFrame	= false;
	_owner.txFifoEmpty(this);
	}

void Sab8252xTxFifo::byteClock(){
	if(_transmitStartOfFrame){
		if(_receiver) _receiver->receiveFlag();
		_transmitStartOfFrame	= false;
		_transmitSourceAddress	= (_nAddressOctets)?true:false;
		_nextSrcAddress			= 0;
		}
	else if(_transmitSourceAddress){
		if(_receiver) _receiver->receive(_srcAddress[_nextSrcAddress++]);
		if(_nextSrcAddress >= _nAddressOctets){
			_transmitSourceAddress	= false;
			}
		}
	else if(_octetsToTransmit){
		if(_receiver) _receiver->receive(_fifo[_currentOctet]);
		_currentOctet++;
		_octetsToTransmit--;
		_crcOctetsToTransmit	= 2;
		}
	else if(_transmitEndOfFrame){
		if(_crcOctetsToTransmit){
			_crcOctetsToTransmit--;
			if(_receiver) _receiver->receive(0xFF);	// bogus CRC since loop receiver doesn't check
			}
		else{
			if(_receiver) _receiver->receiveFlag();
			_transmitEndOfFrame	= false;
			}
		}
	// If all done, then
	if((!_transmitStartOfFrame) && (!_transmitSourceAddress) && (!_octetsToTransmit) && (!_transmitEndOfFrame)){
		_currentOctet	= 0;
		_owner.txFifoEmpty(this);
		}
	}

void Sab8252xTxFifo::put(Octet data){
	if(_octetsToTransmit<_fifoSize){
		_fifo[_octetsToTransmit]	= data;
		_octetsToTransmit++;
		}
	}

void Sab8252xTxFifo::newFrame(){
	_nAddressOctets	= 0;
	_transmitStartOfFrame	= true;
	}

void Sab8252xTxFifo::newFrame(Octet srcAddress){
	_nAddressOctets	= 1;
	_transmitStartOfFrame	= true;
	}

void Sab8252xTxFifo::newFrame(Octet firstOctetOfAddress,Octet secondOctetOfAddress){
	_nAddressOctets	= 2;
	_nextSrcAddress	= 0;
	_srcAddress[0]	= firstOctetOfAddress;
	_srcAddress[1]	= secondOctetOfAddress;
	_transmitStartOfFrame	= true;
	}

void Sab8252xTxFifo::endOfFrame(){
	_transmitEndOfFrame	= true;
	}

bool Sab8252xTxFifo::isLastFifoInFrame(){
	return _transmitEndOfFrame;
	}

// Sab8252xTxFifoState
Sab8252xTxFifoState::Sab8252xTxFifoState(Sab8252xTxFifo& fifo):_fifo(fifo){
	}

// Sab8252xTxFifoStateReset

Sab8252xTxFifoStateReset::Sab8252xTxFifoStateReset(Sab8252xTxFifo& fifo):Sab8252xTxFifoState(fifo){
	}

void Sab8252xTxFifoStateReset::clock(unsigned long bitClocksSinceLastClock){
	}

