#ifndef __sim68k_devices_sab8252xtxh_
#define __sim68k_devices_sab8252xtxh_
#include "framerx.h"
#include "oscl/queue/queue.h"

class Sab8252xSim;
class Sab8252xTxFifo;
class Sab8252xTx;

class Sab8252xTxFifoOwner{
	public:
		virtual void txFifoEmpty(Sab8252xTxFifo* theFifo)=0;
	};

class Sab8252xTxFifoState{
	protected:
		Sab8252xTxFifo&	_fifo;
	public:
		Sab8252xTxFifoState(Sab8252xTxFifo& fifo);
		virtual void clock(unsigned long bitClocksSinceLastClock)=0;
	};

class Sab8252xTxFifoStateReset:public Sab8252xTxFifoState{
	public:
		Sab8252xTxFifoStateReset(Sab8252xTxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab8252xTxFifoStateEmpty:public Sab8252xTxFifoState{
	public:
		Sab8252xTxFifoStateEmpty(Sab8252xTxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab8252xTxFifoStateNewFrame:public Sab8252xTxFifoState{
	public:
		Sab8252xTxFifoStateNewFrame(Sab8252xTxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab8252xTxFifoStateInfo:public Sab8252xTxFifoState{
	public:
		Sab8252xTxFifoStateInfo(Sab8252xTxFifo& fifo);
		void clock(unsigned long bitClocksSinceLastClock);
	};

class Sab8252xTxFifo: public Oscl::QueueItem {
	private:
		Sab8252xTxFifoStateReset	_resetState;
		Sab8252xTxFifoState*		_state;
		Sab8252xTxFifoOwner&		_owner;
		static const unsigned int	_fifoSize = 32;
		FrameReceiver*				_receiver;
		unsigned int				_octetsToTransmit;
		Oscl::Octet					_srcAddress[2];
		unsigned int				_nextSrcAddress;
		unsigned int				_nAddressOctets;
		Oscl::Octet					_fifo[_fifoSize];
		unsigned int				_currentOctet;
		unsigned int				_remainingClocksBeforeNextTransmit;
		unsigned int				_crcOctetsToTransmit;
		bool						_transmitStartOfFrame;
		bool						_transmitSourceAddress;
		bool						_transmitEndOfFrame;
	public:	// Transmitter interface
		Sab8252xTxFifo(Sab8252xTxFifoOwner& owner,FrameReceiver* receiver);
		Sab8252xTxFifo(Sab8252xTxFifoOwner& owner);
		FrameReceiver* changeReceiver(FrameReceiver* receiver);
		void reset();
		void byteClock();
		void put(Oscl::Octet data);
		void newFrame();
		void newFrame(Oscl::Octet srcAddress);
		void newFrame(Oscl::Octet firstOctetOfAddress,Oscl::Octet secondOctetOfAddress);
		void endOfFrame();
		bool isLastFifoInFrame();
	};

class Sab8252xTxState{
	protected:
		Sab8252xTx&	_tx;
	public:
		Sab8252xTxState(Sab8252xTx& tx);
		virtual void reset() = 0;
		virtual void poll(unsigned long cyclesSinceLastPoll) = 0;
		virtual void enable() = 0;
		virtual void disable() = 0;
		virtual void enableInterrupt() = 0;
		virtual void disableInterrupt() = 0;
		virtual void writeByte(unsigned char data) = 0;
		virtual void transmitContentsOfFifo() = 0;
		virtual void endOfFrame() = 0;
	};

class Sab8252xTxStateReset : public Sab8252xTxState{
	public:
		Sab8252xTxStateReset(Sab8252xTx& tx);
		void reset();
		void poll(unsigned long cyclesSinceLastPoll);
		void enable();
		void disable();
		void enableInterrupt();
		void disableInterrupt();
		void writeByte(unsigned char data);
		void transmitContentsOfFifo();
		void endOfFrame();
	};

class Sab8252xTx:public Sab8252xTxFifoOwner{
	protected:
		Sab8252xSim&		_sab;
		FrameReceiver*		_rx;
	private:
		Sab8252xTxStateReset		_resetState;
		Sab8252xTxFifo				_txFifo1;
		Sab8252xTxFifo				_txFifo2;
		Sab8252xTxState*			_state;
		Oscl::Queue<class Sab8252xTxFifo>	_emptyList;
		Oscl::Queue<class Sab8252xTxFifo>	_activeList;
		Sab8252xTxFifo*				_currentFifo;
		Sab8252xTxFifo*				_activeFifo;
		bool						_enabled;
		bool						_frameInProgress;
		bool						_twoOctetAddress;
		unsigned long				_remainingClocksBeforeNextByteClock;
	public:
		Sab8252xTx(Sab8252xSim& sab);
		Sab8252xTx(Sab8252xSim& sab,FrameReceiver* rx);
		void reset();
		void clock(unsigned long bitClocksSinceLastClock);
		void enable();
		void disable();
		void writeByte(unsigned char data);
		void transmitContentsOfFifo();
		void endOfFrame();
		FrameReceiver* changeReceiver(FrameReceiver* rx);	// returns old receiver

		// Sab8252xTxFifoOwner Interface
		void txFifoEmpty(Sab8252xTxFifo* theFifo);
	};

#endif
