
/***************************** 68000 SIMULATOR ****************************

File Name: CODE1.C
Version: 1.0

The instructions implemented in this file are the MOVE instructions:

   MOVE, MOVEP, MOVEA, MOVE_FR_SR, MOVE_TO_CCR, MOVE_TO_SR, MOVEM,
   MOVE_USP, MOVEQ


***************************************************************************/



#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */


/* the following two arrays specify the execution times of the MOVE
	instruction, for byte/word operands and for long operands */

int	move_bw_times [12][9] = {
	{ 4,    4,    8,    8,    8,   12,   14,   12,   16} ,
	{ 4,    4,    8,    8,    8,   12,   14,   12,   16} ,
	{ 8,    8,   12,   12,   12,   16,   18,   16,   20} ,
	{ 8,    8,   12,   12,   12,   16,   18,   16,   20} ,
	{10,   10,   14,   14,   14,   18,   20,   18,   22} ,
	{12,   12,   16,   16,   16,   20,   22,   20,   24} ,
	{14,   14,   18,   18,   18,   22,   24,   22,   26} ,
	{12,   12,   16,   16,   16,   20,   22,   20,   24} ,
	{16,   16,   20,   20,   20,   24,   26,   24,   28} ,
	{12,   12,   16,   16,   16,   20,   22,   20,   24} ,
	{14,   14,   18,   18,   18,   22,   24,   22,   26} ,
	{ 8,    8,   12,   12,   12,   16,   18,   16,   20}
	};


int	move_l_times [12][9] = {
	{ 4,    4,   12,   12,   12,   16,   18,   16,   20} ,
	{ 4,    4,   12,   12,   12,   16,   18,   16,   20} ,
	{12,   12,   20,   20,   20,   24,   26,   24,   28} ,
	{12,   12,   20,   20,   20,   24,   26,   24,   28} ,
	{14,   14,   22,   22,   22,   26,   28,   26,   30} ,
	{16,   16,   24,   24,   24,   28,   30,   28,   32} ,
	{18,   18,   26,   26,   26,   30,   32,   30,   34} ,
	{16,   16,   24,   24,   24,   28,   30,   28,   32} ,
	{20,   20,   28,   28,   28,   32,   34,   32,   36} ,
	{16,   16,   24,   24,   24,   28,   30,   28,   32} ,
	{18,   18,   26,   26,   26,   30,   32,   30,   34} ,
	{12,   12,   20,   20,   20,   24,   26,   24,   28}
	};


/* the following two arrays specify the instruction execution times
	for the MOVEM instruction for memory-to-reg and reg-to-memory cases */

int     movem_t_r_times[11] = {
        0,    0,    12,   12,    0,   16,   18,   16,   20,   16,   18  };

int     mover_t_m_times[11] = {
        0,    0,     8,    0,    8,   12,   14,   12,   16,    0,    0  };



int	MOVE(SimContext *context)
{
long	EV;
long	size;				/* 'size' holds the instruction size */
int	src, dst;		/* 'src' and 'dst' hold the addressing mode codes */
int	result;
							/* for instruction execution time computation */

/* MOVE has a different format for size field than all other instructions */
/* so we can't use the 'decode_size' function */
switch ( (context->inst >> 12) & 0x03)
	{
	case 0x01 : size = BYTE;				/* bit pattern '01' */
		         break;
	case 0x03 : size = WORD;				/* bit pattern '10' */
		         break;
	case 0x02 : size = LONG;				/* bit pattern '11' */
		         break;
	default   : return (BAD_INST);		/* bad size field */
               break;
	}

/* the following gets the effective addresses for the source and destination
	operands */

if (eff_addr (context,size, ALL_ADDR, FALSE)) return (BAD_INST);

value_of_EA1 (context, &EV, size);

if (eff_addr (context,size, DATA_ALT_ADDR, FALSE)) return (BAD_INST);

result	= putEA2(context,EV,size);

if(result != SUCCESS) return result;

src = eff_addr_code(context,context->inst,0);	/* get the addressing mode codes */
dst = eff_addr_code(context,context->inst,6);

if (size == LONG)			/* use the codes in instruction time computation */
	inc_cyc (context,move_l_times [src] [dst]);
else
	inc_cyc (context,move_bw_times [src] [dst]);

/* now update the condition codes */
cc_update (context,N_A, GEN, GEN, ZER, ZER, EV, 0, EV, size, 0);

/* return the value for 'success' */
return SUCCESS;

}



int	MOVEP(SimContext *context)
{
int	address, Dx, disp, count, direction, reg;
long	temp, size;
int		result;

result	= mem_request (context,&context->pc, (long) WORD, &temp);
if(result != SUCCESS) return result;

from_2s_comp (context,temp, (long) WORD, &temp);

address = context->A[a_reg(context,context->inst&0x07)] + (temp & WORD);

direction = context->inst & 0x80;

if (context->inst & 0x40)
	{
	size = LONG;
	count = 4;
	}
else
	{
	size = WORD;
	count = 2;
	}

reg = (context->inst >> 9) & 0x07;
Dx = context->D[reg] & size;

for (result = SUCCESS;count > 0; count--)
	{
	disp = 8 * (count - 1);
	if (direction)
		result = mem_put (context, (long)((Dx >> disp) & BYTE) , address, (long) BYTE);
	else
		{
		result	= mem_req (context,address, (long) BYTE, &temp);
		switch  (count) {
			case 4 :	context->D[reg] = (context->D[reg] & 0x00ffffff) | (temp * 0x1000000);
						break;
			case 3 :	context->D[reg] = (context->D[reg] & 0xff00ffff) | (temp * 0x10000);
						break;
			case 2 :	context->D[reg] = (context->D[reg] & 0xffff00ff) | (temp * 0x100);
						break;
			case 1 : context->D[reg] = (context->D[reg] & 0xffffff00) | (temp);
						break;
			}
		}
	address += 2;
	}

inc_cyc (context, (size == LONG) ? 24 : 16);

return result;

}



int	MOVEA(SimContext *context)
{
long	EV;
long	size;
int	src;

if (context->inst & 0x1000)
	size = WORD;
else
	size = LONG;

src = eff_addr_code(context,context->inst,0);
if (size == WORD)
	inc_cyc (context,move_bw_times[src][2]);
else
	inc_cyc (context,move_l_times[src][2]);

if (eff_addr(context,size, ALL_ADDR, FALSE))
	return (BAD_INST);

value_of_EA1 (context, &EV, size);

if (size == WORD)
	sign_extend (context,(int)EV, (long) WORD, &EV);

context->A[a_reg(context,(context->inst >> 9) & 0x07)] = EV;

return SUCCESS;

}



int	MOVE_FR_SR(SimContext *context)
{
int		result;

if (eff_addr (context,(long) WORD, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);

result	= putEA1 (context, (long) context->sr, (long) WORD);

inc_cyc (context,(context->inst & 0x0030) ? 8 : 6);

return result;

}



int	MOVE_TO_CCR(SimContext *context)
{
long	EV;
int		result;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);

value_of_EA1 (context, &EV, BYTE);

result = putSR (context, EV, (long) BYTE);

inc_cyc (context,12);

return result;

}



int	MOVE_TO_SR(SimContext *context)
{
long	EV;
int		result;

if (! (context->sr & sbit))
	return (NO_PRIVILEGE);

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);

value_of_EA1 (context, &EV, WORD);

result = putSR (context, EV, (long) WORD);

inc_cyc (context,12);

return result;
}



int	MOVEM(SimContext *context)
{
int	direction, addr_modes_mask, counter, addr_mode;
int	displacement, address, total_displacement;
long	size, mask_list, temp;
int		result;

result	= mem_request (context,&context->pc, (long) WORD, &mask_list);
if(result != SUCCESS) return result;

if (context->inst & 0x0040)
	size = LONG;
else
	size = WORD;

if ((direction = (context->inst & 0x0400)))
	addr_modes_mask = CONTROL_ADDR | bit_4;
else
	addr_modes_mask = CONT_ALT_ADDR | bit_5;

if (eff_addr (context,size, addr_modes_mask, FALSE))
	return (BAD_INST);			/* bad effective address */

address = (long)context->EA1;
total_displacement = address;

if ((context->inst & 0x0038) != 0x20)
	{
	if (size == WORD)
		displacement = 2;
	else
		displacement = 4;
	}
else
	{
	if (size == WORD)
		displacement = -2;
	else
		displacement = -4;
	}

addr_mode = eff_addr_code (context,context->inst,0);

if (direction)
	inc_cyc (context,movem_t_r_times[addr_mode]);		/* memory to registers */
else
	inc_cyc (context,mover_t_m_times[addr_mode]);		/* registers to memory */

for (counter = 0; counter < 16; counter++)
	if (mask_list & (1 << counter))
		{
		if (size == LONG)
			inc_cyc (context,8);
		else
			inc_cyc (context,4);
		if (direction)
			{
			if (size == WORD)		/* if size is WORD then sign-extend */
				{
				result	= mem_req (context,address, (long) WORD, &temp);
				if(result != SUCCESS) break;
				sign_extend (context,(int) temp, LONG, &temp);
				}
			else if(size == LONG){
				result = mem_req (context,address, (long) LONG, &temp);
				if(result != SUCCESS) break;
				}
			if (counter < 8)
				context->D[counter] = temp;
			else
				context->A[a_reg(context,counter - 8)] = temp;
			}
		else
 			{		/* registers to memory */
 			if ((context->inst & 0x38) == 0x20)	/* If predecrement -(An) */
				{
				if (counter < 8){
					result	= mem_put (context,context->A[a_reg(context,7 - counter)], address, size);
					if(result != SUCCESS) break;
					}
				else{
					result	= mem_put (context,context->D[15 - counter], address, size);
					if(result != SUCCESS) break;
					}
				}
			else
				{
				if (counter < 8){
					result	= mem_put (context,context->D[counter], address, size);
					if(result != SUCCESS) break;
					}
				else{
					result	= mem_put (context,context->A[a_reg(context,counter - 8)], address, size);
					if(result != SUCCESS) break;
					}
				}
			}
		address += displacement;
		}
address -= displacement;
total_displacement = address - total_displacement;

/* if pre-decrement or post-increment modes then change the value */
/* of the address register appropriately */
if ( ((context->inst & 0x38) == 0x20) || ( (context->inst & 0x38) == 0x18) )
	context->A[a_reg(context,context->inst & 0x7)] += total_displacement;

return result;

}



int	MOVE_USP(SimContext *context)
{
int	reg;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);		/* supervisor state not on */

reg = context->inst & 0x7;
if (reg == 7)
	reg = 8;

if (context->inst & 0x8)
	context->A[reg] = context->A[7];
else
	context->A[7] = context->A[reg];

inc_cyc (context,4);

return SUCCESS;

}

int	MOVEC(SimContext *context)
{
int				registerNum;
int				controlRegisterNum;
int				isAddressRegister;
long			extension;
unsigned long*	registerPtr;
unsigned long*	controlRegisterPtr;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);		/* supervisor state not on */

mem_request (context,&context->pc, (long) WORD, &extension);
from_2s_comp (context,extension, (long) WORD, &extension);

isAddressRegister	= (extension & 0x8000)?1:0;
registerNum			= ((extension>>12) & 0x0007);
controlRegisterNum	= (extension & 0x0FFF);

switch(controlRegisterNum){
	case 0x000:	/* SFC */
		controlRegisterPtr	= &context->SFC;
		break;
	case 0x001: /* DFC */
		controlRegisterPtr	= &context->DFC;
		break;
	case 0x800: /* USP */
		controlRegisterPtr	= &context->A[8];
		break;
	case 0x801: /* VBR */
		controlRegisterPtr	= &context->VBR;
		break;
	default:
		return BAD_INST;
	}

if(isAddressRegister){
	registerPtr	= &context->A[registerNum];
	}
else{
	registerPtr	= &context->D[registerNum];
	}

if(context->inst & 0x0001){
	*controlRegisterPtr	= *registerPtr;
	}
else{
	*registerPtr	= *controlRegisterPtr;
	}

inc_cyc (context,4);

return SUCCESS;

}

int	MOVEQ(SimContext *context)
{
int	reg;

reg = (context->inst >> 9) & 0x7;
context->source = context->inst & 0xff;
context->dest = context->D[reg];

/* the data register is sign extended to a longword */
sign_extend (context,(int)context->source, (long) BYTE, &context->D[reg]);
sign_extend (context,(int)context->D[reg], (long) WORD, &context->D[reg]);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->D[reg], LONG, 0);
inc_cyc (context,4);
return SUCCESS;

}

