
/***************************** 68000 SIMULATOR ****************************

File Name: CODE2.C
Version: 1.0

The context->instructions implemented in this file are the data movement context->instructions
   other than the MOVE context->instructions:

		EXG, LEA, PEA, LINK, UNLK


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */




int	EXG(SimContext *context)
{
long	temp_reg;
int	Rx, Ry;

Rx = (context->inst >> 9) & 0x07;
Ry = context->inst & 0x07;
switch ((context->inst >> 3) & 0x1f)
	{
	case 0x08: temp_reg = context->D[Rx];
		context->D[Rx] = context->D[Ry];
		context->D[Ry] = temp_reg;
		break;
	case 0x09:	temp_reg = context->A[a_reg(context,Rx)];
		context->A[a_reg(context,Rx)] = context->A[a_reg(context,Ry)];
		context->A[a_reg(context,Ry)] = temp_reg;
		break;
	case 0x11: temp_reg = context->D[Rx];
		context->D[Rx] = context->A[a_reg(context,Ry)];
		context->A[a_reg(context,Ry)] = temp_reg;
		break;
	default  : return (BAD_INST);	/* bad op_mode field */
              break;
	}

inc_cyc (context,6);

return SUCCESS;

}




int	LEA(SimContext *context)
{
int	reg;

reg = (context->inst >> 9) & 0x07;

if (eff_addr (context,LONG, CONTROL_ADDR, FALSE))
	return (BAD_INST);		/* bad context->instruction format */

/* perform the LEA operation */
context->A[a_reg(context,reg)] = context->EA1;

switch (eff_addr_code (context,context->inst, 0)) {
	case 0x02 : inc_cyc (context,4);
	            break;
	case 0x05 : inc_cyc (context,8);
	            break;
	case 0x06 : inc_cyc (context,12);
	            break;
	case 0x07 : inc_cyc (context,8);
	            break;
	case 0x08 : inc_cyc (context,12);
	            break;
	case 0x09 : inc_cyc (context,8);
	            break;
	case 0x0a : inc_cyc (context,12);
	            break;
	default   : break;
	}

return SUCCESS;

}




int	PEA(SimContext *context)
{

if (eff_addr (context,LONG, CONTROL_ADDR, FALSE))
	return (BAD_INST);		/* bad context->instruction format */

/* push the longword address computed by the */
/* effective address routine onto the stack */

context->A[a_reg(context,7)] -= 4;
putLocation (context,context->A[a_reg(context,7)], context->EA1, LONG);

switch (eff_addr_code (context,context->inst, 0)) {
	case 0x02 : inc_cyc (context,12);
	            break;
	case 0x05 : inc_cyc (context,16);
	            break;
	case 0x06 : inc_cyc (context,20);
	            break;
	case 0x07 : inc_cyc (context,16);
	            break;
	case 0x08 : inc_cyc (context,20);
	            break;
	case 0x09 : inc_cyc (context,16);
	            break;
	case 0x0a : inc_cyc (context,20);
	            break;
	default   : break;
	}

return SUCCESS;

}




int	LINK(SimContext *context)
{
int	reg;
long	temp, displacement;

reg = context->inst & 0x07;

mem_request (context,&context->pc, (long) WORD, &temp);
from_2s_comp(context,temp, (long) WORD, &displacement);

/* perform the LINK operation */
context->A[a_reg(context,7)] -= 4;
putLocation (context,context->A[a_reg(context,7)], context->A[reg], LONG);
context->A[reg] = context->A[a_reg(context,7)];
context->A[a_reg(context,7)] = context->A[a_reg(context,7)] + displacement;

inc_cyc (context,16);

return SUCCESS;

}



int	UNLK(SimContext *context)
{
int	reg;

reg = context->inst & 0x07;

context->A[a_reg(context,7)] = context->A[reg];
mem_req (context, (int) context->A[a_reg(context,7)], LONG, &context->A[reg]);
context->A[a_reg(context,7)] += 4;

inc_cyc (context,12);

return SUCCESS;

}

