
/***************************** 68000 SIMULATOR ****************************

File Name: CODE3.C
Version: 1.0

The context->instructions implemented in this file are the integer arithmetic
	operations:

		ADD, ADDA, ADDI, ADDQ, ADDX, SUB, SUBA, SUBI, SUBQ, SUBX


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */




int	ADD(SimContext *context)
{
long	EV;
int	addr_modes_mask;			/* holds mask for use in 'eff_addr(context,)' */
int	reg;							/* holds data register number */
long	size;							/* holds context->instruction size */

/* compute addressing modes mask from appropriate bit in context->instruction word */
addr_modes_mask = (context->inst & 0x0100) ? MEM_ALT_ADDR : ALL_ADDR;

/* the following statement decodes the context->instruction size using */
/* decode_size(context,), then decodes the effective address using eff_addr(context,) */
if ((decode_size(context,&size)) ||
    (eff_addr (context,size, addr_modes_mask, TRUE)))
	return (BAD_INST);			/* if size or effective address format */
										/* is bad then return 'bad context->instruction' */

reg = (context->inst >> 9) & 0x0007;		/* get data register number */

/* now perform the addition according to context->instruction format */
/* context->instruction bit 8 'on' means data register to effective-address */
/* context->instruction bit 8 'off' means effective-address to data register */
if (context->inst & 0x0100) {
	context->source = context->D[reg];
	value_of_EA1(context, &EV, size);
	context->dest	= EV;
	putEA1 (context, context->source + context->dest, size);		/* use 'put' to deposit context->result */
	value_of_EA1 (context, &context->result, size);	/* set 'context->result' for use in 'cc_update' */
	}
else {
	value_of_EA1 (context, &EV, size);
	context->source = EV;						/* get source. */
	context->dest = context->D[reg];										/* get context->destination */
	putDataReg(context,reg,context->source+context->dest,size);
	context->result = context->D[reg];				/* set 'context->result' for use in 'cc_update(context,)' */
	}

/* update the condition codes */
cc_update (context,GEN, GEN, GEN, CASE_1, CASE_5, context->source, context->dest, context->result, size, 0);

/* now update the cycle counter appropriately depending on the context->instruction */
/* size and the context->instruction format */
if (context->inst & 0x0100) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	if (size == LONG) {
		if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
			inc_cyc (context,8);
		else
			inc_cyc (context,6);
		}
	else {
		inc_cyc (context,4);
		}
	}

/* return 'success' */
return SUCCESS;

}


int	ADDA(SimContext *context)
{
long	EV;
long	size;
int	reg;

if (context->inst & 0x0100)
	size = LONG;
else
	size = WORD;

if (eff_addr (context,size, ALL_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);
sign_extend (context,(int)EV, size, &EV);	/* Fix to ADDA bug */
context->source = EV;
context->dest = context->A[a_reg(context,reg)];

#if 0
putAddressReg (context,a_reg(context,reg), context->source + context->dest, size);
#else
context->A[a_reg(context,reg)]  = context->source + context->dest;	/* Fix to ADDA bug */
#endif

if (size == LONG) {
	if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
		inc_cyc (context,8);
	else
		inc_cyc (context,6);
	}
else
	inc_cyc (context,8);

return SUCCESS;

}



int	ADDI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, size);
context->dest = EV;

putEA1 (context, context->source + context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,GEN, GEN, GEN, CASE_1, CASE_5, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038) {
	inc_cyc (context, (size == LONG) ? 20 : 12);
	}
else {
	inc_cyc (context, (size == LONG) ? 16 : 8);
	}

return SUCCESS;

}



int	ADDQ(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad size format */

if ((context->inst & 0x38) == 0x8)		/* if address reg direct, operation is long */
	size = LONG;

if (eff_addr (context,size, ALTERABLE_ADDR, TRUE))
	return (BAD_INST);			/* bad effectove address format */

context->source = ((context->inst >> 9) & 0x07);
if (context->source == 0) 			/* if context->source is '0', that corresponds to '8' */
	context->source = 8;

value_of_EA1 (context, &EV, size);
context->dest = EV;

putEA1 (context, context->source + context->dest, size);
value_of_EA1 (context,&context->result, size);

if (!((context->inst & 0x38) == 0x8))   /* if address reg direct, cc's not affected */
cc_update (context,GEN, GEN, GEN, CASE_1, CASE_5, context->source, context->dest, context->result, size, 0);

switch (context->inst & 0x0038) {
	case 0x0  : inc_cyc (context, (size == LONG) ? 8 : 4);
	            break;
	case 0x1  : inc_cyc (context,8);
	            break;
	default   : inc_cyc (context, (size == LONG) ? 12 : 8);
	            break;
	}

return SUCCESS;

}



int	ADDX(SimContext *context)
{
long	size;
int	Rx, Ry;

if (decode_size(context,&size)) return (BAD_INST);

Rx = (context->inst >> 9) & 0x0007;
Ry = context->inst & 0x0007;

/* perform the ADDX operation */
if (context->inst & 0x0008)
	{
	Rx = a_reg (context,Rx);
	Ry = a_reg (context,Ry);
	if (size == BYTE) {
		context->A[Rx]--;
	   context->A[Ry]--;
	   }
	if (size == WORD) {
		context->A[Rx] -= 2;
		context->A[Ry] -= 2;
	   }
	if (size == LONG) {
		context->A[Rx] -= 4;
		context->A[Ry] -= 4;
	   }
	mem_req (context,(int) context->A[Ry], size, &context->source);
	mem_req (context,(int) context->A[Rx], size, &context->dest);
	putLocation (context,context->A[a_reg(context,Rx)], context->source + context->dest + ((context->sr & xbit) >> 4), size);
	mem_req (context,(int) context->A[Rx], size, &context->result);
	}
else
	{
	context->source = context->D[Ry] & size;
	context->dest = context->D[Rx] & size;
	putDataReg (context,Rx, context->source + context->dest + ((context->sr & xbit) >> 4), size);
	context->result = context->D[Rx] & size;
	}

cc_update (context,GEN, GEN, CASE_1, CASE_1, CASE_5, context->source, context->dest, context->result, size, 0);

if (size == LONG)
   inc_cyc (context, (context->inst & 0x0008) ? 30 : 8);
else
   inc_cyc (context, (context->inst & 0x0008) ? 18 : 4);

return SUCCESS;

}



int	SUB(SimContext *context)
{
int	addr_modes_mask, reg;
long	size;
long	EV;

addr_modes_mask = (context->inst & 0x0100) ? MEM_ALT_ADDR : ALL_ADDR;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, addr_modes_mask, TRUE)))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);

if (context->inst & 0x0100)
	{
	context->source = context->D[reg] & size;
	context->dest = EV & size;
	putEA1 (context, context->dest - context->source, size);
	value_of_EA1 (context, &context->result, size);
	}
else
	{
	context->source = EV & size;
	context->dest = context->D[reg] & size;
	putDataReg (context,reg, context->dest - context->source, size);
	context->result = context->D[reg] & size;
	}

cc_update (context,GEN, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0100) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	if (size == LONG) {
		if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
			inc_cyc (context,8);
		else
			inc_cyc (context,6);
		}
	else {
		inc_cyc (context,4);
		}
	}

return SUCCESS;

}


int	SUBA(SimContext *context)
{
long	EV;
long	size;
int	reg;

if (context->inst & 0x0100)
	size = LONG;
else
	size = WORD;

if (eff_addr (context,size, ALL_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);
context->source = EV;

context->dest = context->A[a_reg(context,reg)];

#if 0
putAddressReg (context,a_reg(context,reg), context->dest - context->source, size);
#else
context->A[a_reg(context,reg)]  = context->dest - context->source;
#endif

if (size == LONG) {
	if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
		inc_cyc (context,8);
	else
		inc_cyc (context,6);
	}
else
	inc_cyc (context,8);

return SUCCESS;

}


int	SUBI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, size);
context->dest = EV;

putEA1 (context, context->dest - context->source, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,GEN, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038) {
	inc_cyc (context, (size == LONG) ? 20 : 12);
	}
else {
	inc_cyc (context, (size == LONG) ? 16 : 8);
	}

return SUCCESS;

}



int	SUBQ(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad size format */

if ((context->inst & 0x38) == 0x8)		/* if address reg direct, operation is long */
	size = LONG;

if (eff_addr (context,size, ALTERABLE_ADDR, TRUE))
	return (BAD_INST);			/* bad effectove address format */

context->source = ((context->inst >> 9) & 0x07);
if (context->source == 0) 			/* if context->source is '0', that corresponds to '8' */
	context->source = 8;

value_of_EA1 (context,&EV, size);
context->dest = EV;

putEA1 (context, context->dest - context->source, size);
value_of_EA1 (context, &context->result, size);

if (!((context->inst & 0x38) == 0x8))   /* if address reg direct, cc's not affected */
cc_update (context,GEN, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

switch (context->inst & 0x0038) {
	case 0x0  : inc_cyc (context, (size == LONG) ? 8 : 4);
	            break;
	case 0x1  : inc_cyc (context,8);
	            break;
	default   : inc_cyc (context, (size == LONG) ? 12 : 8);
	            break;
	}

return SUCCESS;

}




int	SUBX(SimContext *context)
{
long	size;
int	Rx, Ry;

if (decode_size(context,&size)==FAILURE)
	return (BAD_INST);

Ry = (context->inst >> 9) & 0x0007;
Rx = context->inst & 0x0007;

/* perform the SUBX operation */
if (context->inst & 0x0008)
	{
	Rx = a_reg(context,Rx);
	Ry = a_reg(context,Ry);
	if (size == LONG)
		{
		context->A[Rx] -= 4;
		context->A[Ry] -= 4;
		}
	else if (size == WORD)
		{
		context->A[Rx] -= 2;
		context->A[Ry] -= 2;
		}
	else
		{
		context->A[Rx]--;
		context->A[Ry]--;
		}
	mem_req (context, (int) context->A[Rx], size, &context->source);
	mem_req (context, (int) context->A[Ry], size, &context->dest);
	putLocation (context,context->A[Ry], context->dest - context->source - ((context->sr & xbit)>> 4), size);
	mem_req (context, (int) context->A[Ry], size, &context->result);
	}
else
	{
	context->source = context->D[Rx] & size;
	context->dest = context->D[Ry] & size;
	putDataReg (context,Ry, context->dest - context->source - ((context->sr & xbit) >> 4), size);
	context->result = context->D[Ry] & size;
	}

cc_update (context,GEN, GEN, CASE_1, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

if (size == LONG)
   inc_cyc (context, (context->inst & 0x0008) ? 30 : 8);
else
   inc_cyc (context, (context->inst & 0x0008) ? 18 : 4);

return SUCCESS;

}

