
/***************************** 68000 SIMULATOR ****************************

File Name: CODE4.C
Version: 1.0

The context->instructions implemented in this file are the integer arithmetic
	operations:

		DIVS, DIVU, MULS, MULU, NEG, NEGX, CMP, CMPA, CMPI, CMPM,
		TST, CLR, EXT


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */





int	DIVS(SimContext *context)
{
long	EV;
int	reg, overflow;
long	remainder;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, WORD);

from_2s_comp (context,EV & WORD, (long) WORD, &context->source);
from_2s_comp (context,context->D[reg], LONG, &context->dest);

if (context->source == 0)
	return (DIV_BY_ZERO);		/* initiate exception processing */

/* check for overflow */
if ( ( (context->source < 0) && (context->dest >= 0) ) || ( (context->source >= 0) && (context->dest < 0) ) )
	{
	if ((context->dest / -context->source) > WORD)
		overflow = TRUE;
	else
		overflow = FALSE;
	}
else
	{
	if ((context->dest / context->source) > WORD)
		overflow = TRUE;
	else
		overflow = FALSE;
	}

if (overflow)
	context->sr |= vbit;			/* if overflow then quit */
else
	{
	context->sr &= ~vbit;
	context->result = (context->dest / context->source) & 0xffff;
	remainder = (context->dest % context->source) & 0xffff;
	context->D[reg] = context->result = context->result | (remainder * 0x10000);
	cc_update (context,N_A, GEN, GEN, N_A, ZER, context->source, context->dest, context->result, (long) WORD, 0);
	if ( ( (context->source < 0) && (context->dest >= 0) ) || ( (context->source >= 0) && (context->dest < 0) ) )
		context->sr |= nbit;
	else
		context->sr &= ~nbit;
	}

inc_cyc (context,158);

return SUCCESS;

}



int	DIVU(SimContext *context)
{
long	EV;
int	reg;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, WORD);

context->source = EV & WORD;
context->dest = context->D[reg];

if (context->source == 0)
	return (DIV_BY_ZERO);		/* initiate exception processing */

if ( (context->dest / context->source) > WORD)
	context->sr |= vbit; 		/* check for overflow */
else
	{
	context->sr &= ~vbit;
	context->D[reg] = context->result =((context->dest / context->source) & 0xffff) | ((context->dest % context->source) * 0x10000);

	cc_update (context,N_A, GEN, GEN, N_A, ZER, context->source, context->dest, context->result, (long) WORD, 0);
	}

inc_cyc (context,140);

return SUCCESS;

}

int	DIVUL(SimContext *context)
{
unsigned long		EV;
int					registerDq;
int					registerDr;
int					size64Bit;
int					result;
long				extensionInstruction;
unsigned long long	dividend	= 0;
unsigned long long	quotient;
unsigned long long	remainder;

if((result = mem_request (context,&context->pc, WORD, &extensionInstruction) != SUCCESS)){
	return result;
	}

registerDq	= (extensionInstruction >> 12) & 0x07;
registerDr	= extensionInstruction & 0x07;
size64Bit	= (extensionInstruction & 0x0400)?1:0;

if (eff_addr (context,(long) LONG, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, (long *)&EV, LONG);

if(EV == 0){
	return DIV_BY_ZERO;
	}

if(size64Bit){
	dividend	= context->D[registerDr];
	dividend	<<= 32;
	}

dividend	|= (unsigned long)context->D[registerDq];

quotient	= dividend/EV;
remainder	= dividend%EV;

if(quotient & 0xFFFFFFFF00000000LL){
	context->sr |= vbit; 		/* check for overflow */
	}
else{
	context->D[registerDr]	= (long)remainder;
	context->D[registerDq]	= (long)quotient;
	cc_update (context,N_A, GEN, GEN, N_A, ZER, context->source, context->dest, context->result, (long) WORD, 0);
	}

return SUCCESS;
}



int	MULS(SimContext *context)
{
long	EV;
int	reg;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, WORD);

from_2s_comp (context,EV & WORD, (long) WORD, &context->source);
from_2s_comp (context,context->D[reg] & WORD, (long) WORD, &context->dest);

context->D[reg] = context->result = context->source * context->dest;

cc_update (context,N_A, GEN, GEN, CASE_9, ZER, context->source, context->dest, context->result, (long) WORD, 0);

inc_cyc (context,70);

return SUCCESS;

}

int	MULU(SimContext *context)
{
long	EV;
int	reg;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, WORD);

context->source = EV & WORD;
context->dest = context->D[reg] & WORD;

context->D[reg] = context->result = context->source * context->dest;

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, LONG, 0);

inc_cyc (context,70);

return SUCCESS;

}

int	MULL(SimContext *context)
{
int						result;
long					EV;
unsigned long long		product;
unsigned long long		multiplicand;
unsigned long long		multiplier;
long					extensionInstruction;
unsigned				signedOperation;
unsigned				productIs64bit;
unsigned				regL;
unsigned				regH;

if((result = mem_request (context,&context->pc, WORD, &extensionInstruction) != SUCCESS)){
	return result;
	}

signedOperation	= 0x0800 & extensionInstruction;
productIs64bit	= 0x0400 & extensionInstruction;
regL			= (extensionInstruction >> 12) & 0x0007;

if (eff_addr (context,(long) LONG, DATA_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, LONG);

context->source	= EV & LONG;
context->dest	= context->D[regL] & LONG;

if(signedOperation){
	if(context->source & 0x80000000){
		multiplicand	= (unsigned long)context->source;
		}
	else{
		multiplicand	= (unsigned long)context->source;
		multiplicand	&= 0x0FFFFFFFF;
		}
	if(context->dest & 0x80000000){
		multiplier	= (unsigned long)context->dest;
		}
	else{
		multiplier	= (unsigned long)context->dest;
		multiplier	&= 0x0FFFFFFFF;
		}
	}
else{
	multiplier	= (unsigned long)context->dest;
	multiplier	&= 0x0FFFFFFFF;
	multiplicand	= (unsigned long)context->source;
	multiplicand	&= 0x0FFFFFFFF;
	}

product = multiplier * multiplicand;

context->D[regL]	= product & LONG;
context->result		= (product >> 32) & LONG;

if(productIs64bit){
	regH				= extensionInstruction & 0x0007;
	context->D[regH]	= context->result;
	}

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, LONG, 0);

inc_cyc (context,70);	/* bogus */

return SUCCESS;
}

int	NEG(SimContext *context)
{
long	EV;
long	size;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, DATA_ALT_ADDR, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->source = context->dest = EV & size;

/* perform the NEG operation */
putEA1 (context, -context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,GEN, GEN, GEN, CASE_3, CASE_4, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0030)
	inc_cyc (context, (size == LONG) ? 12 : 8);
else
	inc_cyc (context, (size == LONG) ? 6 : 4);

return SUCCESS;

}



int	NEGX(SimContext *context)
{
long	EV;
long	size;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, DATA_ALT_ADDR, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->source = context->dest = EV & size;

/* perform the NEGX operation */
putEA1 (context, -context->dest - ((context->sr & xbit) >> 4), size);
value_of_EA1 (context, &context->result, size);

cc_update (context,GEN, GEN, CASE_1, CASE_3, CASE_4, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0030) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	inc_cyc (context, (size == LONG) ? 6 : 4);
	}

return SUCCESS;

}



int	CMP(SimContext *context)
{
long	EV;
long	size;
int	reg;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, ALL_ADDR, TRUE)))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);

context->source = EV & size;
context->dest = context->D[reg] & size;

putResult (context, context->dest - context->source, size);

/* now set the condition codes */
cc_update (context,N_A, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

inc_cyc (context, (size == LONG) ? 6 : 4);

return SUCCESS;

}


int	CMPA(SimContext *context)
{
long	EV;
long	size;
int	reg;

if (context->inst & 0x0100)
	size = LONG;
else
	size = WORD;

if (eff_addr (context,size, ALL_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

reg = a_reg(context,(context->inst >> 9) & 0x0007);

value_of_EA1 (context, &EV, size);

if (size == WORD)
	sign_extend (context,(int)EV, WORD, &EV);

context->source = EV & size;
context->dest = context->A[reg] & size;

putResult (context, context->dest - context->source, size);

/* now set the condition codes according to the context->result */
/* cc_update (context,N_A, GEN, GEN, GEN, GEN, context->source, context->dest, context->result, size, 0); */
cc_update (context,N_A, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

inc_cyc (context,6);

return SUCCESS;

}


int	CMPI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context,&EV, size);

context->dest = EV & size;

putResult (context, context->dest - context->source, size);

cc_update (context,N_A, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	inc_cyc (context, (size == LONG) ? 14 : 8);
	}

return SUCCESS;

}



int	CMPM(SimContext *context)
{
long	size;
int	Rx, Ry;

if (decode_size(context,&size))
	return (BAD_INST);

Rx = a_reg(context,(context->inst >> 9) & 0x07);
Ry = a_reg(context,context->inst & 0x07);

mem_req (context, (int) context->A[Ry], size, &context->source);
mem_req (context, (int) context->A[Rx], size, &context->dest);

putResult (context, context->dest - context->source, size);

if (size == BYTE)  {
	context->A[Rx]++;
	context->A[Ry]++;
	};
if (size == WORD)  {
	context->A[Rx] += 2;
	context->A[Ry] += 2;
	};
if (size == LONG)  {
	context->A[Rx] += 4;
	context->A[Ry] += 4;
	};

/* now set the condition codes according to the context->result */
cc_update (context,N_A, GEN, GEN, CASE_2, CASE_6, context->source, context->dest, context->result, size, 0);

inc_cyc (context, (size == LONG) ? 20 : 12);

return SUCCESS;

}




int	TST(SimContext *context)
{
long	size;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, ALTERABLE_ADDR, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &context->dest, size);

/* test the context->dest operand and set the condition codes accordingly */
cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->dest, size, 0);

if (context->inst & 0x0030) {
	inc_cyc (context, (size == LONG) ? 4 : 4);
	}
else {
	inc_cyc (context, (size == LONG) ? 4 : 4);
	}

return SUCCESS;

}



int	CLR(SimContext *context)
{
long	EV;
long	size;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, DATA_ALT_ADDR, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->source = context->dest = EV & size;

/* perform the CLR operation */
putEA1 (context, (long) 0, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, ZER, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0030) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	inc_cyc (context, (size == LONG) ? 6 : 4);
	}

return SUCCESS;

}




int	EXT(SimContext *context)
{
long	size;
int	reg;

reg = context->inst & 0x07;

if (context->inst & 0x0040)
	size = LONG;
else
	size = WORD;

context->source = context->dest = context->D[reg] & size;

if (size == WORD)
 context->D[reg] = (context->D[reg]&~WORD) | (context->D[reg] & BYTE) | (0xff00 * ((context->D[reg]>>7) & 0x01));
else{
	if(context->inst & 0x0100){
		 context->D[reg] = (context->D[reg] & BYTE) | (0xffffff00 * ((context->D[reg] >> 7) & 0x01));
		}
	else{
		 context->D[reg] = (context->D[reg] & WORD) | (0xffff0000 * ((context->D[reg] >> 15) & 0x01));
		}
	}

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->D[reg], size, 0);

inc_cyc (context,4);

return SUCCESS;

}

