
/***************************** 68000 SIMULATOR ****************************

File Name: CODE5.C
Version: 1.0

The context->instructions implemented in this file are the binary coded decimal
   operations:

		ABCD, SBCD, NBCD


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */




int	ABCD(SimContext *context)
{
int	Rx, Ry, carry, temp_Result;

Rx = (context->inst >> 9) & 0x0007;
Ry = context->inst & 0x0007;

/* perform the ABCD operation */
if (context->inst & 0x0008)  /* Rx & Ry are address registers used in predecrement */
	{	       		  /* mode */
	Rx = a_reg(context,Rx);
	Ry = a_reg(context,Ry);
	context->A[Rx]--;
	context->A[Ry]--;
	value_of_Location(context,context->A[Ry],&context->source,BYTE);
	value_of_Location(context,context->A[Rx],&context->dest,BYTE);
	}
else		/* Rx & Ry are data registers */
	{
	context->source = context->D[Ry] & BYTE;
	context->dest = context->D[Rx] & BYTE;
	}

/* perform the ABCD operation */
context->result = ((context->sr & xbit) >> 4) + (context->source & 0xf) + (context->dest & 0xf);
if (context->result > 9)
	{
	context->result = context->result - 10;
	carry = 1;
	}
else
	carry = 0;
temp_Result = ((context->source >> 4) & 0xf) + ((context->dest >> 4) & 0xf) + carry;
if (temp_Result > 9)
	{
	temp_Result = temp_Result - 10;
	carry = 1;
	}
else
	carry = 0;
context->result = context->result + (temp_Result << 4);

if (context->inst & 0x0008)
	putLocation (context,context->A[Rx], context->result, (long) BYTE);
else
	putDataReg (context,Rx, context->result, (long) BYTE);
if (carry)
	context->sr = context->sr | cbit;
else
	context->sr = context->sr & ~cbit;

cc_update (context,GEN, UND, CASE_1, UND, N_A, context->source, context->dest, context->result, (long) BYTE, 0);

inc_cyc (context, (context->inst & 0x0008) ? 18 : 6);

return SUCCESS;

}



int	SBCD(SimContext *context)
{
int	Rx, Ry, borrow, temp_Result;

Rx = (context->inst >> 9) & 0x0007;
Ry = context->inst & 0x0007;

/* perform the SUB operation */
if (context->inst & 0x0008)     /* Rx & Ry are address registers used in */
	{	       /* predecrement mode */
	Rx = a_reg(context,Rx);
	Ry = a_reg(context,Ry);
	context->A[Rx]--;
	context->A[Ry]--;
	value_of_Location(context,context->A[Ry],&context->source,BYTE);
	value_of_Location(context,context->A[Rx],&context->dest,BYTE);
	}
else
	{		/* Rx & Ry are data registers */
	context->source = context->D[Ry];
	context->dest = context->D[Rx];
	}

/* perform the SBCD operation */
context->result = (context->dest & 0xf) - (context->source & 0xf) - ((context->sr & xbit) >> 4);
if (context->result < 0)
	{
	context->result = context->result + 10;
	borrow = 1;
	}
else
	borrow = 0;
temp_Result = ((context->dest >> 4) & 0xf) - ((context->source >> 4) & 0xf) - borrow;
if (temp_Result < 0)
	{
	temp_Result = temp_Result + 10;
	borrow = 1;
	}
else
	borrow = 0;
context->result = context->result + (temp_Result << 4);

if (context->inst & 0x0008)
	putLocation (context,context->A[Rx], context->result, (long) BYTE);
else
	putDataReg (context,Rx, context->result, (long) BYTE);

if (borrow)
	context->sr = context->sr | cbit;
else
	context->sr = context->sr & ~cbit;

cc_update (context,GEN, UND, CASE_1, UND, N_A, context->source, context->dest, context->result, (long) BYTE, 0);

inc_cyc (context, (context->inst & 0x0008) ? 18 : 6);

return SUCCESS;

}



int	NBCD(SimContext *context)
{
long	EV;
int	borrow, temp_Result;

if (eff_addr (context,(long) BYTE, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* illegal effective address */

value_of_EA1 (context, &EV, BYTE);

context->dest = EV & BYTE;

/* perform the NBCD operation */
context->result = 10 - (context->dest & 0xf) - ((context->sr & xbit) >> 4);

if (context->result < 10)
	borrow = 1;
else
	borrow = 0;

temp_Result = 10 - ((context->dest >> 4) & 0xf) - borrow;

if (temp_Result < 10)
	borrow = 1;
else
	borrow = 0;

context->result = context->result + (temp_Result << 4);

if (borrow)
	context->sr = context->sr | cbit;
else
	context->sr = context->sr & ~cbit;

putEA1 (context, context->result, (long) BYTE);

cc_update (context,GEN, UND, CASE_1, UND, N_A, context->source, context->dest, context->result, (long) BYTE, 0);

inc_cyc (context, (context->inst & 0x0030) ? 8 : 6);

return SUCCESS;

}

