
/***************************** 68000 SIMULATOR ****************************

File Name: CODE6.C
Version: 1.0

The context->instructions implemented in this file are the logical arithmetic
   operations:

		AND, ANDI, ANDI_TO_CCR, ANDI_TO_context->sr, OR, ORI, ORI_TO_CCR,
		ORI_TO_context->sr, EOR, EORI, EORI_TO_CCR, EORI_TO_context->sr, NOT


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */



int	AND(SimContext *context)
{
long	EV;
int	addr_modes_mask, reg;
long	size;

addr_modes_mask = (context->inst & 0x0100) ? MEM_ALT_ADDR : DATA_ADDR;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, addr_modes_mask, TRUE)))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);

if (context->inst & 0x0100)
	{
	context->source = context->D[reg] & size;
	context->dest = EV & size;
	putEA1 (context, context->dest & context->source, size);
	value_of_EA1 (context, &context->result, size);
	}
else
	{
	context->source = EV & size;
	context->dest = context->D[reg] & size;
	putDataReg (context,reg, context->dest & context->source, size);
	context->result = context->D[reg] & size;
	}

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0100)
	inc_cyc (context, (size == LONG) ? 12 : 8);
else {
	if (size == LONG) {
		if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
			inc_cyc (context,8);
		else
			inc_cyc (context,6);
		}
	else {
		inc_cyc (context,4);
		}
	}

return SUCCESS;

}



int	ANDI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->dest = EV & size;

putEA1 (context, context->source & context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038)
	inc_cyc (context, (size == LONG) ? 20 : 12);
else
	inc_cyc (context, (size == LONG) ? 16 : 8);

return SUCCESS;

}


int	ANDI_TO_CCR(SimContext *context)
{
long	temp;

mem_request (context,&context->pc, (long) WORD, &temp);

context->sr &= temp | 0xff00;

inc_cyc (context,20);

return SUCCESS;

}


int	ANDI_TO_SR(SimContext *context)
{
long	temp;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

mem_request (context,&context->pc, (long) WORD, &temp);
context->sr &= temp;

inc_cyc (context,20);

return SUCCESS;

}




int	OR(SimContext *context)
{
long	EV;
long	size;
int	mask, reg;

mask = (context->inst & 0x0100) ? MEM_ALT_ADDR : DATA_ADDR;

if ((decode_size(context,&size)) ||
    (eff_addr(context,size, mask, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);

if (context->inst & 0x0100)
	{
	context->source = context->D[reg] & size;
	context->dest = EV & size;
	putEA1 (context, context->source | context->dest, size);
	value_of_EA1 (context, &context->result, size);
	}
else
	{
	context->source = EV & size;
	context->dest = context->D[reg] & size;
	putDataReg (context,reg, context->source | context->dest, size);
	context->result = context->D[reg] & size;
	}

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0100)
	inc_cyc (context, (size == LONG) ? 12 : 8);
else {
	if (size == LONG) {
		if ( (!(context->inst & 0x0030)) || ((context->inst & 0x003f) == 0x003c) )
			inc_cyc (context,8);
		else
			inc_cyc (context,6);
		}
	else
		inc_cyc (context,4);
	}

return SUCCESS;

}




int	ORI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->dest = EV & size;

putEA1 (context, context->source | context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038) {
	inc_cyc (context, (size == LONG) ? 20 : 12);
	}
else {
	inc_cyc (context, (size == LONG) ? 16 : 8);
	}

return SUCCESS;

}


int	ORI_TO_CCR(SimContext *context)
{
long	temp;

mem_request (context,&context->pc, (long) WORD, &temp);

context->sr |= temp;

inc_cyc (context,20);

return SUCCESS;

}


int	ORI_TO_SR(SimContext *context)
{
long	temp;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

mem_request (context,&context->pc, (long) WORD, &temp);
context->sr |= temp;

inc_cyc (context,20);

return SUCCESS;

}



int	EOR(SimContext *context)
{
long	EV;
long	size;
int	reg;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, DATA_ALT_ADDR, TRUE)))
	return (BAD_INST);			/* bad context->instruction format */

reg = (context->inst >> 9) & 0x0007;

value_of_EA1 (context, &EV, size);

context->source = context->D[reg] & size;
context->dest = EV & size;

putEA1 (context, EV ^ context->D[reg], size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038)
	inc_cyc (context, (size == LONG) ? 12 : 8);
else
	inc_cyc (context, (size == LONG) ? 8 : 4);

return SUCCESS;

}



int	EORI(SimContext *context)
{
long	EV;
long	size;

if (decode_size(context,&size))
	return (BAD_INST);			/* bad context->instruction format */

mem_request (context,&context->pc, size, &context->source);

if (eff_addr (context,size, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);			/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->dest = EV & size;

putEA1 (context, context->source ^ context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0038) {
	inc_cyc (context, (size == LONG) ? 20 : 12);
	}
else {
	inc_cyc (context, (size == LONG) ? 16 : 8);
	}

return SUCCESS;

}


int	EORI_TO_CCR(SimContext *context)
{
long	temp;

mem_request (context,&context->pc, (long) WORD, &temp);

context->sr ^= temp;

inc_cyc (context,20);

return SUCCESS;

}


int	EORI_TO_SR(SimContext *context)
{
long	temp;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

mem_request (context,&context->pc, (long) WORD, &temp);
context->sr ^= temp;

inc_cyc (context,20);

return SUCCESS;

}


int	NOT(SimContext *context)
{
long	EV;
long	size;

if ((decode_size(context,&size)) ||
    (eff_addr (context,size, DATA_ALT_ADDR, TRUE)))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &EV, size);

context->source = context->dest = EV & size;

/* perform the NOT operation */
putEA1 (context, ~context->dest, size);
value_of_EA1 (context, &context->result, size);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->result, size, 0);

if (context->inst & 0x0030) {
	inc_cyc (context, (size == LONG) ? 12 : 8);
	}
else {
	inc_cyc (context, (size == LONG) ? 6 : 4);
	}

return SUCCESS;

}

