
/***************************** 68000 SIMULATOR ****************************

File Name: CODE7.C
Version: 1.0

The context->instructions implemented in this file are shift and rotate operations:

		SHIFT_ROT (ASL, Acontext->sr, LSL, Lcontext->sr, ROL, ROR, ROXL, ROXR), SWAP,
		BIT_OP (BCHG, BCLR, BSET, BTST), TAS


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */




int	SHIFT_ROT(SimContext *context)
{
unsigned long	EV;
long	size;
int		reg;
int		shift_count;
int		shift_size;
int		type;
int		counter;
int		mem_reg;
int		direction;
int		temp_bit;
int		temp_bit_2;

if ((mem_reg = ((context->inst & 0xc0) == 0xc0)))
	{
	if (eff_addr (context,(long) WORD, MEM_ALT_ADDR, TRUE))
		return (BAD_INST);		/* bad context->instruction format */
	size = WORD;
	shift_count = 1;
	value_of_EA1 (context, &EV, size);
	context->source = context->dest = EV & size;
	type = (context->inst & 0x600) >> 9;
	inc_cyc (context,8);
	}
else
	{
	if (decode_size(context,&size))
		return (BAD_INST); /* bad context->instruction format */
	if (context->inst & 0x20)
		shift_count = context->D[(context->inst >> 9) & 0x7] % 64;
	else
		{
		shift_count = (context->inst >> 9) & 0x7;
		if (shift_count == 0)
			shift_count = 8;
		}
	reg = context->inst & 7;
	context->source = context->dest = context->D[reg] & size;
	type = (context->inst & 0x18) >> 3;
	context->EA1 = reg;
	context->typeEA1	= effectiveAddressTypeDataRegister;
	value_of_EA1 (context, &EV, size);
	if (size == LONG)
		inc_cyc (context,8 + 2 * shift_count);
	else
		inc_cyc (context,6 + 2 * shift_count);
	}
direction = context->inst & 0x100;
if (size == LONG)
	shift_size = 31;
else if (size == WORD)
	shift_size = 15;
else
	shift_size = 7;

if (shift_count == 0)
	{
	if (type == 2)
		cc_update (context,N_A, GEN, GEN, ZER, CASE_1,
			context->source, context->dest, EV, size, shift_count);
	else
		cc_update (context,N_A, GEN, GEN, ZER, ZER,
			context->source, context->dest, EV, size, shift_count);
	}
else
switch (type) {
	case 0 : 											/* do an arithmetic shift */
	if (direction) {			/* do a shift left */
		putEA1 (context, (EV & size) << shift_count, size);
		value_of_EA1 (context, &EV, size);
		cc_update (context,GEN, GEN, GEN, CASE_4, CASE_3,
				context->source, context->dest, EV, size, shift_count);
		}
	else {		/* do a shift right */
		/* do the shift replicating the most significant bit */
		if ((EV >> shift_size) & 1)
			temp_bit = 1;
		else
			temp_bit = 0;
		for (counter = 1; counter <= shift_count; counter++)
			{
			putEA1 (context, (EV & size) >> 1, size);
			value_of_EA1 (context, &EV, size);
			if (temp_bit)
				putEA1 (context, EV | (1 << shift_size), size);
			else
				putEA1 (context, EV & (~(1 << shift_size)), size);
			value_of_EA1 (context, &EV, size);
			}
		cc_update (context,GEN, GEN, GEN, ZER, CASE_2,
				context->source, context->dest, EV, size, shift_count);
		}
      break;
	case 1 : 											/* do a logical shift */
				if (direction) {			/* do a shift left */
					putEA1 (context, EV << shift_count, size);
					value_of_EA1 (context, &EV, size);
					cc_update (context,GEN, GEN, GEN, ZER, CASE_3,
							context->source, context->dest, EV, size, shift_count);
					}
				else {		/* do a shift right */
					putEA1 (context, (EV & size) >> shift_count, size);
					value_of_EA1 (context, &EV, size);
					cc_update (context,GEN, GEN, GEN, ZER, CASE_2,
							context->source, context->dest, EV, size, shift_count);
					}
            break;
	case 2 :                    					/* do a rotate with extend */
				if (direction) {			/* do a rotate left */
					for (counter = 1; counter <= shift_count; counter++)
						{
						temp_bit = (EV >> shift_size) & 1;
						temp_bit_2 = (context->sr & xbit) >> 4;
						putEA1 (context, (EV & size) << 1, size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit_2)
							putEA1 (context, EV | 1, size);
						else
							putEA1 (context, EV & ~1, size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit)
							context->sr = context->sr | xbit;
						else
							context->sr = context->sr & ~xbit;
						}
					cc_update (context,GEN, GEN, GEN, ZER, CASE_3,
							context->source, context->dest, EV, size, shift_count);
					}
				else {		/* do a rotate right */
					for (counter = 1; counter <= shift_count; counter++)
						{
						temp_bit = EV & 1;
						temp_bit_2 = (context->sr & xbit) >> 4;
						putEA1 (context, (EV & size) >> 1, size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit_2)
							putEA1 (context,EV | (1 << shift_size), size);
						else
							putEA1 (context, EV & (~(1 << shift_size)), size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit)
							context->sr = context->sr | xbit;
						else
							context->sr = context->sr & ~xbit;
						}
					putEA1 (context, EV, size);
					cc_update (context,GEN, GEN, GEN, ZER, CASE_2,
							context->source, context->dest, EV, size, shift_count);
					}
            break;
	case 3 : 											/* do a rotate */
				if (direction) {			/* do a rotate left */
					for (counter = 1; counter <= shift_count; counter++)
						{
						temp_bit = (EV >> shift_size) & 1;
						putEA1 (context, (EV & size) << 1, size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit)
							putEA1 (context,EV | 1, size);
						else
							putEA1 (context, EV & ~1, size);
						value_of_EA1 (context, &EV, size);
						}
					cc_update (context,N_A, GEN, GEN, ZER, CASE_3,
							context->source, context->dest, EV, size, shift_count);
					}
				else {		/* do a rotate right */
					for (counter = 1; counter <= shift_count; counter++)
						{
						temp_bit = EV & 1;
						putEA1 (context, (EV & size) >> 1, size);
						value_of_EA1 (context, &EV, size);
						if (temp_bit)
							putEA1 (context, EV | (1 << shift_size), size);
						else
							putEA1 (context,EV & (~(1 << shift_size)), size);
						value_of_EA1 (context, &EV, size);
						}
					cc_update (context,N_A, GEN, GEN, ZER, CASE_2,
							context->source, context->dest, EV, size, shift_count);
					}
            break;
	}

return SUCCESS;

}



int	SWAP(SimContext *context)
{
long	reg;

reg = context->inst & 0x07;

/* perform the SWAP operation */
context->D[reg] = ((context->D[reg] & WORD) * 0x10000) | ((context->D[reg] & 0xffff0000) / 0x10000);

cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, context->D[reg], LONG, 0);

inc_cyc (context,4);

return SUCCESS;

}



int	BIT_OP(SimContext *context)
{
long			EV;
int				mem_reg;
long			size;
long			bit_no;
unsigned long	theResult;

if (context->inst & 0x100)
	bit_no = context->D[(context->inst >> 9) & 0x07];
else
	{
	mem_request (context,&context->pc, (long) WORD, &bit_no);
	bit_no = bit_no & 0xff;
	}

mem_reg = (context->inst & 0x38);

if (eff_addr (context,(long) BYTE, DATA_ADDR, TRUE))
	return (BAD_INST);		/* bad context->instruction format */

if (mem_reg)
	{	/* byte sized operations on memory */
	bit_no = bit_no % 8;
	size = BYTE;
	}
else
	{	/* long operation on register */
	bit_no = bit_no % 32;
	size = LONG;
	}

value_of_EA1 (context, &EV, size);

if ((EV >> bit_no) & 1)
	context->sr = context->sr & (~zbit);
else
	context->sr = context->sr | zbit;

switch ((context->inst >> 6) & 0x3) {
	case 0 : 			/* perform a bit test operation */
		  if (mem_reg)
			inc_cyc (context,4);
		  else
			inc_cyc (context,6);
		  break;
	case 1 : 			/* perform a bit change operation */
			  if (mem_reg)
				{	/* byte sized operations on memory */
				if ((EV >> bit_no) & 1){
					value_of_EA1(context,&theResult,size);
					theResult	&= (~(1 << bit_no));
					putEA1 (context, theResult, size);
					}
				else{
					value_of_EA1(context,&theResult,size);
					theResult	|= (1 << bit_no);
					putEA1 (context,theResult, size);
					}
				}
			else
				{	/* long operation on register */
				if ((EV >> bit_no) & 1){
					value_of_EA1(context,&theResult,size);
					theResult	&= (~(1 << bit_no));
					putEA1 (context, theResult, size);
					}
				else{
					value_of_EA1(context,&theResult,size);
					theResult	|= (1 << bit_no);
					putEA1 (context, theResult, size);
					}
				}
			inc_cyc (context,8);
		  break;
	case 2 : /* perform a bit clear operation */
		  if (mem_reg)
				{	/* byte sized operations on memory */
				value_of_EA1(context,&theResult,size);
				theResult	&= (~(1 << bit_no));
				putEA1 (context, theResult, size);
				inc_cyc (context,8);
				}
		  else
				{	/* long operation on register */
				value_of_EA1(context,&theResult,size);
				theResult	&= (~(1 << bit_no));
				putEA1 (context, theResult, size);
				inc_cyc (context,10);
				}
		  break;
	case 3 : /* perform a bit set operation */
			if (mem_reg)
				{	/* byte sized operations on memory */
				value_of_EA1(context,&theResult,size);
				theResult	|= (1 << bit_no);
				putEA1 (context,(long)theResult, size);
				}
			else
				{	/* long operation on register */
				value_of_EA1(context,&theResult,size);
				theResult	|= (1 << bit_no);
				putEA1 (context, theResult, size);
				}
			inc_cyc (context,8);
		  break;
	}

if (mem_reg)
	inc_cyc (context,4);

return SUCCESS;

}




int	TAS(SimContext *context)
{
long	EV;

if (eff_addr (context,(long) BYTE, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);		/* bad effective address format */

value_of_EA1 (context, &EV, BYTE);

/* perform the TAS operation */
/* first set the condition codes according to *context->EA1 */
cc_update (context,N_A, GEN, GEN, ZER, ZER, context->source, context->dest, EV, (long) BYTE, 0);

/* then set the high order bit of the EV byte */
putEA1 (context, EV | 0x80, (long) BYTE);

inc_cyc (context,(context->inst & 0x30) ? 10 : 4);

return SUCCESS;

}

