/***************************** 68000 SIMULATOR ****************************

File Name: CODE8.C
Version: 1.0

The instructions implemented in this file are the program control operations:

		BCC, DBCC, SCC, BRA, BSR, JMP, JSR, RTE, RTR, RTS, NOP


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */





int	BCC(SimContext *context)
{
long	displacement;
int	condition;

displacement = context->inst & 0xff;
if (displacement == 0)
	{
	mem_request (context,&context->pc, (long) WORD, &displacement);
	from_2s_comp (context,displacement, (long) WORD, &displacement);
	}
else if(displacement == 0xff) {
	mem_request (context,&context->pc, (long) LONG, &displacement);
	from_2s_comp (context,displacement, (long) LONG, &displacement);
	}
else
	from_2s_comp (context,displacement, (long) BYTE, &displacement);

condition = (context->inst >> 8) & 0x0f;

/* perform the BCC operation */
if (check_condition (context,condition))
	context->pc = context->old_pc + displacement + 2;
			/* displacement is relative to the end of the context->instructin word */

if (check_condition (context,condition)){
	inc_cyc (context,10);
	}
else{
	inc_cyc (context,((context->inst & 0xff) != 0) ? 12 : 8);
	}

return SUCCESS;

}




int	DBCC(SimContext *context)
{
long	displacement;
int	reg;

mem_request(context,&context->pc, (long) WORD, &displacement);
from_2s_comp (context,displacement, (long) WORD, &displacement);
reg = context->inst & 0x07;

/* perform the DBCC operation */
if (check_condition (context,(context->inst >> 8) & 0x0f))
	inc_cyc (context,12);
else
	{
	unsigned short	r;
	r	= (unsigned short)(context->D[reg] & WORD);
	r	-= 1;
	r	&= WORD;
	context->D[reg]	&= ~WORD;
	context->D[reg] |= r;
/*	context->D[reg] = (context->D[reg] & ~WORD) | ((short)((context->D[reg] & 0xffff) - 1)); */
	if (r == 0xFFFF)
		inc_cyc (context,14);
	else
		{
		inc_cyc (context,10);
		context->pc = context->old_pc + displacement + 2;
			/* displacement is relative to the end of the context->instructin word */
		}
	}

return SUCCESS;

}


int	SCC(SimContext *context)
{
int	condition;

if (eff_addr (context,(long) BYTE, DATA_ALT_ADDR, TRUE))
	return (BAD_INST);		/* bad context->instruction format */

/* perform the SCC operation */
condition = (context->inst >> 8) & 0x0f;
if (check_condition (context,condition))
	putEA1 (context, (long) BYTE, (long) BYTE);
else
	putEA1 (context, (long) 0, (long) BYTE);

if ((context->inst & 0x0030) == 0){
	inc_cyc (context,check_condition (context,condition) ? 6 : 4);
	}
else{
	inc_cyc (context,8);
	}

return SUCCESS;

}



int	BRA(SimContext *context)
{
long	displacement;

displacement = context->inst & 0xff;
if (displacement == 0)
	{
	mem_request (context,&context->pc, (long) WORD, &displacement);
	from_2s_comp (context,displacement, (long) WORD, &displacement);
	}
else if(displacement == 0xff){
	mem_request (context,&context->pc, (long) LONG, &displacement);
	from_2s_comp (context,displacement, (long) LONG, &displacement);
	}
else
	from_2s_comp (context,displacement, (long) BYTE, &displacement);

/* perform the BRA operation */
context->pc = context->old_pc + displacement + 2;
			/* displacement is relative to the end of the context->instructin word */

inc_cyc (context,10);

return SUCCESS;

}



int	BSR(SimContext *context)
{
long	displacement;

displacement = context->inst & 0xff;
if (displacement == 0){
	mem_request (context,&context->pc, (long) WORD, &displacement);
	from_2s_comp (context,displacement, (long) WORD, &displacement);
	}
else if(displacement == 0xFF){
	mem_request (context,&context->pc, (long) LONG, &displacement);
	from_2s_comp (context,displacement, (long) LONG, &displacement);
	}
else{
	from_2s_comp (context,displacement, (long) BYTE, &displacement);
	}

/* perform the Bcontext->sr operation */
context->A[a_reg(context,7)] -= 4;
putLocation (context,context->A[a_reg(context,7)], context->pc, LONG);
context->pc = context->old_pc + displacement + 2;
			/* displacement is relative to the end of the context->instruction word */

inc_cyc (context,18);

return SUCCESS;

}



int	JMP(SimContext *context)
{

if (eff_addr (context,(long) WORD, CONTROL_ADDR, FALSE))
	return (BAD_INST);		/* bad context->instruction format */

/* perform the JMP operation */
context->pc	= context->EA1;

switch (eff_addr_code (context,context->inst, 0)) {
	case 0x02 : inc_cyc (context,8);
	            break;
	case 0x05 : inc_cyc (context,10);
	            break;
	case 0x06 : inc_cyc (context,14);
	            break;
	case 0x07 : inc_cyc (context,10);
	            break;
	case 0x08 : inc_cyc (context,12);
	            break;
	case 0x09 : inc_cyc (context,10);
	            break;
	case 0x0a : inc_cyc (context,14);
	            break;
	default   : break;
	}

return SUCCESS;

}





int	JSR(SimContext *context)
{

if (eff_addr (context,(long) WORD, CONTROL_ADDR, FALSE))
	return (BAD_INST);		/* bad context->instruction format */

/* push the longword address immediately following context->pc on the system stack */
/* then change the context->pc */
context->A[a_reg(context,7)] -= 4;
putLocation (context,context->A[a_reg(context,7)], context->pc, LONG);
context->pc	= context->EA1;

switch (eff_addr_code (context,context->inst, 0)) {
	case 0x02 : inc_cyc (context,16);
	            break;
	case 0x05 : inc_cyc (context,18);
	            break;
	case 0x06 : inc_cyc (context,22);
	            break;
	case 0x07 : inc_cyc (context,18);
	            break;
	case 0x08 : inc_cyc (context,20);
	            break;
	case 0x09 : inc_cyc (context,18);
	            break;
	case 0x0a : inc_cyc (context,22);
	            break;
	default   : break;
	}

return SUCCESS;

}


int	RTE(SimContext *context)
{
long	temp;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

/* set global indication of RTE execution */
context->rte_executing	= TRUE;

mem_request (context,&context->A[8], (long) WORD, &temp);
context->sr = temp & WORD;
mem_request (context,&context->A[8], LONG, &context->pc);
mem_request (context,&context->A[8], (long) WORD, &temp);	/* format code */

/* clear global indication of RTE execution */

context->rte_executing	= FALSE;

inc_cyc (context,20);

return SUCCESS;

}



int	RTR(SimContext *context)
{
long	temp;

mem_request (context,&context->A[a_reg(context,7)], (long) BYTE, &temp);
context->sr = (context->sr & 0xff00) | (temp & 0xff);

mem_request (context,&context->A[a_reg(context,7)], LONG, &context->pc);

inc_cyc (context,20);

return SUCCESS;

}



int	RTS(SimContext *context)
{

mem_request (context,&context->A[a_reg(context,7)], LONG, &context->pc);

inc_cyc (context,16);

return SUCCESS;

}



int	NOP(SimContext *context)
{

inc_cyc (context,4);

return SUCCESS;

}
