
/***************************** 68000 SIMULATOR ****************************

File Name: CODE9.C
Version: 1.0

The context->instructions implemented in this file are the exception processing
related operations:

        CHK, ILLEGAL, RESET, STOP, TRAP, TRAPV


***************************************************************************/


#include <stdio.h>
#include "proto.h"         /* contains global "extern" declarations */




int	CHK(SimContext *context)
{
long	EV;
int		reg;

reg = (context->inst >> 9) & 0x07;

if (eff_addr (context,(long) WORD, DATA_ADDR, TRUE))
	return (BAD_INST);		/* bad context->instruction format */

value_of_EA1 (context, &EV, WORD);

from_2s_comp (context,EV, (long) WORD, &context->source);
context->dest = context->D[reg] & WORD;

cc_update (context,N_A, GEN, UND, UND, UND, context->source, context->D[reg], context->D[reg], WORD, 0);

/* perform the CHK operation */
if ((context->dest < 0) || (context->dest > context->source))
	return(CHK_EXCEPTION);

inc_cyc (context,10);

return SUCCESS;

}



int	ILLEGAL(SimContext *context)
{

return (ILLEGAL_TRAP);

}



int	RESET(SimContext *context)
{

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

/* assert the reset line to reset external devices */

inc_cyc (context,132);

return SUCCESS;

}



int	STOP(SimContext *context)
{
long	temp;
int	tr_on;

inc_cyc (context,4);

mem_request (context,&context->pc, (long) WORD, &temp);

if (context->sr & tbit)
	tr_on = TRUE;
else
	tr_on = FALSE;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

context->sr = temp & WORD;
if (tr_on)
	context->sr = context->sr | tbit;

if (!(context->sr & sbit))
	return (NO_PRIVILEGE);

if (!(context->sr & tbit))
	fprintf (stderr,"Processor has entered stop mode.  Processing halted.");
else
	{
	fprintf (stderr,"Processor has entered stop mode.\n");
	fprintf (stderr,"Processing halted, but resumed due to a context->TRACE exception.");
	}

return (STOP_TRAP);

}



int	TRAP(SimContext *context)
{

return (TRAP_TRAP);

}




int	TRAPV(SimContext *context)
{

if (context->sr & vbit)
	return (TRAPV_TRAP);

inc_cyc (context,4);

return SUCCESS;

}

