#ifndef _sim68k_simulator_externh_
#define _sim68k_simulator_externh_

/***************************** 68000 SIMULATOR ****************************

File Name: EXTERN.H
Version: 1.0

This file contains all extern global variable definitions for the
simulator program.  It is included in all modules other than the module
"sim.c" which contains the main() function.


        BE CAREFUL TO KEEP THESE DECLARATIONS IDENTICAL TO THE GLOBAL
        VARIABLE DECLARATIONS IN "VAR.H"


***************************************************************************/


#include <stdio.h>
#include "def.h"			/* constant declarations */

#ifdef __cplusplus
extern "C" {
#endif

typedef enum{
	pcTraceRecordTypeInstruction,
	pcTraceRecordTypeRead,
	pcTraceRecordTypeWrite
	}PCTraceRecordType;

typedef enum{
	pcTraceRecordSizeByte,
	pcTraceRecordSizeWord,
	pcTraceRecordSizeLong
	}PCTraceRecordSize;

typedef struct PCTraceRecord{
	struct PCTraceRecord*	link;
	PCTraceRecordType		type;
	PCTraceRecordSize		size;
	unsigned long			address;
	unsigned long			data;
	}PCTraceRecord;

typedef enum{
	accessModeAny,
	accessModeRead,
	accessModeWrite
	}AccessMode;

typedef enum{
	accessSizeNone,
	accessSizeByte,
	accessSize16,
	accessSize32
	}AccessSize;

typedef struct SimAccessBreakpoint{
	struct SimAccessBreakpoint*	link;
	char*						name;
	unsigned long				address;
	AccessMode					accessMode;
	AccessSize					accessSize;
	unsigned char				data;
	unsigned char				mask;
	}SimAccessBreakpoint;

typedef enum {
	effectiveAddressTypeDataRegister,
	effectiveAddressTypeAddressRegister,
	effectiveAddressTypeMemoryAddress,
	effectiveAddressTypeImmediate
	} EffectiveAddressType ;

typedef struct SimContext{
	long					D[D_REGS], old_d[D_REGS], A[A_REGS], old_a[A_REGS];
	long					pc, old_pc;
	unsigned long			sr, old_sr;
	unsigned long			SFC;
	unsigned long			DFC;
	unsigned long			VBR;
	char					memory[MEMSIZE];
	int						bpoints;
	char					lbuf[82], *wordptr[20], buffer[40];
	unsigned long			cycles, old_CYCLES;
	unsigned long			brkpt[100];
	char					p1dif;
	unsigned				int port1[4];
	int						errflg;
	int						TRACE, SSTEP, old_TRACE, old_SSTEP, exceptions;
	int						inst;
	long					EA1;
	EffectiveAddressType	typeEA1;
	long					EA2;
	EffectiveAddressType	typeEA2;
	long					source, dest, result;	/* keep these signed */
	int						rte_executing;
	FILE*					inputFileDesc;
	FILE*					outputFileDesc;
	int						dataBreakPoint;
	int						busOrAddressErrorInProgress;
	int						doubleBusFault;
	SimAccessBreakpoint*	writeDataBreakList;
	SimAccessBreakpoint*	writeAnyBreakList;
	SimAccessBreakpoint*	readDataBreakList;
	SimAccessBreakpoint*	readAnyBreakList;
	SimAccessBreakpoint*	rwAnyBreakList;
	PCTraceRecord*			head;
	PCTraceRecord*			tail;
	PCTraceRecord*			pool;
	unsigned long			initialPC;
	unsigned long			initialSP;
	const char*				objFileName;
	}SimContext;

void			putTraceRec(SimContext* context,PCTraceRecord* t);
PCTraceRecord*	getTraceRec(SimContext* context);
PCTraceRecord*	allocTraceRec(SimContext* context);
void			freeTraceRec(SimContext* context,PCTraceRecord* t);
void			flushTrace(SimContext* context);

int	/* Bool */ isWrite8Breakpoint(SimContext *context,unsigned long address,unsigned char data);
int	/* Bool */ isWrite16Breakpoint(SimContext *context,unsigned long address,unsigned short data);
int	/* Bool */ isWrite32Breakpoint(SimContext *context,unsigned long address,unsigned long data);
int	/* Bool */ isRead8Breakpoint(SimContext *context,unsigned long address,unsigned char data);
int	/* Bool */ isRead16Breakpoint(SimContext *context,unsigned long address,unsigned short data);
int	/* Bool */ isRead32Breakpoint(SimContext *context,unsigned long address,unsigned long data);
SimAccessBreakpoint*	newSimAccessBreakpoint(	const char		*name,
												unsigned long	address,
												AccessMode		accessMode,
												unsigned char	data,
												unsigned char	mask
												);
void putAccessBreakpoint(SimContext* context,SimAccessBreakpoint* b);
SimAccessBreakpoint* removeAccessBreakpoint(SimContext* context,const char *name);

#ifdef __cplusplus
}
#endif

#endif /* _externh_ */

