/***************************** 68000 SIMULATOR ****************************

File Name: PROTO.H
Version: 1.0

This file contains function prototype definitions for all functions
	in the program

***************************************************************************/

#ifndef _sim68k_simulator_protoh_
#define _sim68k_simulator_protoh_
#include "extern.h"
#include "simops2.h"

#ifdef __cplusplus
extern "C" {
#endif

int MOVE(SimContext *context);
int MOVEP(SimContext *context);
int MOVEA(SimContext *context);
int MOVE_FR_SR(SimContext *context);
int MOVE_TO_CCR(SimContext *context);
int MOVE_TO_SR(SimContext *context);
int MOVEM(SimContext *context);
int MOVE_USP(SimContext *context);
int MOVEQ(SimContext *context);
int EXG(SimContext *context);
int LEA(SimContext *context);
int PEA(SimContext *context);
int LINK(SimContext *context);
int UNLK(SimContext *context);
int ADD(SimContext *context);
int ADDA(SimContext *context);
int ADDI(SimContext *context);
int ADDQ(SimContext *context);
int ADDX(SimContext *context);
int SUB(SimContext *context);
int SUBA(SimContext *context);
int SUBI(SimContext *context);
int SUBQ(SimContext *context);
int SUBX(SimContext *context);
int DIVS(SimContext *context);
int DIVU(SimContext *context);
int DIVUL(SimContext *context);
int MULS(SimContext *context);
int MULU(SimContext *context);
int MULL(SimContext *context);
int NEG(SimContext *context);
int NEGX(SimContext *context);
int CMP(SimContext *context);
int CMPA(SimContext *context);
int CMPI(SimContext *context);
int CMPM(SimContext *context);
int TST(SimContext *context);
int CLR(SimContext *context);
int EXT(SimContext *context);
int ABCD(SimContext *context);
int SBCD(SimContext *context);
int NBCD(SimContext *context);
int AND(SimContext *context);
int ANDI(SimContext *context);
int ANDI_TO_CCR(SimContext *context);
int ANDI_TO_SR(SimContext *context);
int OR(SimContext *context);
int ORI(SimContext *context);
int ORI_TO_CCR(SimContext *context);
int ORI_TO_SR(SimContext *context);
int EOR(SimContext *context);
int EORI(SimContext *context);
int EORI_TO_CCR(SimContext *context);
int EORI_TO_SR(SimContext *context);
int NOT(SimContext *context);
int SHIFT_ROT(SimContext *context);
int SWAP(SimContext *context);
int BIT_OP(SimContext *context);
int TAS(SimContext *context);
int BCC(SimContext *context);
int DBCC(SimContext *context);
int SCC(SimContext *context);
int BRA(SimContext *context);
int BSR(SimContext *context);
int JMP(SimContext *context);
int JSR(SimContext *context);
int RTE(SimContext *context);
int RTR(SimContext *context);
int RTS(SimContext *context);
int NOP(SimContext *context);
int CHK(SimContext *context);
int ILLEGAL(SimContext *context);
int RESET(SimContext *context);
int STOP(SimContext *context);
int TRAP(SimContext *context);
int TRAPV(SimContext *context);
int parse(SimContext *context,char *str,char * *ptrbuf,int maxcnt);
int iswhite(char c,char *qflag);
int decode_size(SimContext *context,long *result);
int eff_addr(SimContext *context,long size,int mask,int add_times);
int runprog(SimContext *context);
void exec_inst(SimContext *context);
int exception(SimContext *context,int xclass,long loc,int r_w,long pc,int vectorOffset);
int main(int argc,char *argv[]);
int finish(SimContext *context);
int errmess(SimContext *context);
int cmderr(SimContext *context);
int setdis(SimContext *context);
int scrshow(SimContext *context);
int to_2s_comp(SimContext *context,long number,long size,long *result);
int from_2s_comp(SimContext *context,long number,long size,long *result);
int sign_extend(SimContext *context,int number,long size_from,long *result);
void inc_cyc(SimContext *context,int num);
int eff_addr_code(SimContext *context,int inst,int start);
int a_reg(SimContext *context,int reg_num);
int mem_put(SimContext *context,long data,int loc,long size);
int mem_req(SimContext *context,long loc,long size,long *result);
int mem_request(SimContext *context,long *loc,long size,long *result);
#if 0
int put(SimContext *context,long *dest,long source,long size);
int value_of(SimContext *context,long *EA,long *EV,long size);
#else
int putEA2(SimContext* context,long source,long size);
int putEA1(SimContext* context,long source,long size);
int value_of_EA2(SimContext* context,long* EV,long size);
int value_of_EA1(SimContext* context,long* EV,long size);
#if 0
int putSR(SimContext* context,long source,long size);
int putResult(SimContext* context,long source,long size);
int putDataReg(SimContext* context,int regIndex,long source,long size);
int putAddressReg(SimContext* context,int regIndex,long source,long size);
int putLocation(SimContext* context,long address,long source,long size);
int value_of_SR(SimContext* context,long* EV,long size);
int value_of_DataReg(SimContext* context,int regIndex,long* EV,long size);
int value_of_AddressReg(SimContext* context,int regIndex,long* EV,long size);
int value_of_Location(SimContext* context,long address,long* EV,long size);
#else
inline static int putDataReg(SimContext* context,int regIndex,long source,long size){
	context->D[regIndex]	= (source & size) | (context->D[regIndex] & ~size);
	return SUCCESS;
	}
inline static int putAddressReg(SimContext* context,int regIndex,long source,long size){
	context->A[regIndex]	= (source & size) | (context->A[regIndex] & ~size);
	return SUCCESS;
	}
inline static int putSR(SimContext* context,long source,long size){
	context->sr	= (source & size) | (context->sr & ~size);
	return SUCCESS;
	}
inline static int putResult(SimContext* context,long source,long size){
	context->result	= (source & size) | (context->result & ~size);
	return SUCCESS;
	}
inline static int putLocation(SimContext* context,long address,long source,long size){
	return mem_put (context,source, (int) address, size);
	}
inline static int value_of_DataReg(SimContext* context,int regIndex,long* EV,long size){
	*EV	= context->D[regIndex] & size;
	return SUCCESS;
	}
inline static int value_of_AddressReg(SimContext* context,int regIndex,long* EV,long size){
	*EV	= context->A[regIndex] & size;
	return SUCCESS;
	}
inline static int value_of_SR(SimContext* context,long* EV,long size){
	*EV	= context->sr & size;
	return SUCCESS;
	}
inline static int value_of_Location(SimContext* context,long address,long* EV,long size){
	return mem_req (context, address, size, EV);
	}
#endif

#endif
int cc_update(SimContext *context,int x,int n,int z,int v,int c,long source,long dest,long result,long size,int r);
int check_condition(SimContext *context,int condition);

#ifdef __cplusplus
}
#endif

#endif /* _protoh_ */
