
/***************************** 68000 SIMULATOR ****************************

File Name: SIMOPS2.C
Version: 1.0

This file contains various routines for simulator operation

The routines are :

	mfill()

***************************************************************************/



#include <stdio.h>
#include <errno.h>
#include "proto.h"
#include "simops2.h"
#include "oscl/compiler/types.h"

using namespace Oscl;

static char *scan32(char *buff,Unsigned32 *lword);
static char *scan24(char *buff,Unsigned32 *word);
static char *scan16(char *buff,Unsigned16 *word);
static char *scanByte(char *buff,UByte *byte);
static UByte	hexDigitToBinary(UByte c);

/***************************************************************************/

int mfill(const char *filename,SimContext *context)                  /* load memory with contents of s_record file */
{
	FILE 		*fp;
	int			line;
	int			end__of__file;
	int			loc=0;
	UByte 		byte;
	char		s_type;
	const char	*name;
	char 		*bufptr;
	char		*bufend;
	int			len;
	Unsigned32	lw;
	Unsigned16	w;
	UByte		c;

	name	= filename;

	fp = fopen(name, "r");
	if (fp == NULL)
		{                         /* if file cannot be opened, print message */
		int	err	= errno;
		fprintf(stderr,"error:  %d cannot open file %s\n",err, name);
		return FAILURE;
		}

	line = 0;
	context->errflg = FALSE;
	end__of__file = FALSE;
	s_type = 0;

	fgets(context->lbuf, 80, fp);
	len=sscanf (context->lbuf,"S%c",&s_type);
	if (s_type != '0')                     /* if first record is not a 'S0' */
		context->errflg = TRUE;                  /* record then incorrect file format */
	line++;
	if (~context->errflg)
		while (fgets(context->lbuf, 80, fp) != NULL)           /* read file until end */
		{
		bufptr = context->lbuf;
		for (bufend = bufptr; *bufend != '\0'; bufend++);
		line++;
		len = sscanf (context->lbuf, "S%c", &s_type);
		bufptr	+=2;
		bufptr=scanByte(bufptr,&c);
		switch (s_type)
			{
			case '1' :
					bufptr=scan16(bufptr,&w);
					loc	= w;
				   break;
			case '2' :
					bufptr=scan24(bufptr,&lw);
					loc	= lw;
				   break;
			case '3' :
					bufptr=scan32(bufptr,&lw);
					loc	= lw;
				   break;
			case '7' :
			case '8' :
			case '9' : end__of__file = TRUE;
				   break;
			default :  context->errflg = TRUE;
			}
		if (end__of__file) break;
		if (context->errflg) break;
		for (bufend = bufptr; *bufend && *bufend != '\r' && *bufend != '\n'; bufend++);
		bufend	-= 2;	/* ignore that CRC! */
		while(bufptr < bufend){
			bufptr=scanByte(bufptr,&byte);
			if ((loc < 0) || (loc > (MEMSIZE - 1)))
				{
				fprintf(stderr,"\nInvalid Address on line %d...\n",line);
				context->errflg = TRUE;
				break;
				}
			else
				context->memory[loc++] = byte;
			}
		if (context->errflg) break;
		}

	if (context->errflg)                     /* if error reading file, print message */
		{
		fprintf(stderr,"\nInvalid data on line %d of file...",line);
		fprintf (stderr,"\n%d: %s", line, context->lbuf);
	 	fprintf(stderr,"\nRemainder of load aborted...\n");
		}
	fclose(fp);			/* close file specified */
	return SUCCESS;

}

static char *scan32(char *buff,Unsigned32 *lword)
{
Unsigned32	lw;
Unsigned16	w;

	buff	= scan16(buff,&w);
	lw		= w;
	lw		<<= 16;
	buff	= scan16(buff,&w);
	lw		|= w;
	*lword	= lw;

return buff;
}

static char *scan24(char *buff,Unsigned32 *word)
{
Unsigned32	l;
Unsigned16	w;
UByte		c;

	buff	= scan16(buff,&w);
	l		= w;
	l		<<= 8;
	buff	= scanByte(buff,&c);
	l		|= c;
	*word	= l;

return buff;
}

static char *scan16(char *buff,Unsigned16 *word)
{
Unsigned16		w;
UByte			c;
	buff	= scanByte(buff,&c);
	w		= c;
	w		<<= 8;
	buff	= scanByte(buff,&c);
	w		|= c;
	*word	= w;

return buff;
}

static char *scanByte(char *buff,UByte *byte)
{
UByte	c;

	c	= hexDigitToBinary(*buff);
	c	<<= 4;
	buff++;
	c	|= hexDigitToBinary(*buff);
	buff++;
	*byte	= c;

	return buff;
}

static UByte	hexDigitToBinary(UByte c)
{
	c	= (c < 'A')?c-'0':
			((c < 'a')?c-'A'+10:c-'a'+10);

return c;
}


